@@ -43,6 +43,26 @@
 #include "ipa_krb5.h"
 #include "ipa_asn1.h"
 #include "ipa-client-common.h"
+#include "ipa_ldap.h"
+
+
+static int check_sasl_mech(const char *mech)
+{
+    int i;
+    int ret = 1;
+    const char *supported_sasl_mechs[] = {
+        LDAP_SASL_EXTERNAL,
+        LDAP_SASL_GSSAPI,
+        NULL
+    };
+
+    for (i=0; NULL != supported_sasl_mechs[i]; i++) {
+        if (strcmp(mech, supported_sasl_mechs[i]) == 0) {
+            return 0;
+        }
+    }
+    return ret;
+}
 
 static int ldap_sasl_interact(LDAP *ld, unsigned flags, void *priv_data, void *sit)
 {
@@ -135,61 +155,37 @@ int filter_keys(krb5_context krbctx, struct keys_container *keys,
     return n;
 }
 
-static int ipa_ldap_init(LDAP ** ld, const char * scheme, const char * servername, const int  port)
+static int ipa_server_to_uri(const char *servername, const char *mech,
+                             char **ldap_uri)
 {
-	char* url = NULL;
-	int  url_len = snprintf(url,0,"%s://%s:%d",scheme,servername,port) +1;
+    char *url = NULL;
+    int url_len = 0;
+    int port = 389;
 
-	url = (char *)malloc (url_len);
-	if (!url){
-		fprintf(stderr, _("Out of memory \n"));
-		return LDAP_NO_MEMORY;
-	}
-	sprintf(url,"%s://%s:%d",scheme,servername,port);
-	int rc = ldap_initialize(ld, url);
+    url_len = asprintf(&url, "%s%s:%d", SCHEMA_LDAP, servername, port);
 
-	free(url);
-	return rc;
+    if (url_len == -1) {
+        fprintf(stderr, _("Out of memory \n"));
+        return LDAP_NO_MEMORY;
+    }
+    *ldap_uri = url;
+    return 0;
 }
 
-const char *ca_cert_file = "/etc/ipa/ca.crt";
-
-static int ipa_ldap_bind(const char *server_name, krb5_principal bind_princ,
-			 const char *bind_dn, const char *bind_pw, LDAP **_ld)
+static int ipa_ldap_bind(const char *ldap_uri, krb5_principal bind_princ,
+                         const char *bind_dn, const char *bind_pw,
+                         const char *mech, const char *ca_cert_file,
+                         LDAP **_ld)
 {
     char *msg = NULL;
     struct berval bv;
-    int version;
     LDAP *ld;
-    int ssl;
     int ret;
 
     /* TODO: support referrals ? */
-    if (bind_dn) {
-        ret = ldap_set_option(NULL, LDAP_OPT_X_TLS_CACERTFILE, ca_cert_file);
-        if (ret != LDAP_OPT_SUCCESS) {
-            fprintf(stderr, _("Unable to set LDAP_OPT_X_TLS_CERTIFICATE\n"));
-            return ret;
-        }
-
-        ret = ipa_ldap_init(&ld, "ldaps", server_name, 636);
-        if (ret != LDAP_SUCCESS) {
-            fprintf(stderr, _("Unable to init for ldaps(636) connection\n"));
-            return ret;
-        }
-
-        ssl = LDAP_OPT_X_TLS_HARD;;
-        ret = ldap_set_option(ld, LDAP_OPT_X_TLS, &ssl);
-        if (ret != LDAP_OPT_SUCCESS) {
-            fprintf(stderr, _("Unable to set LDAP_OPT_X_TLS\n"));
-            goto done;
-        }
-    } else {
-        ret = ipa_ldap_init(&ld, "ldap", server_name, 389);
-        if (ret != LDAP_SUCCESS) {
-            fprintf(stderr, _("Unable to init for ldap(389) connection\n"));
-            return ret;
-        }
+    ret = ipa_ldap_init(&ld, ldap_uri);
+    if (ret != LDAP_SUCCESS) {
+        return ret;
     }
 
     if (ld == NULL) {
@@ -197,21 +193,10 @@ static int ipa_ldap_bind(const char *server_name, krb5_principal bind_princ,
         return LDAP_OPERATIONS_ERROR;
     }
 
-#ifdef LDAP_OPT_X_SASL_NOCANON
-    /* Don't do DNS canonicalization */
-    ret = ldap_set_option(ld, LDAP_OPT_X_SASL_NOCANON, LDAP_OPT_ON);
-    if (ret != LDAP_SUCCESS) {
-	fprintf(stderr, _("Unable to set LDAP_OPT_X_SASL_NOCANON\n"));
+    ret = ipa_tls_ssl_init(ld, ldap_uri, ca_cert_file);
+    if (ret != LDAP_OPT_SUCCESS) {
         goto done;
     }
-#endif
-
-    version = LDAP_VERSION3;
-    ret = ldap_set_option(ld, LDAP_OPT_PROTOCOL_VERSION, &version);
-    if (ret != LDAP_SUCCESS) {
-	fprintf(stderr, _("Unable to set LDAP_OPT_PROTOCOL_VERSION\n"));
-	goto done;
-    }
 
     if (bind_dn) {
         bv.bv_val = discard_const(bind_pw);
@@ -224,9 +209,15 @@ static int ipa_ldap_bind(const char *server_name, krb5_principal bind_princ,
             goto done;
         }
     } else {
-        ret = ldap_sasl_interactive_bind_s(ld, NULL, "GSSAPI",
-                                           NULL, NULL, LDAP_SASL_QUIET,
-                                           ldap_sasl_interact, bind_princ);
+        if (strcmp(mech, LDAP_SASL_EXTERNAL) == 0) {
+            ret = ldap_sasl_bind_s(ld, NULL, LDAP_SASL_EXTERNAL,
+                                   NULL, NULL, NULL, NULL);
+        } else {
+            ret = ldap_sasl_interactive_bind_s(ld, NULL, LDAP_SASL_GSSAPI,
+                                               NULL, NULL, LDAP_SASL_QUIET,
+                                               ldap_sasl_interact, bind_princ);
+        }
+
         if (ret != LDAP_SUCCESS) {
 #ifdef LDAP_OPT_DIAGNOSTIC_MESSAGE
             ldap_get_option(ld, LDAP_OPT_DIAGNOSTIC_MESSAGE, (void*)&msg);
@@ -267,14 +258,18 @@ static int ipa_ldap_extended_op(LDAP *ld, const char *reqoid,
         return ret;
     }
 
-    /* wait max 10 secs for the answer */
-    tv.tv_sec = 10;
+    /* wait max 100 secs for the answer */
+    tv.tv_sec = 100;
     tv.tv_usec = 0;
     ret = ldap_result(ld, msgid, 1, &tv, &res);
     if (ret == -1) {
         fprintf(stderr, _("Failed to get result: %s\n"), ldap_err2string(ret));
         goto done;
     }
+    else if (res == NULL) {
+        fprintf(stderr, _("Timeout exceeded."));
+        goto done;
+    }
 
     ret = ldap_parse_extended_result(ld, res, &retoid, &retdata, 0);
     if (ret != LDAP_SUCCESS) {
@@ -334,11 +329,13 @@ static BerElement *get_control_data(LDAPControl **list, const char *repoid)
 }
 
 static int ldap_set_keytab(krb5_context krbctx,
-			   const char *servername,
+			   const char *ldap_uri,
 			   const char *principal_name,
 			   krb5_principal princ,
 			   const char *binddn,
 			   const char *bindpw,
+			   const char *mech,
+			   const char *ca_cert_file,
 			   struct keys_container *keys)
 {
 	LDAP *ld = NULL;
@@ -365,7 +362,7 @@ static int ldap_set_keytab(krb5_context krbctx,
 		goto error_out;
 	}
 
-    ret = ipa_ldap_bind(servername, princ, binddn, bindpw, &ld);
+    ret = ipa_ldap_bind(ldap_uri, princ, binddn, bindpw, mech, ca_cert_file, &ld);
     if (ret != LDAP_SUCCESS) {
         fprintf(stderr, _("Failed to bind to server!\n"));
         goto error_out;
@@ -493,9 +490,11 @@ done:
 #define GKREP_SALT_TAG (LBER_CLASS_CONTEXT | LBER_CONSTRUCTED | 1)
 
 static int ldap_get_keytab(krb5_context krbctx, bool generate, char *password,
-                           const char *enctypes, const char *bind_server,
+                           const char *enctypes, const char *ldap_uri,
                            const char *svc_princ, krb5_principal bind_princ,
                            const char *bind_dn, const char *bind_pw,
+                           const char *mech,
+                           const char *ca_cert_file,
                            struct keys_container *keys, int *kvno,
                            char **err_msg)
 {
@@ -525,7 +524,8 @@ static int ldap_get_keytab(krb5_context krbctx, bool generate, char *password,
         goto done;
     }
 
-    ret = ipa_ldap_bind(bind_server, bind_princ, bind_dn, bind_pw, &ld);
+    ret = ipa_ldap_bind(ldap_uri, bind_princ, bind_dn, bind_pw, mech,
+                        ca_cert_file, &ld);
     if (ret != LDAP_SUCCESS) {
         *err_msg = _("Failed to bind to server!\n");
         goto done;
@@ -680,6 +680,9 @@ int main(int argc, const char *argv[])
 	static const char *enctypes_string = NULL;
 	static const char *binddn = NULL;
 	static const char *bindpw = NULL;
+	char *ldap_uri = NULL;
+	static const char *sasl_mech = NULL;
+	static const char *ca_cert_file = NULL;
 	int quiet = 0;
 	int askpass = 0;
 	int permitted_enctypes = 0;
@@ -694,7 +697,8 @@ int main(int argc, const char *argv[])
               _("The principal to get a keytab for (ex: ftp/ftp.example.com@EXAMPLE.COM)"),
               _("Kerberos Service Principal Name") },
             { "keytab", 'k', POPT_ARG_STRING, &keytab, 0,
-              _("File were to store the keytab information"),
+              _("The keytab file to append the new key to (will be "
+                "created if it does not exist)."),
               _("Keytab File Name") },
 	    { "enctypes", 'e', POPT_ARG_STRING, &enctypes_string, 0,
               _("Encryption types to request"),
@@ -708,6 +712,14 @@ int main(int argc, const char *argv[])
               _("LDAP DN"), _("DN to bind as if not using kerberos") },
 	    { "bindpw", 'w', POPT_ARG_STRING, &bindpw, 0,
               _("LDAP password"), _("password to use if not using kerberos") },
+	    { "cacert", 0, POPT_ARG_STRING, &ca_cert_file, 0,
+              _("Path to the IPA CA certificate"), _("IPA CA certificate")},
+	    { "ldapuri", 'H', POPT_ARG_STRING, &ldap_uri, 0,
+              _("LDAP uri to connect to. Mutually exclusive with --server"),
+              _("url")},
+	    { "mech", 'Y', POPT_ARG_STRING, &sasl_mech, 0,
+              _("LDAP SASL bind mechanism if no bindd/bindpw"),
+              _("GSSAPI|EXTERNAL") },
 	    { "retrieve", 'r', POPT_ARG_NONE, &retrieve, 0,
               _("Retrieve current keys without changing them"), NULL },
             POPT_AUTOHELP
@@ -779,7 +791,34 @@ int main(int argc, const char *argv[])
 		exit(10);
 	}
 
-    if (!server) {
+    if (NULL != binddn && NULL != sasl_mech) {
+        fprintf(stderr, _("Cannot specify both SASL mechanism "
+                          "and bind DN simultaneously.\n"));
+        if (!quiet)
+            poptPrintUsage(pc, stderr, 0);
+        exit(2);
+    }
+
+    if (sasl_mech && check_sasl_mech(sasl_mech)) {
+        fprintf(stderr, _("Invalid SASL bind mechanism\n"));
+        if (!quiet)
+            poptPrintUsage(pc, stderr, 0);
+        exit(2);
+    }
+
+    if (!binddn && !sasl_mech) {
+        sasl_mech = LDAP_SASL_GSSAPI;
+    }
+
+    if (server && ldap_uri) {
+        fprintf(stderr, _("Cannot specify server and LDAP uri "
+                          "simultaneously.\n"));
+        if (!quiet)
+            poptPrintUsage(pc, stderr, 0);
+        exit(2);
+    }
+
+    if (!server && !ldap_uri) {
         struct ipa_config *ipacfg = NULL;
 
         ret = read_ipa_config(&ipacfg);
@@ -793,6 +832,16 @@ int main(int argc, const char *argv[])
             exit(2);
         }
     }
+    if (server) {
+        ret = ipa_server_to_uri(server, sasl_mech, &ldap_uri);
+        if (ret) {
+            exit(ret);
+        }
+    }
+
+    if (!ca_cert_file) {
+        ca_cert_file = DEFAULT_CA_CERT_FILE;
+    }
 
     if (askpass && retrieve) {
         fprintf(stderr, _("Incompatible options provided (-r and -P)\n"));
@@ -822,7 +871,7 @@ int main(int argc, const char *argv[])
 		exit(4);
 	}
 
-	if (NULL == bindpw) {
+	if (NULL == bindpw && strcmp(sasl_mech, LDAP_SASL_GSSAPI) == 0) {
 		krberr = krb5_cc_default(krbctx, &ccache);
 		if (krberr) {
 			fprintf(stderr,
@@ -848,7 +897,8 @@ int main(int argc, const char *argv[])
 
     kvno = -1;
     ret = ldap_get_keytab(krbctx, (retrieve == 0), password, enctypes_string,
-                          server, principal, uprinc, binddn, bindpw,
+                          ldap_uri, principal, uprinc, binddn, bindpw,
+                          sasl_mech, ca_cert_file,
                           &keys, &kvno, &err_msg);
     if (ret) {
         if (!quiet && err_msg != NULL) {
@@ -873,7 +923,8 @@ int main(int argc, const char *argv[])
             exit(8);
         }
 
-        kvno = ldap_set_keytab(krbctx, server, principal, uprinc, binddn, bindpw, &keys);
+        kvno = ldap_set_keytab(krbctx, ldap_uri, principal, uprinc, binddn,
+                               bindpw, sasl_mech, ca_cert_file, &keys);
     }
 
     if (kvno == -1) {
