@@ -207,8 +207,10 @@ static int ipapwd_chpwop(Slapi_PBlock *pb, struct ipapwd_krbcfg *krbcfg)
 	char *attrlist[] = {"*", "passwordHistory", NULL };
 	struct ipapwd_data pwdata;
 	int is_krb, is_smb, is_ipant;
-    char *principal = NULL;
+	char *principal = NULL;
 	Slapi_PBlock *chpwop_pb = NULL;
+	Slapi_DN     *target_sdn = NULL;
+	const char   *target_dn = NULL;
 
 	/* Get the ber value of the extended operation */
 	slapi_pblock_get(pb, SLAPI_EXT_OP_REQ_VALUE, &extop_value);
@@ -327,14 +329,32 @@ parse_req_done:
 		}
 	}
 
-	 /* Determine the target DN for this operation */
-	 /* Did they give us a DN ? */
-	if (dn == NULL || *dn == '\0') {
-	 	/* Get the DN from the bind identity on this connection */
-		dn = slapi_ch_strdup(bindDN);
-		LOG_TRACE("Missing userIdentity in request, "
-                          "using the bind DN instead.\n");
+	/* Determine the target DN for this operation */
+	slapi_pblock_get(pb, SLAPI_TARGET_SDN, &target_sdn);
+	if (target_sdn != NULL) {
+		/* If there is a TARGET_DN we are consuming it */
+		slapi_pblock_set(pb, SLAPI_TARGET_SDN, NULL);
+		target_dn = slapi_sdn_get_ndn(target_sdn);
 	}
+	if (target_dn == NULL || *target_dn == '\0') {
+		/* Did they give us a DN ? */
+		if (dn == NULL || *dn == '\0') {
+			/* Get the DN from the bind identity on this connection */
+			dn = slapi_ch_strdup(bindDN);
+			LOG_TRACE("Missing userIdentity in request, "
+				"using the bind DN instead.\n");
+		}
+		LOG_TRACE("extop dn %s (from ber)\n", dn ? dn : "<empty>");
+	} else {
+		/* At this point if SLAPI_TARGET_SDN was set that means
+		 * that a SLAPI_PLUGIN_PRE_EXTOP_FN plugin sets it
+		 * So take this one rather that the raw one that is in the ber
+		 */
+		LOG_TRACE("extop dn %s was translated to %s\n", dn ? dn : "<empty>", target_dn);
+		slapi_ch_free_string(&dn);
+		dn = slapi_ch_strdup(target_dn);
+	}
+	slapi_sdn_free(&target_sdn);
 
 	 if (slapi_pblock_set( pb, SLAPI_ORIGINAL_TARGET, dn )) {
 		LOG_FATAL("slapi_pblock_set failed!\n");
@@ -374,21 +394,30 @@ parse_req_done:
 
     if (dn) {
         Slapi_DN *bind_sdn;
-        Slapi_DN *target_sdn;
 
         /* if the user changing the password is self, we must request the
          * old password and verify it matches the current one before
          * proceeding with the password change */
-        bind_sdn = slapi_sdn_new_dn_byref(bindDN);
-        target_sdn = slapi_sdn_new_dn_byref(dn);
-        if (!bind_sdn || !target_sdn) {
-            LOG_OOM();
-            rc = LDAP_OPERATIONS_ERROR;
-            goto free_and_return;
-        }
+        bind_sdn = slapi_sdn_new_dn_byval(bindDN);
+        target_sdn = slapi_sdn_new_dn_byval(dn);
+
+        rc = (!bind_sdn || !target_sdn) ? LDAP_OPERATIONS_ERROR : 0;
+
         /* this one will normalize and compare, so difference in case will be
          * correctly handled */
         ret = slapi_sdn_compare(bind_sdn, target_sdn);
+
+        slapi_sdn_free(&bind_sdn);
+        slapi_sdn_free(&target_sdn);
+
+        /* rc should always be 0 (else slapi_sdn_new_dn_byval should have sigsev)
+         * but if we end in rc==LDAP_OPERATIONS_ERROR be sure to stop here
+         * because ret is not significant */
+        if (rc != 0) {
+            LOG_OOM();
+            goto free_and_return;
+        }
+
         if (ret == 0) {
             Slapi_Value *cpw[2] = { NULL, NULL };
             Slapi_Value *pw;
@@ -566,7 +595,8 @@ parse_req_done:
     } else {
         principal = slapi_ch_smprintf("root/admin@%s", krbcfg->realm);
     }
-    ipapwd_set_extradata(pwdata.dn, principal, pwdata.timeNow);
+    if (principal)
+        ipapwd_set_extradata(pwdata.dn, principal, pwdata.timeNow);
 
 	/* Free anything that we allocated above */
 free_and_return:
