@@ -32,7 +32,17 @@ define([
         './entity'],
     function(on, IPA, $, phases, reg) {
 
-var exp = IPA.group = {};
+var exp = IPA.group = {
+    search_facet_group: {
+        name: 'search',
+        label: '@i18n:objects.group.group_categories',
+        facets: {
+            search_group: 'group_search',
+            search_hostgroup: 'hostgroup_search',
+            search_netgroup: 'netgroup_search'
+        }
+    }
+};
 
 var make_spec = function() {
 return {
@@ -40,6 +50,10 @@ return {
     facets: [
         {
             $type: 'search',
+            tab_label: '@i18n:objects.group.user_groups',
+            facet_groups: [IPA.group.search_facet_group],
+            tabs_in_sidebar: true,
+            disable_facet_tabs: false,
             columns: [
                 'cn',
                 'gidnumber',
@@ -62,7 +76,7 @@ return {
                             name: 'external',
                             param: 'objectclass',
                             label: '@i18n:objects.group.type',
-                            default_label: '@i18n:objects.group.normal',
+                            default_label: '@i18n:objects.group.nonposix',
                             value_map: {
                                 ipaexternalgroup: '@i18n:objects.group.external',
                                 posixgroup: '@i18n:objects.group.posix'
@@ -108,11 +122,21 @@ return {
                     name: 'uid',
                     primary_key: true
                 }
-            ]
+            ],
+            add_title: '@i18n:objects.group.add_users',
+            remove_title: '@i18n:objects.group.remove_users'
         },
         {
             $type: 'association',
-            name: 'member_group'
+            name: 'member_group',
+            add_title: '@i18n:objects.group.add_groups',
+            remove_title: '@i18n:objects.group.remove_groups'
+        },
+        {
+            $type: 'association',
+            name: 'member_service',
+            add_title: '@i18n:objects.group.add_services',
+            remove_title: '@i18n:objects.group.remove_services'
         },
         {
             $type: 'attribute',
@@ -130,35 +154,46 @@ return {
         {
             $type: 'association',
             name: 'memberof_group',
-            associator: IPA.serial_associator
+            associator: IPA.serial_associator,
+            add_title: '@i18n:objects.group.add_into_groups',
+            remove_title: '@i18n:objects.group.remove_from_groups'
         },
         {
             $type: 'association',
             name: 'memberof_netgroup',
-            associator: IPA.serial_associator
+            associator: IPA.serial_associator,
+            add_title: '@i18n:objects.group.add_into_netgroups',
+            remove_title: '@i18n:objects.group.remove_from_netgroups'
         },
         {
             $type: 'association',
             name: 'memberof_role',
-            associator: IPA.serial_associator
+            associator: IPA.serial_associator,
+            add_title: '@i18n:objects.group.add_into_roles',
+            remove_title: '@i18n:objects.group.remove_from_roles'
         },
         {
             $type: 'association',
             name: 'memberof_hbacrule',
             associator: IPA.serial_associator,
             add_method: 'add_user',
-            remove_method: 'remove_user'
+            add_title: '@i18n:objects.group.add_into_hbac',
+            remove_method: 'remove_user',
+            remove_title: '@i18n:objects.group.remove_from_hbac'
         },
         {
             $type: 'association',
             name: 'memberof_sudorule',
             associator: IPA.serial_associator,
             add_method: 'add_user',
-            remove_method: 'remove_user'
+            add_title: '@i18n:objects.group.add_into_sudo',
+            remove_method: 'remove_user',
+            remove_title: '@i18n:objects.group.remove_from_sudo'
         }
     ],
     standard_association_facets: true,
     adder_dialog: {
+        title: '@i18n:objects.group.add',
         $factory: IPA.group_adder_dialog,
         fields: [
             'cn',
@@ -174,8 +209,8 @@ return {
                 default_value: 'posix',
                 options: [
                     {
-                        value: 'normal',
-                        label: '@i18n:objects.group.normal'
+                        value: 'nonposix',
+                        label: '@i18n:objects.group.nonposix'
                     },
                     {
                         value: 'external',
@@ -189,6 +224,9 @@ return {
             },
             'gidnumber'
         ]
+    },
+    deleter_dialog: {
+        title: '@i18n:objects.group.remove'
     }
 };};
 
@@ -224,10 +262,8 @@ IPA.group_adder_dialog = function(spec) {
         var type_field = that.fields.get_field('type');
         var type = type_field.save()[0];
 
-        if (type === 'normal') {
-            command.set_option('nonposix', true);
-        } else if (type === 'external') {
-            command.set_option('external', true);
+        if (type === 'nonposix' || type === 'external') {
+            command.set_option(type, true);
         }
 
         return command;
