@@ -93,7 +93,8 @@ return {
                             name: 'krbprincipalname',
                             item_name: 'principal',
                             child_spec: {
-                                $type: 'krb_principal'
+                                $type: 'non_editable_row',
+                                data_name: 'krb-principal'
                             }
                         },
                         {
@@ -123,11 +124,11 @@ return {
                             add_field_label: '@i18n:authtype.auth_indicator',
                             options: [
                                 {
-                                    label: '@i18n:authtype.otp',
+                                    label: 'otp',
                                     value: 'otp'
                                 },
                                 {
-                                    label: '@i18n:authtype.type_radius',
+                                    label: 'radius',
                                     value: 'radius'
                                 }
                             ],
@@ -197,10 +198,11 @@ return {
                             $type: 'association_table',
                             id: 'host_ipaallowedtoperform_read_keys_user',
                             name: 'ipaallowedtoperform_read_keys_user',
+                            flags: ['w_if_no_aci'],
                             add_method: 'allow_retrieve_keytab',
                             remove_method: 'disallow_retrieve_keytab',
-                            add_title: '@i18n:keytab.add_retrive',
-                            remove_title: '@i18n:keytab.remove_retrieve',
+                            add_title: '@i18n:keytab.add_users_retrieve',
+                            remove_title: '@i18n:keytab.remove_users_retrieve',
                             columns: [
                                 {
                                     name: 'ipaallowedtoperform_read_keys_user',
@@ -213,10 +215,11 @@ return {
                             $type: 'association_table',
                             id: 'host_ipaallowedtoperform_read_keys_group',
                             name: 'ipaallowedtoperform_read_keys_group',
+                            flags: ['w_if_no_aci'],
                             add_method: 'allow_retrieve_keytab',
                             remove_method: 'disallow_retrieve_keytab',
-                            add_title: '@i18n:keytab.add_retrive',
-                            remove_title: '@i18n:keytab.remove_retrieve',
+                            add_title: '@i18n:keytab.add_groups_retrieve',
+                            remove_title: '@i18n:keytab.remove_groups_retrieve',
                             columns: [
                                 {
                                     name: 'ipaallowedtoperform_read_keys_group',
@@ -229,10 +232,11 @@ return {
                             $type: 'association_table',
                             id: 'service_ipaallowedtoperform_read_keys_host',
                             name: 'ipaallowedtoperform_read_keys_host',
+                            flags: ['w_if_no_aci'],
                             add_method: 'allow_retrieve_keytab',
                             remove_method: 'disallow_retrieve_keytab',
-                            add_title: '@i18n:keytab.add_retrive',
-                            remove_title: '@i18n:keytab.remove_retrieve',
+                            add_title: '@i18n:keytab.add_hosts_retrieve',
+                            remove_title: '@i18n:keytab.remove_hosts_retrieve',
                             columns: [
                                 {
                                     name: 'ipaallowedtoperform_read_keys_host',
@@ -245,10 +249,11 @@ return {
                             $type: 'association_table',
                             id: 'service_ipaallowedtoperform_read_keys_hostgroup',
                             name: 'ipaallowedtoperform_read_keys_hostgroup',
+                            flags: ['w_if_no_aci'],
                             add_method: 'allow_retrieve_keytab',
                             remove_method: 'disallow_retrieve_keytab',
-                            add_title: '@i18n:keytab.add_retrive',
-                            remove_title: '@i18n:keytab.remove_retrieve',
+                            add_title: '@i18n:keytab.add_hostgroups_retrieve',
+                            remove_title: '@i18n:keytab.remove_hostgroups_retrieve',
                             columns: [
                                 {
                                     name: 'ipaallowedtoperform_read_keys_hostgroup',
@@ -268,10 +273,11 @@ return {
                             $type: 'association_table',
                             id: 'host_ipaallowedtoperform_write_keys_user',
                             name: 'ipaallowedtoperform_write_keys_user',
+                            flags: ['w_if_no_aci'],
                             add_method: 'allow_create_keytab',
                             remove_method: 'disallow_create_keytab',
-                            add_title: '@i18n:keytab.add_create',
-                            remove_title: '@i18n:keytab.remove_create',
+                            add_title: '@i18n:keytab.add_users_create',
+                            remove_title: '@i18n:keytab.remove_users_create',
                             columns: [
                                 {
                                     name: 'ipaallowedtoperform_write_keys_user',
@@ -284,10 +290,11 @@ return {
                             $type: 'association_table',
                             id: 'host_ipaallowedtoperform_write_keys_group',
                             name: 'ipaallowedtoperform_write_keys_group',
+                            flags: ['w_if_no_aci'],
                             add_method: 'allow_create_keytab',
                             remove_method: 'disallow_create_keytab',
-                            add_title: '@i18n:keytab.add_create',
-                            remove_title: '@i18n:keytab.remove_create',
+                            add_title: '@i18n:keytab.add_groups_create',
+                            remove_title: '@i18n:keytab.remove_groups_create',
                             columns: [
                                 {
                                     name: 'ipaallowedtoperform_write_keys_group',
@@ -300,10 +307,11 @@ return {
                             $type: 'association_table',
                             id: 'service_ipaallowedtoperform_write_keys_host',
                             name: 'ipaallowedtoperform_write_keys_host',
+                            flags: ['w_if_no_aci'],
                             add_method: 'allow_create_keytab',
                             remove_method: 'disallow_create_keytab',
-                            add_title: '@i18n:keytab.add_create',
-                            remove_title: '@i18n:keytab.remove_create',
+                            add_title: '@i18n:keytab.add_hosts_create',
+                            remove_title: '@i18n:keytab.remove_hosts_create',
                             columns: [
                                 {
                                     name: 'ipaallowedtoperform_write_keys_host',
@@ -316,10 +324,11 @@ return {
                             $type: 'association_table',
                             id: 'service_ipaallowedtoperform_write_keys_hostgroup',
                             name: 'ipaallowedtoperform_write_keys_hostgroup',
+                            flags: ['w_if_no_aci'],
                             add_method: 'allow_create_keytab',
                             remove_method: 'disallow_create_keytab',
-                            add_title: '@i18n:keytab.add_create',
-                            remove_title: '@i18n:keytab.remove_create',
+                            add_title: '@i18n:keytab.add_hostgroups_create',
+                            remove_title: '@i18n:keytab.remove_hostgroups_create',
                             columns: [
                                 {
                                     name: 'ipaallowedtoperform_write_keys_hostgroup',
@@ -365,7 +374,10 @@ return {
                     enable_cond: ['userpassword_w'],
                     show_cond: ['has_password']
                 },
-                'cert_request'
+                {
+                    $type: 'cert_request',
+                    title: '@i18n:objects.cert.issue_for_host'
+                }
             ],
             header_actions: ['automember_rebuild', 'unprovision', 'set_otp', 'reset_otp',
                 'request_cert'],
@@ -395,40 +407,53 @@ return {
             $type: 'association',
             name: 'managedby_host',
             add_method: 'add_managedby',
-            remove_method: 'remove_managedby'
+            add_title: '@i18n:objects.host.add_hosts_managing',
+            remove_method: 'remove_managedby',
+            remove_title: '@i18n:objects.host.remove_hosts_managing'
         },
         {
             $type: 'association',
             name: 'memberof_hostgroup',
-            associator: IPA.serial_associator
+            associator: IPA.serial_associator,
+            add_title: '@i18n:objects.host.add_into_groups',
+            remove_title: '@i18n:objects.host.remove_from_groups'
         },
         {
             $type: 'association',
             name: 'memberof_netgroup',
-            associator: IPA.serial_associator
+            associator: IPA.serial_associator,
+            add_title: '@i18n:objects.host.add_into_netgroups',
+            remove_title: '@i18n:objects.host.remove_from_netgroups'
         },
         {
             $type: 'association',
             name: 'memberof_role',
-            associator: IPA.serial_associator
+            associator: IPA.serial_associator,
+            add_title: '@i18n:objects.host.add_into_roles',
+            remove_title: '@i18n:objects.host.remove_from_roles'
         },
         {
             $type: 'association',
             name: 'memberof_hbacrule',
             associator: IPA.serial_associator,
             add_method: 'add_host',
-            remove_method: 'remove_host'
+            add_title: '@i18n:objects.host.add_into_hbac',
+            remove_method: 'remove_host',
+            remove_title: '@i18n:objects.host.remove_from_hbac'
         },
         {
             $type: 'association',
             name: 'memberof_sudorule',
             associator: IPA.serial_associator,
             add_method: 'add_host',
-            remove_method: 'remove_host'
+            add_title: '@i18n:objects.host.add_into_sudo',
+            remove_method: 'remove_host',
+            remove_title: '@i18n:objects.host.remove_from_sudo'
         }
     ],
     standard_association_facets: true,
     adder_dialog: {
+        title: '@i18n:objects.host.add',
         $factory: IPA.host_adder_dialog,
         height: 300,
         sections: [
@@ -468,6 +493,7 @@ return {
         ]
     },
     deleter_dialog: {
+        title: '@i18n:objects.host.remove',
         $factory: IPA.host_deleter_dialog
     }
 };};
@@ -494,6 +520,7 @@ IPA.host.details_facet = function(spec, no_init) {
             retry: false,
             options: {
                 host: [ pkey ],
+                sizelimit: 0,
                 all: true
             }
         });
@@ -508,6 +535,12 @@ IPA.host.details_facet = function(spec, no_init) {
         return that.entity.name+'_show_'+that.get_pkey();
     };
 
+    that.update_on_success = function(data, text_status, xhr) {
+        that.on_update.notify();
+        that.nofify_update_success();
+        that.refresh();
+    };
+
     if (!no_init) that.init_details_facet();
 
     return that;
@@ -904,8 +937,6 @@ IPA.host_unprovision_dialog = function(spec) {
     var that = IPA.dialog(spec);
     that.facet = spec.facet;
 
-    that.title = that.title.replace('${entity}', that.entity.metadata.label_singular);
-
     that.create_content = function() {
         that.container.append(text.get('@i18n:objects.host.unprovision_confirmation'));
     };
