@@ -22,6 +22,7 @@
 define([
         'dojo/on',
         './ipa',
+        './builder',
         './jquery',
         './menu',
         './phases',
@@ -32,7 +33,8 @@ define([
         './facet',
         './search',
         './entity'],
-            function(on, IPA, $, menu, phases, reg, rpc, text, mod_details, mod_facet) {
+            function(on, IPA, builder, $, menu, phases, reg, rpc, text,
+                                                    mod_details, mod_facet) {
 /**
  * ID Views module
  * @class
@@ -97,6 +99,11 @@ return {
                     name: 'details',
                     fields: [
                         'cn',
+                        {
+                            name: 'ipadomainresolutionorder',
+                            flags: ['w_if_no_aci'],
+                            tooltip: '@mc-opt:idview_mod:ipadomainresolutionorder:doc'
+                        },
                         {
                             $type: 'textarea',
                             name: 'description'
@@ -201,6 +208,7 @@ return {
     ],
 
     adder_dialog: {
+        title: '@i18n:objects.idview.add',
         fields: [
             'cn',
             {
@@ -208,6 +216,9 @@ return {
                 name: 'description'
             }
         ]
+    },
+    deleter_dialog: {
+        title: '@i18n:objects.idview.remove'
     }
 };};
 
@@ -223,11 +234,18 @@ return {
             source_facet: 'details',
             dest_entity: 'idview',
             dest_facet: 'idoverrideuser'
+        },
+        {
+            $factory: IPA.cert.cert_update_policy,
+            source_facet: 'details',
+            dest_entity: 'cert',
+            dest_facet: 'search'
         }
     ],
     containing_entity: 'idview',
     facets: [
         {
+            $factory: idviews.id_override_user_details_facet,
             $type: 'details',
             disable_breadcrumb: false,
             containing_facet: 'idoverrideuser',
@@ -261,6 +279,11 @@ return {
                             $type: 'sshkeys',
                             name: 'ipasshpubkey',
                             label: '@i18n:objects.sshkeystore.keys'
+                        },
+                        {
+                            $type: 'idviews_certs',
+                            name: 'usercertificate',
+                            label: '@i18n:objects.cert.certificates'
                         }
                     ]
                 }
@@ -269,6 +292,7 @@ return {
     ],
 
     adder_dialog: {
+        title: '@i18n:objects.idview.add_user',
         policies: [
             { $factory: idviews.idoverride_adder_policy }
         ],
@@ -294,6 +318,14 @@ return {
             'gecos',
             'uidnumber',
             'gidnumber',
+            {
+                $type: 'cert_textarea',
+                name: 'usercertificate'
+            },
+            {
+                $type: 'sshkey',
+                name: 'ipasshpubkey'
+            },
             'loginshell',
             'homedirectory',
             {
@@ -301,6 +333,9 @@ return {
                 name: 'description'
             }
         ]
+    },
+    deleter_dialog: {
+        title: '@i18n:objects.idview.remove_users'
     }
 };};
 
@@ -353,6 +388,7 @@ return {
     ],
 
     adder_dialog: {
+        title: '@i18n:objects.idview.add_group',
         policies: [
             { $factory: idviews.idoverride_adder_policy }
         ],
@@ -381,10 +417,178 @@ return {
                 name: 'description'
             }
         ]
+    },
+    deleter_dialog: {
+        title: '@i18n:objects.idview.remove_groups'
     }
 };};
 
 
+/**
+ * Facet for User ID override, uses batch command to fetch certificates.
+ *
+ * @class
+ * @extends IPA.details_facet
+ */
+idviews.id_override_user_details_facet = function(spec) {
+
+    spec = spec || {};
+
+    var that = IPA.details_facet(spec);
+
+    that.certificate_updated = IPA.observer();
+
+    that.create_refresh_command = function() {
+
+        var user_command = that.details_facet_create_refresh_command();
+
+        var batch = rpc.batch_command({
+            name: that.entity.name + "_details_refresh"
+        });
+
+        batch.add_command(user_command);
+
+        var pkey = that.get_pkey();
+
+        var certs = rpc.command({
+            entity: 'cert',
+            method: 'find',
+            retry: false,
+            options: {
+                idoverrideuser: [ pkey ],
+                sizelimit: 0,
+                all: true
+            }
+        });
+
+        batch.add_command(certs);
+
+        return batch;
+    };
+
+    that.update_on_success = function(data, text_status, xhr) {
+        that.on_update.notify();
+        that.nofify_update_success();
+        that.refresh();
+    };
+
+    return that;
+};
+
+
+idviews.aduser_idoverrideuser_pre_op = function(spec, context) {
+    spec = spec || [];
+
+    if (!IPA.is_aduser_selfservice) return spec;
+
+    var facet = spec.facets[0];
+    facet.label = '@i18n:objects.idoverrideuser.profile';
+    facet.actions = [];
+    facet.header_actions = [];
+    facet.disable_breadcrumb = true;
+
+    return spec;
+};
+
+/**
+ * @extends IPA.cert.certs_widget
+ */
+idviews.idviews_certs_widget = function(spec) {
+
+    spec = spec || {};
+    spec.child_spec = {
+        $factory: idviews.idviews_cert_widget,
+        css_class: 'certificate-widget',
+        facet: spec.facet
+    };
+
+    var that = IPA.cert.certs_widget(spec);
+
+    /* Adds two args to add command - special nested entities. */
+    that.create_add_args = function() {
+        return that.facet.get_pkeys();
+    };
+
+    /* Adds two args to remove command - special nested entities. */
+    that.create_remove_args = function() {
+        return that.facet.get_pkeys();
+    };
+
+    return that;
+};
+
+/**
+ * This widget uses cert_find instead of cert_show, because cert_show does not
+ * support nested entities.
+ *
+ * @extends IPA.cert.cert_widget
+ */
+idviews.idviews_cert_widget = function(spec) {
+
+    spec = spec || {};
+
+    var that = IPA.cert.cert_widget(spec);
+
+    that.adapter = builder.build('adapter', spec.adapter || 'object_adapter', {});
+
+    that.fetch_certificate_data = function(cert) {
+        var result = {};
+        var adapter = that.adapter;
+
+        if (!cert) return;
+
+        var command = rpc.command({
+            entity: 'cert',
+            method: 'find',
+            options: {
+                certificate: cert,
+                all: true
+            },
+            hide_activity_icon: true,
+            on_success: function(data) {
+                var normalized_data = adapter.load(data);
+                that.certificate = $.extend(normalized_data[0], {});
+                that.update_displayed_data();
+                that.spinner.emit('hide-spinner');
+            },
+            on_error: function() {
+                that.update_displayed_data();
+                that.spinner.emit('hide-spinner');
+            }
+        }).execute();
+    };
+
+    that.update = function(values) {
+        that.spinner.emit('display-spinner');
+
+        var certificate = values[0];
+
+        that.fetch_certificate_data(certificate);
+    };
+
+    that.save = function() {
+        if (!that.certificate) return '';
+        return that.certificate.certificate;
+    };
+
+    return that;
+};
+
+idviews.cert_textarea_widget = function(spec) {
+    spec = spec || {};
+
+    var that = IPA.textarea_widget(spec);
+
+    that.save = function() {
+        var value = that.input.val();
+        var blob = IPA.cert.get_base64(value);
+
+        return [blob];
+    };
+
+    return that;
+};
+
 /**
  * Facet for hosts which have current id view applied on
  *
@@ -514,10 +718,8 @@ idviews.apply_action = function(spec) {
 
         var pkey = facet.get_pkey();
         var other_entity = reg.entity.get(that.other_entity);
-        var other_entity_label = other_entity.metadata.label;
         var exclude = that.get_exclude(facet);
         var title = text.get(that.dialog_title);
-        title = title.replace('${entity}', other_entity_label);
         title = title.replace('${primary_key}', pkey);
 
         var dialog = IPA.association_adder_dialog({
@@ -779,8 +981,14 @@ idviews.register = function() {
     var e = reg.entity;
     var f = reg.facet;
     var a = reg.action;
+    var w = reg.widget;
+
     e.register({type: 'idview', spec: idviews.spec});
-    e.register({type: 'idoverrideuser', spec: idviews.idoverrideuser_spec});
+    e.register({
+        type: 'idoverrideuser',
+        spec: idviews.idoverrideuser_spec,
+        pre_ops: [idviews.aduser_idoverrideuser_pre_op]
+    });
     e.register({type: 'idoverridegroup', spec: idviews.idoverridegroup_spec});
     f.copy('attribute', 'idview_appliedtohosts', {
         factory: idviews.appliedtohosts_facet
@@ -790,6 +998,9 @@ idviews.register = function() {
     a.register('idview_unapply', idviews.unapply_action);
     a.register('idview_unapply_host', idviews.unapply_host_action);
     a.register('idview_unapply_hostgroups', idviews.unapply_hostgroups_action);
+
+    w.register('idviews_certs', idviews.idviews_certs_widget);
+    w.register('cert_textarea', idviews.cert_textarea_widget);
 };
 
 phases.on('registration', idviews.register);
