@@ -25,13 +25,15 @@
 define([
     'dojo/_base/lang',
     'dojo/Deferred',
+    'dojo/on',
+    'dojo/topic',
     './auth',
     './ipa',
     './text',
     './util',
     'exports'
    ],
-   function(lang, Deferred, auth, IPA, text, util, rpc /*exports*/) {
+   function(lang, Deferred, on, topic, auth, IPA, text, util, rpc /*exports*/) {
 
 /**
  * Call an IPA command over JSON-RPC.
@@ -69,6 +71,12 @@ rpc.command = function(spec) {
     /** @property {Object} options Option map */
     that.options = $.extend({}, spec.options || {});
 
+    /**
+     * @property {Array} suppress_warnings array of message codes which
+     * are suppressed
+     */
+    that.suppress_warnings = spec.suppress_warnings || [];
+
     /**
      * Success handler
      * @property {Function}
@@ -99,9 +107,10 @@ rpc.command = function(spec) {
     /**
      * Allow turning off the activity icon.
      *
-     * @property {Boolean} show=true
+     * @property {Boolean} notify_globally=true
      */
-    that.hide_activity_icon = spec.hide_activity_icon || false;
+    that.notify_globally = spec.notify_globally === undefined ? true :
+        spec.notify_globally;
 
     /**
      * Allow set function which will be called when the activity of the command
@@ -109,7 +118,7 @@ rpc.command = function(spec) {
      *
      * @property {Function}
      */
-    that.notify_activity_start = spec.notify_activity_start || null;
+    that.start_handler = spec.start_handler || null;
 
     /**
      * Allow set function which will be called when the activity of the command
@@ -117,14 +126,15 @@ rpc.command = function(spec) {
      *
      * @property {Function}
      */
-    that.notify_activity_end = spec.notify_activity_end || null;
+    that.end_handler = spec.end_handler || null;
 
     /** @property {string} error_message Default error message */
     that.error_message = text.get(spec.error_message || '@i18n:dialogs.batch_error_message', 'Some operations failed.');
 
     /** @property {ordered_map.<number,string>} error_messages Error messages map */
     that.error_messages = $.ordered_map({
-        911: 'Missing HTTP referer. You have to configure your browser to send HTTP referer header.'
+        911: 'Missing HTTP referer. You have to configure your browser to send HTTP referer header.',
+        404: 'Cannot connect to the server, please check API accesibility (certificate, API, proxy, etc.)'
     });
 
     /**
@@ -215,6 +225,7 @@ rpc.command = function(spec) {
 
         for (var i=0,l=msgs.length; i<l; i++) {
             var msg = lang.clone(msgs[i]);
+            if (that.suppress_warnings.indexOf(msg.code) > -1) continue;
             // escape and reformat message
             msg.message = util.beautify_message(msg.message);
             IPA.notify(msg.message, msg.type);
@@ -222,11 +233,11 @@ rpc.command = function(spec) {
     };
 
     that.handle_notify_execution_end = function() {
-        if (that.hide_activity_icon) {
-            if (that.notify_activity_end) that.notify_activity_end();
+        if (that.notify_globally) {
+            topic.publish('rpc-end');
         }
         else {
-            IPA.hide_activity_icon();
+            that.emit('end');
         }
     };
 
@@ -284,7 +295,7 @@ rpc.command = function(spec) {
 
             var self = this;
             function proceed() {
-                // error_handler() calls IPA.hide_activity_icon()
+                // error_handler() publishes 'rpc-end'
                 error_handler.call(self, xhr, text_status, error_thrown);
             }
 
@@ -317,6 +328,12 @@ rpc.command = function(spec) {
             if (xhr.status === 401) {
                 error_handler_auth(xhr, text_status, error_thrown);
                 return;
+            } else if (xhr.status === 404) {
+                error_thrown = {
+                    code: xhr.status,
+                    name: xhr.responseText || text.get('@i18n:errors.http_error',
+                                                    'HTTP Error')+' '+xhr.status
+                };
             } else if (!error_thrown) {
                 error_thrown = {
                     name: xhr.responseText || text.get('@i18n:errors.unknown_error', 'Unknown Error'),
@@ -368,21 +385,23 @@ rpc.command = function(spec) {
         function success_handler(data, text_status, xhr) {
 
             if (!data) {
-                // error_handler() calls IPA.hide_activity_icon()
+                // error_handler() publishes 'rpc-end'
                 error_handler.call(this, xhr, text_status, /* error_thrown */ {
                     name: text.get('@i18n:errors.http_error', 'HTTP Error')+' '+xhr.status,
                     url: this.url,
-                    message: data ? xhr.statusText : text.get('@i18n:errors.no_response', 'No response')
+                    message: xhr ? xhr.statusText :
+                            text.get('@i18n:errors.no_response', 'No response')
                 });
 
             } else if (IPA.version && data.version && IPA.version !== data.version) {
                 window.location.reload();
 
-            } else if (IPA.principal && data.principal && IPA.principal !== data.principal) {
+            } else if (IPA.principal && data.principal &&
+                IPA.principal.toLowerCase() !== data.principal.toLowerCase()) {
                 window.location.reload();
 
             } else if (data.error) {
-                // error_handler() calls IPA.hide_activity_icon()
+                // error_handler() publishes 'rpc-end'
                 error_handler.call(this, xhr, text_status,  /* error_thrown */ {
                     name: text.get('@i18n:errors.ipa_error', 'IPA Error') + ' ' +
                           data.error.code + ': ' + data.error.name,
@@ -411,14 +430,24 @@ rpc.command = function(spec) {
 
                     dialog.on_ok = function() {
                         dialog.close();
-                        if (that.on_success) that.on_success.call(ajax, data, text_status, xhr);
+                        that.emit('success', {
+                            that: ajax,
+                            data: data,
+                            text_status: text_status,
+                            xhr: xhr
+                        });
                     };
 
                     dialog.open();
 
                 } else {
                     //custom success handling, maintaining AJAX call's context
-                    if (that.on_success) that.on_success.call(this, data, text_status, xhr);
+                    that.emit('success', {
+                        that: this,
+                        data: data,
+                        text_status: text_status,
+                        xhr: xhr
+                    });
                 }
                 that.process_warnings(data.result);
                 deferred.resolve({
@@ -445,11 +474,11 @@ rpc.command = function(spec) {
             error: error_handler_login
         };
 
-        if (that.hide_activity_icon) {
-            if (that.notify_activity_start) that.notify_activity_start();
+        if (that.notify_globally) {
+            topic.publish('rpc-start');
         }
         else {
-            IPA.display_activity_icon();
+            that.emit('start');
         }
 
         $.ajax(that.request);
@@ -531,6 +560,26 @@ rpc.command = function(spec) {
         return string;
     };
 
+    that.register_handlers = function() {
+        on(that, 'start', function() {
+            if (that.start_handler) that.start_handler();
+        });
+
+        on(that, 'end', function() {
+            if (that.end_handler) that.end_handler();
+        });
+
+        on(that, 'success', function(e) {
+            if (that.on_success) that.on_success(e.data, e.text_status, e.xhr);
+        });
+
+        on(that, 'error', function(xhr, text_status, error_thrown) {
+            if (that.on_error) that.on_error(xhr, text_status, error_thrown);
+        });
+    };
+
+    that.register_handlers();
+
     return that;
 };
 
@@ -628,7 +677,8 @@ rpc.batch_command = function(spec) {
 
             if (!result) {
                 name = text.get('@i18n:errors.internal_error', 'Internal Error')+' '+xhr.status;
-                message = result ? xhr.statusText : text.get('@i18n:errors.internal_error', 'Internal Error');
+                message = xhr ? xhr.statusText :
+                    text.get('@i18n:errors.internal_error', 'Internal Error');
 
                 that.errors.add(command, name, message, text_status);
 
@@ -665,11 +715,14 @@ rpc.batch_command = function(spec) {
                 var failed = that.get_failed(command, result, text_status, xhr);
                 that.errors.add_range(failed);
 
-                if (command.on_success) command.on_success.call(this, result, text_status, xhr);
+                command.emit('success', {
+                    data: result,
+                    text_status: text_status,
+                    xhr: xhr
+                });
             }
         }
 
-        //check for partial errors and show error dialog
         if (that.show_error && that.errors.errors.length > 0) {
             var ajax = this;
             var dialog = IPA.error_dialog({
@@ -686,13 +739,22 @@ rpc.batch_command = function(spec) {
 
             dialog.on_ok = function() {
                 dialog.close();
-                if (that.on_success) that.on_success.call(ajax, data, text_status, xhr);
+                that.emit('success', {
+                    that: ajax,
+                    data: data,
+                    text_status: text_status,
+                    xhr: xhr
+                });
             };
 
             dialog.open();
 
         } else {
-            if (that.on_success) that.on_success.call(this, data, text_status, xhr);
+            that.emit('success', {
+                data: data,
+                text_status: text_status,
+                xhr: xhr
+            });
         }
     };
 
