@@ -170,6 +170,7 @@ return {
         }
     ],
     adder_dialog: {
+        title: '@i18n:objects.topologysegment.add',
         fields: [
             {
                 name: 'cn',
@@ -190,6 +191,9 @@ return {
                 z_index: 1
             }
         ]
+    },
+    deleter_dialog: {
+        title: '@i18n:objects.topologysegment.remove'
     }
 };};
 
@@ -445,7 +449,9 @@ return {
                                 {
                                     name: 'service_relative_weight'
                                 }
-                            ]
+                            ],
+                            add_title: '@i18n:objects.topologylocation.add_server',
+                            remove_title: '@i18n:objects.topologylocation.remove_servers'
                         }
                     ]
                 }
@@ -453,6 +459,7 @@ return {
         }
     ],
     adder_dialog: {
+        title: '@i18n:objects.topologylocation.add',
         fields: [
             {
                 $type: 'text',
@@ -461,6 +468,9 @@ return {
             },
             'description'
         ]
+    },
+    deleter_dialog: {
+        title: '@i18n:objects.topologylocation.remove'
     }
 };};
 
@@ -490,30 +500,45 @@ topology.servers_search_facet = function(spec, no_init) {
 
     var that = IPA.search_facet(spec);
 
-    that.create_get_records_command = function(pkeys, on_success, on_error) {
+    that.create_get_records_command = function(records, pkeys, on_success, on_error) {
 
         var on_success_extended = function(data, text_status, xhr) {
             // Call original on_success handler
             on_success(data, text_status, xhr);
 
             var result = data.result.results;
-            var counter = 0;
+
+            // Do not show warning if there is only one master
+            if (result.length <= 1) return;
+
+            var ca_counter = 0;
+            var kra_counter = 0;
 
             for (var i=0, l=result.length; i<l; i++) {
                 var current = result[i];
                 var roles = current.result.enabled_role_servrole;
                 for (var k=0, m=roles.length; k<m; k++) {
-                    if (roles[k] === 'CA server') counter++;
+                    if (roles[k] === 'CA server') ca_counter++;
+                    if (roles[k] === 'KRA server') kra_counter++;
                 }
             }
 
-            // Create dialog and show it only when there is only one CA server
-            if (counter != 1) return;
+            // Show a dialog when there is only one CA or KRA server
+            if (ca_counter != 1 && kra_counter != 1) return;
+
+            var message = text.get(
+                '@i18n:objects.servers.svc_warning_message') + '<ul>';
+            if (ca_counter === 1) {
+                message += '<li>CA</li>\n';
+            }
+            if (kra_counter === 1) {
+                message += '<li>KRA</li>\n';
+            }
+            message += '</ul>';
 
-            var message = text.get('@i18n:objects.servers.ca_warning_message');
             var dialog = IPA.dialog({
-                name: 'ca_warning',
-                title: '@i18n:objects.servers.ca_warning_title',
+                name: 'svc_warning',
+                title: '@i18n:objects.servers.svc_warning_title',
                 sections: [
                     {
                         show_header: false,
@@ -545,7 +570,7 @@ topology.servers_search_facet = function(spec, no_init) {
             dialog.open();
         };
 
-        var batch = that.table_facet_create_get_records_command(pkeys,
+        var batch = that.table_facet_create_get_records_command(records, pkeys,
                                                 on_success_extended, on_error);
 
         return batch;
@@ -691,14 +716,10 @@ topology.location_association_table_widget = function(spec) {
 
     that.create_add_dialog = function() {
 
-        var entity_label = that.entity.metadata.label_singular;
         var pkey = that.facet.get_pkey();
-        var other_entity_label = that.other_entity.metadata.label_singular;
 
         var title = that.add_title;
-        title = title.replace('${entity}', entity_label);
         title = title.replace('${primary_key}', pkey);
-        title = title.replace('${other_entity}', other_entity_label);
 
         return topology.location_server_adder_dialog({
             title: title,
@@ -993,10 +1014,7 @@ topology.create_add_dialog = function(spec) {
 
     spec.entity = spec.entity || 'topologysegment';
 
-    var entity = reg.entity.get('topologysegment');
-    var title = text.get('@i18n:dialogs.add_title');
-    var label = entity.metadata.label_singular;
-    spec.title = title.replace('${entity}', label);
+    spec.title = text.get('@i18n:objects.topologysegment.add');
 
     spec.fields = spec.fields || [
         {
