@@ -325,8 +325,8 @@ topology_graph.TopoGraph = declare([Evented], {
                 off = dir ? -1 : 1, // determines shift direction of curve
                 ns = 5, // shift on normal vector
                 s = target_count > 1 ? 1 : 0, // shift from center?
-                spad = d.left ? 18 : 18, // source padding
-                tpad = d.right ? 18 : 18, // target padding
+                spad = 18, // source padding
+                tpad = 18, // target padding
                 sourceX = d.source.x + (spad * ux) + off * nx * ns * s,
                 sourceY = d.source.y + (spad * uy) + off * ny * ns * s,
                 targetX = d.target.x - (tpad * ux) + off * nx * ns * s,
