@@ -19,7 +19,6 @@
 */
 
 define(['dojo/_base/declare',
-        'dojo/_base/lang',
         'dojo/_base/array',
         'dojo/dom',
         'dojo/dom-construct',
@@ -35,7 +34,7 @@ define(['dojo/_base/declare',
         '../widget',
         'dojo/NodeList-dom'
        ],
-       function(declare, lang, array, dom, construct, prop, dom_class,
+       function(declare, array, dom, construct, prop, dom_class,
                 dom_style, query, on, Menu, DropdownWidget,
                 FacetContainer, text, widgets) {
 
@@ -223,6 +222,10 @@ define(['dojo/_base/declare',
             }
         },
 
+        disable_user_menu_item: function(name) {
+            this.user_menu.disable_item(name);
+        },
+
         on_menu_item_click: function(item) {
             this.collapse_menu();
         },
@@ -235,6 +238,8 @@ define(['dojo/_base/declare',
                 this.emit('logout-click');
             } else if (item.name == 'password_reset') {
                 this.emit('password-reset-click');
+            } else if (item.name == 'configuration') {
+                this.emit('configuration-click');
             } else if (item.name == 'about') {
                 this.emit('about-click');
             }
@@ -254,20 +259,28 @@ define(['dojo/_base/declare',
                 items: [
                     {
                         name: 'profile',
-                        label: 'Profile',
+                        label: text.get('@i18n:profile-menu.profile',
+                                        'Profile'),
                         icon: 'fa-user'
                     },
                     {
                         name: 'password_reset',
-                        label: 'Change password',
+                        label: text.get('@i18n:profile-menu.password_reset',
+                                        'Change password'),
                         icon: 'fa-key'
                     },
                     {
                         'class': 'divider'
                     },
+                    {
+                        name: 'configuration',
+                        label: text.get('@i18n:profile-menu.configuration',
+                                        'Customization'),
+                        icon: 'fa-gear'
+                    },
                     {
                         name: 'about',
-                        label: 'About',
+                        label: text.get('@i18n:profile-menu.about', 'About'),
                         icon: 'fa-question'
                     },
                     {
@@ -275,16 +288,17 @@ define(['dojo/_base/declare',
                     },
                     {
                         name: 'logout',
-                        label: 'Logout',
+                        label: text.get('@i18n:profile-menu.logout',
+                                        'Log out'),
                         icon: 'fa-sign-out'
                     }
                 ]
             });
-            on(this.user_menu, 'item-click', lang.hitch(this, this.on_user_menu_click));
-            on(this.menu_widget, 'item-select', lang.hitch(this, this.on_menu_item_click));
+            on(this.user_menu, 'item-click', this.on_user_menu_click.bind(this));
+            on(this.menu_widget, 'item-select', this.on_menu_item_click.bind(this));
         }
 
     });
 
     return app;
-});
\ No newline at end of file
+});
