@@ -19,9 +19,58 @@ if six.PY3:
 __doc__ = _("""
 Topology
 
-Management of a replication topology.
+Management of a replication topology at domain level 1.
 
-Requires minimum domain level 1.
+IPA server's data is stored in LDAP server in two suffixes:
+* domain suffix, e.g., 'dc=example,dc=com', contains all domain related data
+* ca suffix, 'o=ipaca', is present only on server with CA installed. It
+  contains data for Certificate Server component
+
+Data stored on IPA servers is replicated to other IPA servers. The way it is
+replicated is defined by replication agreements. Replication agreements needs
+to be set for both suffixes separately. On domain level 0 they are managed
+using ipa-replica-manage and ipa-csreplica-manage tools. With domain level 1
+they are managed centrally using `ipa topology*` commands.
+
+Agreements are represented by topology segments. By default topology segment
+represents 2 replication agreements - one for each direction, e.g., A to B and
+B to A. Creation of unidirectional segments is not allowed.
+
+To verify that no server is disconnected in the topology of the given suffix,
+use:
+  ipa topologysuffix-verify $suffix
+
+
+Examples:
+  Find all IPA servers:
+    ipa server-find
+
+  Find all suffixes:
+    ipa topologysuffix-find
+
+  Add topology segment to 'domain' suffix:
+    ipa topologysegment-add domain --left IPA_SERVER_A --right IPA_SERVER_B
+
+  Add topology segment to 'ca' suffix:
+    ipa topologysegment-add ca --left IPA_SERVER_A --right IPA_SERVER_B
+
+  List all topology segments in 'domain' suffix:
+    ipa topologysegment-find domain
+
+  List all topology segments in 'ca' suffix:
+    ipa topologysegment-find ca
+
+  Delete topology segment in 'domain' suffix:
+    ipa topologysegment-del domain segment_name
+
+  Delete topology segment in 'ca' suffix:
+    ipa topologysegment-del ca segment_name
+
+  Verify topology of 'domain' suffix:
+    ipa topologysuffix-verify domain
+
+  Verify topology of 'ca' suffix:
+    ipa topologysuffix-verify ca
 """)
 
 register = Registry()
@@ -92,9 +141,9 @@ class topologysuffix(Object):
             primary_key=True,
             label=_(u'Suffix name'),
         ),
-        parameters.Str(
+        parameters.DNParam(
             'iparepltopoconfroot',
-            label=_(u'LDAP suffix to be managed'),
+            label=_(u'Managed LDAP suffix DN'),
         ),
     )
 
@@ -103,8 +152,6 @@ class topologysuffix(Object):
 class topologysegment_add(Method):
     __doc__ = _("Add a new segment.")
 
-    NO_CLI = True
-
     takes_args = (
         parameters.Str(
             'topologysuffixcn',
@@ -145,6 +192,7 @@ class topologysegment_add(Method):
             doc=_(u'Direction of replication between left and right replication node'),
             exclude=('cli', 'webui'),
             default=u'both',
+            autofill=True,
         ),
         parameters.Str(
             'nsds5replicastripattrs',
@@ -233,8 +281,6 @@ class topologysegment_add(Method):
 class topologysegment_del(Method):
     __doc__ = _("Delete a segment.")
 
-    NO_CLI = True
-
     takes_args = (
         parameters.Str(
             'topologysuffixcn',
@@ -282,8 +328,6 @@ class topologysegment_del(Method):
 class topologysegment_find(Method):
     __doc__ = _("Search for topology segments.")
 
-    NO_CLI = True
-
     takes_args = (
         parameters.Str(
             'topologysuffixcn',
@@ -433,8 +477,6 @@ class topologysegment_find(Method):
 class topologysegment_mod(Method):
     __doc__ = _("Modify a segment.")
 
-    NO_CLI = True
-
     takes_args = (
         parameters.Str(
             'topologysuffixcn',
@@ -554,8 +596,6 @@ class topologysegment_mod(Method):
 class topologysegment_reinitialize(Method):
     __doc__ = _("Request a full re-initialization of the node retrieving data from the other node.")
 
-    NO_CLI = True
-
     takes_args = (
         parameters.Str(
             'topologysuffixcn',
@@ -618,8 +658,6 @@ class topologysegment_reinitialize(Method):
 class topologysegment_show(Method):
     __doc__ = _("Display a segment.")
 
-    NO_CLI = True
-
     takes_args = (
         parameters.Str(
             'topologysuffixcn',
@@ -690,11 +728,10 @@ class topologysuffix_add(Method):
         ),
     )
     takes_options = (
-        parameters.Str(
+        parameters.DNParam(
             'iparepltopoconfroot',
-            cli_name='suffix',
-            label=_(u'LDAP suffix to be managed'),
-            no_convert=True,
+            cli_name='suffix_dn',
+            label=_(u'Managed LDAP suffix DN'),
         ),
         parameters.Str(
             'setattr',
@@ -782,9 +819,7 @@ class topologysuffix_del(Method):
 
 @register()
 class topologysuffix_find(Method):
-    __doc__ = _("Search for topology suffices.")
-
-    NO_CLI = True
+    __doc__ = _("Search for topology suffixes.")
 
     takes_args = (
         parameters.Str(
@@ -800,12 +835,11 @@ class topologysuffix_find(Method):
             cli_name='name',
             label=_(u'Suffix name'),
         ),
-        parameters.Str(
+        parameters.DNParam(
             'iparepltopoconfroot',
             required=False,
-            cli_name='suffix',
-            label=_(u'LDAP suffix to be managed'),
-            no_convert=True,
+            cli_name='suffix_dn',
+            label=_(u'Managed LDAP suffix DN'),
         ),
         parameters.Int(
             'timelimit',
@@ -878,12 +912,11 @@ class topologysuffix_mod(Method):
         ),
     )
     takes_options = (
-        parameters.Str(
+        parameters.DNParam(
             'iparepltopoconfroot',
             required=False,
-            cli_name='suffix',
-            label=_(u'LDAP suffix to be managed'),
-            no_convert=True,
+            cli_name='suffix_dn',
+            label=_(u'Managed LDAP suffix DN'),
         ),
         parameters.Str(
             'setattr',
@@ -948,8 +981,6 @@ class topologysuffix_mod(Method):
 class topologysuffix_show(Method):
     __doc__ = _("Show managed suffix.")
 
-    NO_CLI = True
-
     takes_args = (
         parameters.Str(
             'cn',
@@ -1008,8 +1039,6 @@ Checks done:
      replication agreements
     """)
 
-    NO_CLI = True
-
     takes_args = (
         parameters.Str(
             'cn',
