@@ -25,7 +25,8 @@ import six
 # The Python re module doesn't do nested parenthesis
 
 # Break the ACI into 3 pieces: target, name, permissions/bind_rules
-ACIPat = re.compile(r'\(version\s+3.0\s*;\s*ac[li]\s+\"([^\"]*)\"\s*;\s*([^;]*);\s*\)', re.UNICODE)
+ACIPat = re.compile(r'\(version\s+3.0\s*;\s*ac[li]\s+\"([^\"]*)\"\s*;'
+                    r'\s*(.*);\s*\)', re.UNICODE)
 
 # Break the permissions/bind_rules out
 PermPat = re.compile(r'(\w+)\s*\(([^()]*)\)\s*(.*)', re.UNICODE)
@@ -39,12 +40,15 @@ ACTIONS = ["allow", "deny"]
 PERMISSIONS = ["read", "write", "add", "delete", "search", "compare",
                "selfwrite", "proxy", "all"]
 
-class ACI:
+
+class ACI(object):
     """
     Holds the basic data for an ACI entry, as stored in the cn=accounts
     entry in LDAP.  Has methods to parse an ACI string and export to an
     ACI String.
     """
+    __hash__ = None
+
     def __init__(self,acistr=None):
         self.name = None
         self.source_group = None
@@ -102,8 +106,6 @@ class ACI:
         lexer = shlex.shlex(aci)
         lexer.wordchars = lexer.wordchars + "."
 
-        l = []
-
         var = False
         op = "="
         for token in lexer:
@@ -111,10 +113,10 @@ class ACI:
             if token == "(":
                 var = next(lexer).strip()
                 operator = next(lexer)
-                if operator != "=" and operator != "!=":
+                if operator not in ("=", "!="):
                     # Peek at the next char before giving up
                     operator = operator + next(lexer)
-                    if operator != "=" and operator != "!=":
+                    if operator not in ("=", "!="):
                         raise SyntaxError("No operator in target, got '%s'" % operator)
                 op = operator
                 val = next(lexer).strip()
@@ -125,7 +127,7 @@ class ACI:
 
             if var == 'targetattr':
                 # Make a string of the form attr || attr || ... into a list
-                t = re.split('[^a-zA-Z0-9;\*]+', val)
+                t = re.split(r'[^a-zA-Z0-9;\*]+', val)
                 self.target[var] = {}
                 self.target[var]['operator'] = op
                 self.target[var]['expression'] = t
@@ -156,10 +158,10 @@ class ACI:
 
            returns True if valid
         """
-        if not type(self.permissions) in (tuple, list):
+        if type(self.permissions) not in (tuple, list):
             raise SyntaxError("permissions must be a list")
         for p in self.permissions:
-            if not p.lower() in PERMISSIONS:
+            if p.lower() not in PERMISSIONS:
                 raise SyntaxError("invalid permission: '%s'" % p)
         if not self.name:
             raise SyntaxError("name must be set")
@@ -185,7 +187,7 @@ class ACI:
             if 'targetattr' in self.target:
                 del self.target['targetattr']
             return
-        if not type(attr) in (tuple, list):
+        if type(attr) not in (tuple, list):
             attr = [attr]
         self.target['targetattr'] = {}
         self.target['targetattr']['expression'] = attr
