@@ -22,27 +22,44 @@ Functionality for Command Line Interface.
 """
 from __future__ import print_function
 
+import atexit
 import importlib
+import logging
 import textwrap
 import sys
 import getpass
 import code
-import optparse
+import optparse  # pylint: disable=deprecated-module
+import os
+import pprint
 import fcntl
 import termios
 import struct
 import base64
 import traceback
+try:
+    import readline
+    import rlcompleter
+except ImportError:
+    readline = rlcompleter = None
+
 
 import six
 from six.moves import input
 
+from ipalib.util import (
+    check_client_configuration, get_terminal_height, open_in_pager
+)
+
 if six.PY3:
     unicode = str
+    import builtins  # pylint: disable=import-error
+else:
+    import __builtin__ as builtins  # pylint: disable=import-error
 
 if six.PY2:
-    reload(sys)                         # pylint: disable=reload-builtin
-    sys.setdefaultencoding('utf-8')     # pylint: disable=no-member
+    reload(sys)  # pylint: disable=reload-builtin, undefined-variable
+    sys.setdefaultencoding('utf-8')  # pylint: disable=no-member
 
 from ipalib import frontend
 from ipalib import backend
@@ -51,13 +68,16 @@ from ipalib.errors import (PublicError, CommandError, HelpError, InternalError,
                            NoSuchNamespaceError, ValidationError, NotFound,
                            NotConfiguredError, PromptFailed)
 from ipalib.constants import CLI_TAB, LDAP_GENERALIZED_TIME_FORMAT
-from ipalib.parameters import File, Str, Enum, Any, Flag
+from ipalib.parameters import File, BinaryFile, Str, Enum, Any, Flag
 from ipalib.text import _
 from ipalib import api  # pylint: disable=unused-import
 from ipapython.dnsutil import DNSName
+from ipapython.admintool import ScriptError
 
 import datetime
 
+logger = logging.getLogger(__name__)
+
 
 def to_cli(name):
     """
@@ -95,7 +115,8 @@ class textui(backend.Backend):
                                       struct.pack('HHHH', 0, 0, 0, 0))
                 return struct.unpack('HHHH', winsize)[1]
             except IOError:
-                return None
+                pass
+        return None
 
     def max_col_width(self, rows, col=None):
         """
@@ -599,6 +620,8 @@ class textui(backend.Backend):
             elif default is not None and data == u'':
                 return default
 
+        return default  # pylint consinstent return statements
+
     def prompt_password(self, label, confirm=True):
         """
         Prompt user for a password or read it in via stdin depending
@@ -614,7 +637,8 @@ class textui(backend.Backend):
                 pw2 = self.prompt_helper(repeat_prompt, label, prompt_func=getpass.getpass)
                 if pw1 == pw2:
                     return pw1
-                self.print_error( _('Passwords do not match!'))
+                else:
+                    self.print_error(_('Passwords do not match!'))
         else:
             return self.decode(sys.stdin.readline().strip())
 
@@ -664,7 +688,7 @@ class textui(backend.Backend):
                 return -1
             try:
                 selection = int(resp) - 1
-                if (selection >= 0 and selection < counter):
+                if (counter > selection >= 0):
                     break
             except Exception:
                 # fall through to the error msg
@@ -675,10 +699,39 @@ class textui(backend.Backend):
         self.print_line('')
         return selection
 
+
 class help(frontend.Local):
     """
     Display help for a command or topic.
     """
+    class Writer(object):
+        """
+        Writer abstraction
+        """
+        def __init__(self, outfile):
+            self.outfile = outfile
+            self.buffer = []
+
+        @property
+        def buffer_length(self):
+            length = 0
+            for line in self.buffer:
+                length += len(line.split("\n"))
+            return length
+
+        def append(self, string=u""):
+            self.buffer.append(unicode(string))
+
+        def write(self):
+            if self.buffer_length > get_terminal_height():
+                data = "\n".join(self.buffer).encode("utf-8")
+                open_in_pager(data)
+            else:
+                try:
+                    for line in self.buffer:
+                        print(line, file=self.outfile)
+                except IOError:
+                    pass
 
     takes_args = (
         Str('command?', cli_name='topic', label=_('Topic or Command'),
@@ -697,7 +750,7 @@ class help(frontend.Local):
         parent_topic = None
 
         for package in self.api.packages:
-            module_name = '%s.%s' % (package.__name__, topic)
+            module_name = '{0}.{1}'.format(package.__name__, topic)
             try:
                 module = sys.modules[module_name]
             except KeyError:
@@ -720,7 +773,8 @@ class help(frontend.Local):
         self._topics[topic_name][1] = mcl
 
     def _on_finalize(self):
-        # {topic: ["description", mcl, {"subtopic": ["description", mcl, [commands]]}]}
+        # {topic: ["description", mcl, {
+        #     "subtopic": ["description", mcl, [commands]]}]}
         # {topic: ["description", mcl, [commands]]}
         self._topics = {}
         # [builtin_commands]
@@ -744,22 +798,26 @@ class help(frontend.Local):
                         self._topics[topic_name] = [doc, 0, [c]]
                     mcl = max((self._topics[topic_name][1], len(c.name)))
                     self._topics[topic_name][1] = mcl
-                else: # a module grouped in a topic
+                else:  # a module grouped in a topic
                     topic = self._get_topic(topic_name)
                     mod_name = c.topic
                     if topic_name in self._topics:
                         if mod_name in self._topics[topic_name][2]:
                             self._topics[topic_name][2][mod_name][2].append(c)
                         else:
-                            self._topics[topic_name][2][mod_name] = [doc, 0, [c]]
+                            self._topics[topic_name][2][mod_name] = [
+                                doc, 0, [c]]
                             self._count_topic_mcl(topic_name, mod_name)
                         # count mcl for for the subtopic
-                        mcl = max((self._topics[topic_name][2][mod_name][1], len(c.name)))
+                        mcl = max((
+                            self._topics[topic_name][2][mod_name][1],
+                            len(c.name)))
                         self._topics[topic_name][2][mod_name][1] = mcl
                     else:
-                        self._topics[topic_name] = [topic[0].split('\n', 1)[0],
-                                                    0,
-                                                    {mod_name: [doc, 0, [c]]}]
+                        self._topics[topic_name] = [
+                            topic[0].split('\n', 1)[0],
+                            0,
+                            {mod_name: [doc, 0, [c]]}]
                         self._count_topic_mcl(topic_name, mod_name)
             else:
                 self._builtins.append(c)
@@ -773,8 +831,10 @@ class help(frontend.Local):
     def run(self, key=None, outfile=None, **options):
         if outfile is None:
             outfile = sys.stdout
-        writer = self._writer(outfile)
+
+        writer = self.Writer(outfile)
         name = from_cli(key)
+
         if key is None:
             self.api.parser.print_help(outfile)
             return
@@ -799,33 +859,30 @@ class help(frontend.Local):
                 if cmd_plugin.NO_CLI:
                     continue
                 mcl = max(mcl, len(cmd_plugin.name))
-                writer('%s  %s' % (to_cli(cmd_plugin.name).ljust(mcl),
-                                   cmd_plugin.summary))
+                writer.append('{0}  {1}'.format(
+                    to_cli(cmd_plugin.name).ljust(mcl), cmd_plugin.summary))
         else:
             raise HelpError(topic=name)
-
-    def _writer(self, outfile):
-        def writer(string=''):
-            try:
-                print(unicode(string), file=outfile)
-            except IOError:
-                pass
-        return writer
+        writer.write()
 
     def print_topics(self, outfile):
-        writer = self._writer(outfile)
+        writer = self.Writer(outfile)
 
         for t, topic in sorted(self._topics.items()):
-            writer('%s  %s' % (to_cli(t).ljust(self._mtl), topic[0]))
+            writer.append('{0}  {1}'.format(
+                to_cli(t).ljust(self._mtl), topic[0]))
+        writer.write()
 
     def print_commands(self, topic, outfile):
-        writer = self._writer(outfile)
+        writer = self.Writer(outfile)
+
         if topic in self._topics and type(self._topics[topic][2]) is dict:
             # we want to display topic which has subtopics
             for subtopic in self._topics[topic][2]:
                 doc = self._topics[topic][2][subtopic][0]
                 mcl = self._topics[topic][1]
-                writer('  %s  %s' % (to_cli(subtopic).ljust(mcl), doc))
+                writer.append('  {0}  {1}'.format(
+                    to_cli(subtopic).ljust(mcl), doc))
         else:
             # we want to display subtopic or a topic which has no subtopics
             if topic in self._topics:
@@ -847,17 +904,20 @@ class help(frontend.Local):
             if topic not in self.Command and len(commands) == 0:
                 raise HelpError(topic=topic)
 
-            writer(doc)
+            writer.append(doc)
             if commands:
-                writer()
-                writer(_('Topic commands:'))
+                writer.append()
+                writer.append(_('Topic commands:'))
                 for c in commands:
-                    writer(
-                        '  %s  %s' % (to_cli(c.name).ljust(mcl), c.summary))
-                writer()
-                writer(_('To get command help, use:'))
-                writer(_('  ipa <command> --help'))
-            writer()
+                    writer.append(
+                        '  {0}  {1}'.format(
+                            to_cli(c.name).ljust(mcl), c.summary))
+                writer.append()
+                writer.append(_('To get command help, use:'))
+                writer.append(_('  ipa <command> --help'))
+            writer.append()
+        writer.write()
+
 
 class show_mappings(frontend.Command):
     """
@@ -901,22 +961,61 @@ class console(frontend.Command):
 
     topic = None
 
+    def _setup_tab_completion(self, local):
+        readline.parse_and_bind("tab: complete")
+        # completer with custom locals
+        readline.set_completer(rlcompleter.Completer(local).complete)
+        # load history
+        history = os.path.join(api.env.dot_ipa, "console.history")
+        try:
+            readline.read_history_file(history)
+        except OSError:
+            pass
+
+        def save_history():
+            directory = os.path.dirname(history)
+            if not os.path.isdir(directory):
+                os.makedirs(directory)
+            readline.set_history_length(50)
+            try:
+                readline.write_history_file(history)
+            except OSError:
+                logger.exception("Unable to store history %s", history)
+
+        atexit.register(save_history)
+
     def run(self, filename=None, **options):
-        local = dict(api=self.api)
+        local = dict(
+            api=self.api,
+            pp=pprint.pprint,  # just too convenient
+            __builtins__=builtins,
+        )
         if filename:
             try:
-                script = open(filename)
+                with open(filename) as f:
+                    source = f.read()
             except IOError as e:
-                exit("%s: %s" % (e.filename, e.strerror))
+                sys.exit("%s: %s" % (e.filename, e.strerror))
             try:
-                with script:
-                    exec(script, globals(), local)
+                compiled = compile(
+                    source,
+                    filename,
+                    'exec',
+                    flags=print_function.compiler_flag
+                )
+                exec(compiled, globals(), local)
             except Exception:
                 traceback.print_exc()
-                exit(1)
+                sys.exit(1)
         else:
+            if readline is not None:
+                self._setup_tab_completion(local)
             code.interact(
-                '(Custom IPA interactive Python console)',
+                "\n".join((
+                    "(Custom IPA interactive Python console)",
+                    "    api: IPA API object",
+                    "    pp: pretty printer",
+                )),
                 local=local
             )
 
@@ -994,7 +1093,9 @@ class Collector(object):
                 value = v + (value,)
             else:
                 value = (v, value)
+        # pylint: disable=unsupported-assignment-operation
         self.__options[name] = value
+        # pylint: enable=unsupported-assignment-operation
         object.__setattr__(self, name, value)
 
     def __todict__(self):
@@ -1070,7 +1171,7 @@ class cli(backend.Executioner):
             self.Command.help(outfile=sys.stderr)
             print(file=sys.stderr)
             print('Error: Command not specified', file=sys.stderr)
-            exit(2)
+            sys.exit(2)
         (key, argv) = (argv[0], argv[1:])
         name = from_cli(key)
         if name not in self.Command and len(argv) == 0:
@@ -1100,7 +1201,7 @@ class cli(backend.Executioner):
     def run(self, argv):
         cmd = self.get_command(argv)
         if cmd is None:
-            return
+            return None
         name = cmd.full_name
         kw = self.parse(cmd, argv[1:])
         if not isinstance(cmd, frontend.Local):
@@ -1120,6 +1221,7 @@ class cli(backend.Executioner):
                     return 0
         finally:
             self.destroy_context()
+        return None
 
     def parse(self, cmd, argv):
         parser = self.build_parser(cmd)
@@ -1203,7 +1305,7 @@ class cli(backend.Executioner):
 
     def __get_arg_name(self, arg, format_name=True):
         if arg.password:
-            return
+            return None
 
         name = to_cli(arg.cli_name).upper()
         if not format_name:
@@ -1282,7 +1384,7 @@ class cli(backend.Executioner):
         3) the webUI will use a different way of loading files
         """
         for p in cmd.params():
-            if isinstance(p, File):
+            if isinstance(p, (File, BinaryFile)):
                 # FIXME: this only reads the first file
                 raw = None
                 if p.name in kw:
@@ -1291,9 +1393,8 @@ class cli(backend.Executioner):
                     else:
                         fname = kw[p.name]
                     try:
-                        f = open(fname, 'r')
-                        raw = f.read()
-                        f.close()
+                        with open(fname, p.open_mode) as f:
+                            raw = f.read()
                     except IOError as e:
                         raise ValidationError(
                             name=to_cli(p.cli_name),
@@ -1301,14 +1402,22 @@ class cli(backend.Executioner):
                         )
                 elif p.stdin_if_missing:
                     try:
-                        raw = sys.stdin.read()
+                        if six.PY3 and p.type is bytes:
+                            # pylint: disable=no-member
+                            raw = sys.stdin.buffer.read()
+                            # pylint: enable=no-member
+                        else:
+                            raw = sys.stdin.read()
                     except IOError as e:
                         raise ValidationError(
                             name=to_cli(p.cli_name), error=e.args[1]
                         )
 
                 if raw:
-                    kw[p.name] = self.Backend.textui.decode(raw)
+                    if p.type is bytes:
+                        kw[p.name] = raw
+                    else:
+                        kw[p.name] = self.Backend.textui.decode(raw)
                 elif p.required:
                     raise ValidationError(
                         name=to_cli(p.cli_name), error=_('No file to read')
@@ -1342,7 +1451,13 @@ cli_plugins = (
 def run(api):
     error = None
     try:
-        (options, argv) = api.bootstrap_with_global_options(context='cli')
+        (_options, argv) = api.bootstrap_with_global_options(context='cli')
+
+        try:
+            check_client_configuration()
+        except ScriptError as e:
+            sys.exit(e)
+
         for klass in cli_plugins:
             api.add_plugin(klass)
         api.finalize()
@@ -1351,13 +1466,13 @@ def run(api):
         sys.exit(api.Backend.cli.run(argv))
     except KeyboardInterrupt:
         print('')
-        api.log.info('operation aborted')
+        logger.info('operation aborted')
     except PublicError as e:
         error = e
     except Exception as e:
-        api.log.exception('%s: %s', e.__class__.__name__, str(e))
+        logger.exception('%s: %s', e.__class__.__name__, str(e))
         error = InternalError()
     if error is not None:
         assert isinstance(error, PublicError)
-        api.log.error(error.strerror)
+        logger.error(error.strerror)
         sys.exit(error.rval)
