@@ -23,26 +23,37 @@ This base module contains default implementations of IPA interface for
 interacting with system services.
 '''
 
+from __future__ import absolute_import
+
 import os
 import json
 import time
-import collections
+import logging
+import warnings
 
 import six
 
-import ipalib
 from ipapython import ipautil
 from ipaplatform.paths import paths
 
+# pylint: disable=no-name-in-module, import-error
+if six.PY3:
+    from collections.abc import Mapping
+else:
+    from collections import Mapping
+# pylint: enable=no-name-in-module, import-error
+
+logger = logging.getLogger(__name__)
+
 # Canonical names of services as IPA wants to see them. As we need to have
 # *some* naming, set them as in Red Hat distributions. Actual implementation
 # should make them available through knownservices.<name> and take care of
 # re-mapping internally, if needed
 wellknownservices = ['certmonger', 'dirsrv', 'httpd', 'ipa', 'krb5kdc',
-                     'messagebus', 'nslcd', 'nscd', 'ntpd', 'portmap',
+                     'dbus', 'nslcd', 'nscd', 'ntpd', 'portmap',
                      'rpcbind', 'kadmin', 'sshd', 'autofs', 'rpcgssd',
                      'rpcidmapd', 'pki_tomcatd', 'chronyd', 'domainname',
-                     'named', 'ods_enforcerd', 'ods_signerd']
+                     'named', 'ods_enforcerd', 'ods_signerd', 'gssproxy']
 
 # The common ports for these services. This is used to wait for the
 # service to become available.
@@ -56,14 +67,13 @@ wellknownports = {
 SERVICE_POLL_INTERVAL = 0.1 # seconds
 
 
-class KnownServices(collections.Mapping):
+class KnownServices(Mapping):
     """
     KnownServices is an abstract class factory that should give out instances
     of well-known platform services. Actual implementation must create these
     instances as its own attributes on first access (or instance creation)
     and cache them.
     """
-
     def __init__(self, d):
         self.__d = d
 
@@ -93,9 +103,19 @@ class PlatformService(object):
 
     """
 
-    def __init__(self, service_name, api=ipalib.api):
+    def __init__(self, service_name, api=None):
+        # pylint: disable=ipa-forbidden-import
+        import ipalib  # FixMe: break import cycle
+        # pylint: enable=ipa-forbidden-import
         self.service_name = service_name
-        self.api = api
+        if api is not None:
+            self.api = api
+        else:
+            self.api = ipalib.api
+            warnings.warn(
+                "{s.__class__.__name__}('{s.service_name}', api=None) "
+                "is deprecated.".format(s=self),
+                RuntimeWarning, stacklevel=2)
 
     def start(self, instance_name="", capture_output=True, wait=True,
         update_service_list=True):
@@ -120,8 +140,6 @@ class PlatformService(object):
         with open(paths.SVC_LIST_FILE, 'w') as f:
             json.dump(svc_list, f)
 
-        return
-
     def stop(self, instance_name="", capture_output=True,
              update_service_list=True):
         """
@@ -143,10 +161,12 @@ class PlatformService(object):
         with open(paths.SVC_LIST_FILE, 'w') as f:
             json.dump(svc_list, f)
 
-        return
+    def reload_or_restart(self, instance_name="", capture_output=True,
+                          wait=True):
+        pass
 
     def restart(self, instance_name="", capture_output=True, wait=True):
-        return
+        pass
 
     def is_running(self, instance_name="", wait=True):
         return False
@@ -161,34 +181,32 @@ class PlatformService(object):
         return False
 
     def enable(self, instance_name=""):
-        return
+        pass
 
     def disable(self, instance_name=""):
-        return
+        pass
 
     def mask(self, instance_name=""):
-        return
+        pass
 
     def unmask(self, instance_name=""):
-        return
+        pass
 
     def install(self, instance_name=""):
-        return
+        pass
 
     def remove(self, instance_name=""):
-        return
-
-    def get_config_dir(self, instance_name=""):
-        return
+        pass
 
 
 class SystemdService(PlatformService):
     SYSTEMD_SRV_TARGET = "%s.target.wants"
 
-    def __init__(self, service_name, systemd_name, **kwargs):
-        super(SystemdService, self).__init__(service_name, **kwargs)
+    def __init__(self, service_name, systemd_name, api=None):
+        super(SystemdService, self).__init__(service_name, api=api)
         self.systemd_name = systemd_name
-        self.lib_path = os.path.join(paths.LIB_SYSTEMD_SYSTEMD_DIR, self.systemd_name)
+        self.lib_path = os.path.join(paths.LIB_SYSTEMD_SYSTEMD_DIR,
+                                     self.systemd_name)
         self.lib_path_exists = None
 
     def service_instance(self, instance_name, operation=None):
@@ -271,37 +289,49 @@ class SystemdService(PlatformService):
 
         ipautil.run(args, skip_output=not capture_output)
 
-        if getattr(self.api.env, 'context', None) in ['ipactl', 'installer']:
-            update_service_list = True
-        else:
-            update_service_list = False
+        update_service_list = getattr(self.api.env, 'context',
+                                      None) in ['ipactl', 'installer']
         super(SystemdService, self).stop(
             instance_name,
             update_service_list=update_service_list)
+        logger.debug('Stop of %s complete', instance)
 
     def start(self, instance_name="", capture_output=True, wait=True):
         ipautil.run([paths.SYSTEMCTL, "start",
                      self.service_instance(instance_name)],
                     skip_output=not capture_output)
 
-        if getattr(self.api.env, 'context', None) in ['ipactl', 'installer']:
-            update_service_list = True
-        else:
-            update_service_list = False
+        update_service_list = getattr(self.api.env, 'context',
+                                      None) in ['ipactl', 'installer']
 
         if wait and self.is_running(instance_name):
             self.wait_for_open_ports(self.service_instance(instance_name))
         super(SystemdService, self).start(
             instance_name,
             update_service_list=update_service_list)
+        logger.debug('Start of %s complete',
+                     self.service_instance(instance_name))
 
-    def restart(self, instance_name="", capture_output=True, wait=True):
-        ipautil.run([paths.SYSTEMCTL, "restart",
-                     self.service_instance(instance_name)],
+    def _restart_base(self, instance_name, operation, capture_output=True,
+                      wait=False):
+
+        ipautil.run([paths.SYSTEMCTL, operation,
+                    self.service_instance(instance_name)],
                     skip_output=not capture_output)
 
         if wait and self.is_running(instance_name):
             self.wait_for_open_ports(self.service_instance(instance_name))
+        logger.debug('Restart of %s complete',
+                     self.service_instance(instance_name))
+
+    def reload_or_restart(self, instance_name="", capture_output=True,
+                          wait=True):
+        self._restart_base(instance_name, "reload-or-restart",
+                           capture_output, wait)
+
+    def restart(self, instance_name="", capture_output=True, wait=True):
+        self._restart_base(instance_name, "restart",
+                           capture_output, wait)
 
     def is_running(self, instance_name="", wait=True):
         instance = self.service_instance(instance_name, 'is-active')
@@ -337,7 +367,7 @@ class SystemdService(PlatformService):
                 return False
             else:
                 svar = self.parse_variables(result.output)
-                if not self.service_instance("") in svar:
+                if self.service_instance("") not in svar:
                     # systemd doesn't show the service
                     return False
         except ipautil.CalledProcessError:
@@ -406,7 +436,7 @@ class SystemdService(PlatformService):
                                    self.service_instance(instance_name))
 
             try:
-                if not ipautil.dir_exists(srv_tgt):
+                if not os.path.isdir(srv_tgt):
                     os.mkdir(srv_tgt)
                     os.chmod(srv_tgt, 0o755)
                 if os.path.exists(srv_lnk):
@@ -441,7 +471,7 @@ class SystemdService(PlatformService):
                                    self.service_instance(instance_name))
 
             try:
-                if ipautil.dir_exists(srv_tgt):
+                if os.path.isdir(srv_tgt):
                     if os.path.islink(srv_lnk):
                         os.unlink(srv_lnk)
                 ipautil.run([paths.SYSTEMCTL, "--system", "daemon-reload"])
@@ -487,9 +517,13 @@ class SystemdService(PlatformService):
 
 # Objects below are expected to be exported by platform module
 
-service = None
-knownservices = None
+def base_service_class_factory(name, api=None):
+    raise NotImplementedError
 
-# System may support more time&date services. FreeIPA supports ntpd only, other
-# services will be disabled during IPA installation
+
+service = base_service_class_factory
+knownservices = KnownServices({})
+
+# System may support more time&date services. FreeIPA supports chrony only.
+# Other services will be disabled during IPA installation
 timedate_services = ['ntpd', 'chronyd']
