@@ -22,27 +22,25 @@
 This module contains default platform-specific implementations of system tasks.
 '''
 
-import pwd
-import grp
+from __future__ import absolute_import
+
+import logging
 
 from pkg_resources import parse_version
 
 from ipaplatform.paths import paths
-from ipapython.ipa_log_manager import log_mgr
 from ipapython import ipautil
 
-log = log_mgr.get_logger(__name__)
+logger = logging.getLogger(__name__)
 
 
 class BaseTaskNamespace(object):
 
-    def restore_context(self, filepath):
-        """
-        Restore SELinux security context on the given filepath.
+    def restore_context(self, filepath, force=False):
+        """Restore SELinux security context on the given filepath.
 
         No return value expected.
         """
-
         raise NotImplementedError()
 
     def backup_hostname(self, fstore, statestore):
@@ -135,9 +133,10 @@ class BaseTaskNamespace(object):
 
         raise NotImplementedError()
 
-    def modify_nsswitch_pam_stack(self, sssd, mkhomedir, statestore):
+    def modify_nsswitch_pam_stack(self, sssd, mkhomedir, statestore,
+                                  sudo=True):
         """
-        If sssd flag is true, configure pam and nsswtich so that SSSD is used
+        If sssd flag is true, configure pam and nsswitch so that SSSD is used
         for retrieving user information and authentication.
 
         Otherwise, configure pam and nsswitch to leverage pure LDAP.
@@ -152,6 +151,13 @@ class BaseTaskNamespace(object):
 
         raise NotImplementedError()
 
+    def is_nosssd_supported(self):
+        """
+        Check if the flag --no-sssd is supported for client install.
+        """
+
+        return True
+
     def backup_auth_configuration(self, path):
         """
         Create backup of access control configuration.
@@ -167,6 +173,11 @@ class BaseTaskNamespace(object):
         """
         raise NotImplementedError()
 
+    def migrate_auth_configuration(self, statestore):
+        """
+        Migrate pam stack configuration to authselect.
+        """
+
     def set_selinux_booleans(self, required_settings, backup_func=None):
         """Set the specified SELinux booleans
 
@@ -186,53 +197,8 @@ class BaseTaskNamespace(object):
 
         raise NotImplementedError()
 
-    def create_system_user(self, name, group, homedir, shell, uid=None, gid=None, comment=None, create_homedir=False):
-        """Create a system user with a corresponding group"""
-        try:
-            grp.getgrnam(group)
-        except KeyError:
-            log.debug('Adding group %s', group)
-            args = [paths.GROUPADD, '-r', group]
-            if gid:
-                args += ['-g', str(gid)]
-            try:
-                ipautil.run(args)
-                log.debug('Done adding group')
-            except ipautil.CalledProcessError as e:
-                log.critical('Failed to add group: %s', e)
-                raise
-        else:
-            log.debug('group %s exists', group)
-
-        try:
-            pwd.getpwnam(name)
-        except KeyError:
-            log.debug('Adding user %s', name)
-            args = [
-                paths.USERADD,
-                '-g', group,
-                '-d', homedir,
-                '-s', shell,
-                '-r', name,
-            ]
-            if uid:
-                args += ['-u', str(uid)]
-            if comment:
-                args += ['-c', comment]
-            if create_homedir:
-                args += ['-m']
-            else:
-                args += ['-M']
-            try:
-                ipautil.run(args)
-                log.debug('Done adding user')
-            except ipautil.CalledProcessError as e:
-                log.critical('Failed to add user: %s', e)
-                raise
-        else:
-            log.debug('user %s exists', name)
-
-    def parse_ipa_version(self, version):
+    @staticmethod
+    def parse_ipa_version(version):
         """
         :param version: textual version
         :return: object implementing proper __cmp__ method for version compare
@@ -251,6 +217,31 @@ class BaseTaskNamespace(object):
         """Configure httpd service to work with IPA"""
         raise NotImplementedError()
 
+    def configure_http_gssproxy_conf(self, ipauser):
+        raise NotImplementedError()
+
     def remove_httpd_service_ipa_conf(self):
         """Remove configuration of httpd service of IPA"""
         raise NotImplementedError()
+
+    def configure_httpd_wsgi_conf(self):
+        """Configure WSGI for correct Python version"""
+        raise NotImplementedError()
+
+    def is_fips_enabled(self):
+        return False
+
+    def add_user_to_group(self, user, group):
+        logger.debug('Adding user %s to group %s', user, group)
+        args = [paths.USERMOD, '-a', '-G', group, user]
+        try:
+            ipautil.run(args)
+            logger.debug('Done adding user to group')
+        except ipautil.CalledProcessError as e:
+            logger.debug('Failed to add user to group: %s', e)
+
+    def setup_httpd_logging(self):
+        raise NotImplementedError()
+
+
+tasks = BaseTaskNamespace()
