@@ -20,3 +20,14 @@
 '''
 This module contains Fedora specific platform files.
 '''
+import sys
+import warnings
+
+NAME = 'fedora'
+
+if sys.version_info < (3, 6):
+    warnings.warn(
+        "Support for Python 2.7 and 3.5 is deprecated. Python version "
+        "3.6 or newer will be required in the next major release.",
+        category=DeprecationWarning
+    )
