@@ -23,11 +23,16 @@ in Fedora-based systems.
 '''
 
 # Fallback to default path definitions
+from __future__ import absolute_import
+
 from ipaplatform.redhat.paths import RedHatPathNamespace
 
 
 class FedoraPathNamespace(RedHatPathNamespace):
-    pass
+    HTTPD_IPA_WSGI_MODULES_CONF = (
+        "/etc/httpd/conf.modules.d/02-ipa-wsgi.conf"
+    )
+    NAMED_CRYPTO_POLICY_FILE = "/etc/crypto-policies/back-ends/bind.config"
 
 
 paths = FedoraPathNamespace()
