@@ -22,14 +22,21 @@
 Handles common operations like option parsing and logging
 """
 
+import logging
 import sys
 import os
 import traceback
-from optparse import OptionGroup
+from optparse import OptionGroup  # pylint: disable=deprecated-module
 
 from ipapython import version
 from ipapython import config
-from ipapython import ipa_log_manager
+from ipapython.ipa_log_manager import standard_logging_setup
+
+SUCCESS = 0
+SERVER_INSTALL_ERROR = 1
+SERVER_NOT_CONFIGURED = 2
+
+logger = logging.getLogger(__name__)
 
 
 class ScriptError(Exception):
@@ -86,8 +93,8 @@ class AdminTool(object):
     log_file_name = None
     usage = None
     description = None
+    ignore_return_codes = ()
 
-    log = None
     _option_parsers = dict()
 
     @classmethod
@@ -171,9 +178,13 @@ class AdminTool(object):
             self.setup_logging()
             return_value = self.run()
         except BaseException as exception:
+            if isinstance(exception, ScriptError):
+                # pylint: disable=no-member
+                if exception.rval and exception.rval > return_value:
+                    return_value = exception.rval  # pylint: disable=no-member
             traceback = sys.exc_info()[2]
             error_message, return_value = self.handle_error(exception)
-            if return_value:
+            if return_value and return_value not in self.ignore_return_codes:
                 self.log_failure(error_message, return_value, exception,
                     traceback)
                 return return_value
@@ -201,7 +212,6 @@ class AdminTool(object):
 
         Any options that might be asked for should also be validated here.
         """
-        pass
 
     def setup_logging(self, log_file_mode='w'):
         """Set up logging
@@ -225,11 +235,18 @@ class AdminTool(object):
         - a plain print for things that should not be log (for example,
             interactive prompting)
 
-        To log, use `self.log.info()`, `self.log.warning()`, etc.
+        To log, use a module-level logger.
 
         Logging to file is only set up after option validation and prompting;
         before that, all output will go to the console only.
         """
+        root_logger = logging.getLogger()
+        for handler in root_logger.handlers:
+            if (isinstance(handler, logging.StreamHandler) and
+                    handler.stream is sys.stderr):  # pylint: disable=no-member
+                root_logger.removeHandler(handler)
+                break
+
         self._setup_logging(log_file_mode=log_file_mode)
 
     def _setup_logging(self, log_file_mode='w', no_file=False):
@@ -250,14 +267,13 @@ class AdminTool(object):
                 verbose = False
             else:
                 verbose = True
-        ipa_log_manager.standard_logging_setup(
+        standard_logging_setup(
             log_file_name, console_format=console_format,
             filemode=log_file_mode, debug=debug, verbose=verbose)
-        self.log = ipa_log_manager.log_mgr.get_logger(self)
         if log_file_name:
-            self.log.debug('Logging to %s' % log_file_name)
+            logger.debug('Logging to %s', log_file_name)
         elif not no_file:
-            self.log.debug('Not logging to a file')
+            logger.debug('Not logging to a file')
 
 
     def handle_error(self, exception):
@@ -282,20 +298,22 @@ class AdminTool(object):
         assumed to have run successfully, and the return value is used as the
         SystemExit code.
         """
-        self.log.debug('%s was invoked with arguments %s and options: %s',
-                self.command_name, self.args, self.safe_options)
-        self.log.debug('IPA version %s' % version.VENDOR_VERSION)
+        logger.debug('%s was invoked with arguments %s and options: %s',
+                     self.command_name, self.args, self.safe_options)
+        logger.debug('IPA version %s', version.VENDOR_VERSION)
 
     def log_failure(self, error_message, return_value, exception, backtrace):
-        self.log.debug(''.join(traceback.format_tb(backtrace)))
-        self.log.debug('The %s command failed, exception: %s: %s',
-            self.command_name, type(exception).__name__, exception)
+        logger.debug('%s', ''.join(traceback.format_tb(backtrace)))
+        logger.debug('The %s command failed, exception: %s: %s',
+                     self.command_name, type(exception).__name__, exception)
         if error_message:
-            self.log.error(error_message)
+            logger.error('%s', error_message)
         message = "The %s command failed." % self.command_name
-        if self.log_file_name:
+        if self.log_file_name and return_value != 2:
+            # magic value because this is common between server and client
+            # but imports are not straigthforward
             message += " See %s for more information" % self.log_file_name
-        self.log.error(message)
+        logger.error('%s', message)
 
     def log_success(self):
-        self.log.info('The %s command was successful', self.command_name)
+        logger.info('The %s command was successful', self.command_name)
