@@ -17,19 +17,27 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
+import copy
+import logging
+import operator
+import random
+
 import dns.name
 import dns.exception
 import dns.resolver
-import copy
+import dns.rdataclass
+import dns.rdatatype
+
 
 import six
 
 from ipapython.ipautil import UnsafeIPAddress
-from ipapython.ipa_log_manager import root_logger
 
 if six.PY3:
     unicode = str
 
+logger = logging.getLogger(__name__)
+
 
 @six.python_2_unicode_compatible
 class DNSName(dns.name.Name):
@@ -69,9 +77,14 @@ class DNSName(dns.name.Name):
     def __str__(self):
         return self.to_unicode()
 
-    def ToASCII(self):
-        #method named by RFC 3490 and python standard library
-        return self.to_text().decode('ascii')  # must be unicode string
+    # method ToASCII named by RFC 3490 and python standard library
+    if six.PY2:
+        def ToASCII(self):
+            # must be unicode string in Py2
+            return self.to_text().decode('ascii')
+    else:
+        def ToASCII(self):
+            return self.to_text()
 
     def canonicalize(self):
         return DNSName(super(DNSName, self).canonicalize())
@@ -303,18 +316,19 @@ def resolve_rrsets(fqdn, rdtypes):
     for rdtype in rdtypes:
         try:
             answer = dns.resolver.query(fqdn, rdtype)
-            root_logger.debug('found %d %s records for %s: %s',
-                              len(answer), rdtype, fqdn, ' '.join(
-                                  str(rr) for rr in answer))
+            logger.debug('found %d %s records for %s: %s',
+                         len(answer),
+                         rdtype,
+                         fqdn,
+                         ' '.join(str(rr) for rr in answer))
             rrsets.append(answer.rrset)
         except dns.resolver.NXDOMAIN as ex:
-            root_logger.debug(ex)
+            logger.debug('%s', ex)
             break  # no such FQDN, do not iterate
         except dns.resolver.NoAnswer as ex:
-            root_logger.debug(ex)  # record type does not exist for given FQDN
+            logger.debug('%s', ex)  # record type does not exist for given FQDN
         except dns.exception.DNSException as ex:
-            root_logger.error('DNS query for %s %s failed: %s',
-                              fqdn, rdtype, ex)
+            logger.error('DNS query for %s %s failed: %s', fqdn, rdtype, ex)
             raise
 
     return rrsets
@@ -333,7 +347,7 @@ def resolve_ip_addresses(fqdn):
 
 
 def check_zone_overlap(zone, raise_on_error=True):
-    root_logger.info("Checking DNS domain %s, please wait ..." % zone)
+    logger.info("Checking DNS domain %s, please wait ...", zone)
     if not isinstance(zone, DNSName):
         zone = DNSName(zone).make_absolute()
 
@@ -349,18 +363,103 @@ def check_zone_overlap(zone, raise_on_error=True):
         if raise_on_error:
             raise ValueError(msg)
         else:
-            root_logger.warning(msg)
+            logger.warning('%s', msg)
             return
 
     if containing_zone == zone:
         try:
             ns = [ans.to_text() for ans in dns.resolver.query(zone, 'NS')]
         except dns.exception.DNSException as e:
-            root_logger.debug("Failed to resolve nameserver(s) for domain"
-                              " {0}: {1}".format(zone, e))
+            logger.debug("Failed to resolve nameserver(s) for domain %s: %s",
+                         zone, e)
             ns = []
 
         msg = u"DNS zone {0} already exists in DNS".format(zone)
         if ns:
             msg += u" and is handled by server(s): {0}".format(', '.join(ns))
         raise ValueError(msg)
+
+
+def _mix_weight(records):
+    """Weighted population sorting for records with same priority
+    """
+    # trivial case
+    if len(records) <= 1:
+        return records
+
+    # Optimization for common case: If all weights are the same (e.g. 0),
+    # just shuffle the records, which is about four times faster.
+    if all(rr.weight == records[0].weight for rr in records):
+        random.shuffle(records)
+        return records
+
+    noweight = 0.01  # give records with 0 weight a small chance
+    result = []
+    records = set(records)
+    while len(records) > 1:
+        # Compute the sum of the weights of those RRs. Then choose a
+        # uniform random number between 0 and the sum computed (inclusive).
+        urn = random.uniform(0, sum(rr.weight or noweight for rr in records))
+        # Select the RR whose running sum value is the first in the selected
+        # order which is greater than or equal to the random number selected.
+        acc = 0.
+        for rr in records.copy():
+            acc += rr.weight or noweight
+            if acc >= urn:
+                records.remove(rr)
+                result.append(rr)
+    if records:
+        result.append(records.pop())
+    return result
+
+
+def sort_prio_weight(records):
+    """RFC 2782 sorting algorithm for SRV and URI records
+
+    RFC 2782 defines a sorting algorithms for SRV records, that is also used
+    for sorting URI records. Records are sorted by priority and than randomly
+    shuffled according to weight.
+
+    This implementation also removes duplicate entries.
+    """
+    # order records by priority
+    records = sorted(records, key=operator.attrgetter("priority"))
+
+    # remove duplicate entries
+    uniquerecords = []
+    seen = set()
+    for rr in records:
+        # A SRV record has target and port, URI just has target.
+        target = (rr.target, getattr(rr, "port", None))
+        if target not in seen:
+            uniquerecords.append(rr)
+            seen.add(target)
+
+    # weighted randomization of entries with same priority
+    result = []
+    sameprio = []
+    for rr in uniquerecords:
+        # add all items with same priority in a bucket
+        if not sameprio or sameprio[0].priority == rr.priority:
+            sameprio.append(rr)
+        else:
+            # got different priority, shuffle bucket
+            result.extend(_mix_weight(sameprio))
+            # start a new priority list
+            sameprio = [rr]
+    # add last batch of records with same priority
+    if sameprio:
+        result.extend(_mix_weight(sameprio))
+    return result
+
+
+def query_srv(qname, resolver=None, **kwargs):
+    """Query SRV records and sort reply according to RFC 2782
+
+    :param qname: query name, _service._proto.domain.
+    :return: list of dns.rdtypes.IN.SRV.SRV instances
+    """
+    if resolver is None:
+        resolver = dns.resolver
+    answer = resolver.query(qname, rdtype=dns.rdatatype.SRV, **kwargs)
+    return sort_prio_weight(answer)
