@@ -18,10 +18,10 @@
 #
 
 # The full version including strings
-VERSION="4.4.4"
+VERSION="4.7.2"
 
 # A fuller version including the vendor tag (e.g. 3.3.3-34.fc20)
-VENDOR_VERSION="4.4.4"
+VENDOR_VERSION="4.7.2"
 
 
 # Just the numeric portion of the version so one can do direct numeric
@@ -41,11 +41,11 @@ VENDOR_VERSION="4.4.4"
 #   IPA 3.2.1:  NUM_VERSION=30201
 #   IPA 3.2.99: NUM_VERSION=30299 (development version)
 #   IPA 3.3.0:  NUM_VERSION=30300
-NUM_VERSION=40404
+NUM_VERSION=40702
 
 
 # The version of the API.
-API_VERSION=u'2.215'
+API_VERSION=u'2.230'
 
 
 DEFAULT_PLUGINS = frozenset(l.strip() for l in """
@@ -66,6 +66,7 @@ automember_default_group_set/1
 automember_default_group_show/1
 automember_del/1
 automember_find/1
+automember_find_orphans/1
 automember_mod/1
 automember_rebuild/1
 automember_remove_condition/1
@@ -125,6 +126,19 @@ cert_request/1
 cert_revoke/1
 cert_show/1
 cert_status/1
+certmap/1
+certmap_match/1
+certmapconfig/1
+certmapconfig_mod/1
+certmapconfig_show/1
+certmaprule/1
+certmaprule_add/1
+certmaprule_del/1
+certmaprule_disable/1
+certmaprule_enable/1
+certmaprule_find/1
+certmaprule_mod/1
+certmaprule_show/1
 certprofile/1
 certprofile_del/1
 certprofile_find/1
@@ -215,6 +229,7 @@ dnssrvrecord/1
 dnssshfprecord/1
 dnstlsarecord/1
 dnstxtrecord/1
+dnsurirecord/1
 dnszone/1
 dnszone_add/1
 dnszone_add_permission/1
@@ -374,7 +389,7 @@ permission_remove_member/1
 permission_show/1
 ping/1
 pkinit/1
-pkinit_anonymous/1
+pkinit_status/1
 plugins/1
 privilege/1
 privilege_add/1
@@ -478,11 +493,17 @@ sidgen_was_run/1
 stageuser/1
 stageuser_activate/1
 stageuser_add/1
+stageuser_add_cert/1
+stageuser_add_certmapdata/1
 stageuser_add_manager/1
+stageuser_add_principal/1
 stageuser_del/1
 stageuser_find/1
 stageuser_mod/1
+stageuser_remove_cert/1
+stageuser_remove_certmapdata/1
 stageuser_remove_manager/1
+stageuser_remove_principal/1
 stageuser_show/1
 sudocmd/1
 sudocmd_add/1
@@ -558,6 +579,7 @@ trustdomain_mod/1
 user/1
 user_add/1
 user_add_cert/1
+user_add_certmapdata/1
 user_add_manager/1
 user_add_principal/1
 user_del/1
@@ -566,6 +588,7 @@ user_enable/1
 user_find/1
 user_mod/1
 user_remove_cert/1
+user_remove_certmapdata/1
 user_remove_manager/1
 user_remove_principal/1
 user_show/1
@@ -593,4 +616,5 @@ vaultcontainer_add_owner/1
 vaultcontainer_del/1
 vaultcontainer_remove_owner/1
 vaultcontainer_show/1
+whoami/1
 """.strip().splitlines())
