@@ -17,31 +17,33 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
+from __future__ import absolute_import
+
+import logging
 import os
 import pwd
 import shutil
 import tempfile
+import base64
 
-from six.moves.configparser import ConfigParser
+import six
+# pylint: disable=import-error
+from six.moves.configparser import RawConfigParser
+# pylint: enable=import-error
 
 from ipalib import api
 from ipalib import x509
-from ipaplatform import services
-from ipaplatform.constants import constants
 from ipaplatform.paths import paths
-from ipapython import certdb
+from ipapython import directivesetter
 from ipapython import ipautil
 from ipapython.dn import DN
-from ipapython.admintool import ScriptError
-from ipaserver.install import certs
 from ipaserver.install import cainstance
 from ipaserver.install import installutils
 from ipaserver.install import ldapupdate
-from ipaserver.install import service
-from ipaserver.install.dogtaginstance import (export_kra_agent_pem,
-                                              DogtagInstance)
+from ipaserver.install.dogtaginstance import DogtagInstance
 from ipaserver.plugins import ldap2
-from ipapython.ipa_log_manager import log_mgr
+
+logger = logging.getLogger(__name__)
 
 # When IPA is installed with DNS support, this CNAME should hold all IPA
 # replicas with KRA configured
@@ -53,7 +55,6 @@ ADMIN_GROUPS = [
     'Security Domain Administrators'
 ]
 
-LDAPMOD_ERR_ALREADY_EXISTS = 68
 
 class KRAInstance(DogtagInstance):
     """
@@ -64,38 +65,42 @@ class KRAInstance(DogtagInstance):
     be the same for both the CA and KRA.
     """
 
-    tracking_reqs = (('auditSigningCert cert-pki-kra', None),
-                     ('transportCert cert-pki-kra', None),
-                     ('storageCert cert-pki-kra', None))
+    tracking_reqs = ('auditSigningCert cert-pki-kra',
+                     'transportCert cert-pki-kra',
+                     'storageCert cert-pki-kra')
 
     def __init__(self, realm):
         super(KRAInstance, self).__init__(
             realm=realm,
             subsystem="KRA",
             service_desc="KRA server",
+            config=paths.KRA_CS_CFG_PATH,
         )
 
         self.basedn = DN(('o', 'kra'), ('o', 'ipaca'))
-        self.log = log_mgr.get_logger(self)
 
     def configure_instance(self, realm_name, host_name, dm_password,
                            admin_password, pkcs12_info=None, master_host=None,
-                           subject_base=None):
+                           subject_base=None, subject=None,
+                           promote=False):
         """Create a KRA instance.
 
            To create a clone, pass in pkcs12_info.
         """
         self.fqdn = host_name
         self.dm_password = dm_password
+        self.admin_groups = ADMIN_GROUPS
         self.admin_password = admin_password
         self.pkcs12_info = pkcs12_info
-        if self.pkcs12_info is not None:
+        if self.pkcs12_info is not None or promote:
             self.clone = True
         self.master_host = master_host
-        if subject_base is None:
-            self.subject_base = DN(('O', self.realm))
-        else:
-            self.subject_base = subject_base
+
+        self.subject_base = \
+            subject_base or installutils.default_subject_base(realm_name)
+        self.subject = \
+            subject or installutils.default_ca_subject_dn(self.subject_base)
+
         self.realm = realm_name
         self.suffix = ipautil.realm_to_suffix(realm_name)
 
@@ -104,26 +109,39 @@ class KRAInstance(DogtagInstance):
             raise RuntimeError(
                 "KRA already installed.")
         # Confirm that a Dogtag 10 CA instance already exists
-        ca = cainstance.CAInstance(api.env.realm, certs.NSS_DIR)
+        ca = cainstance.CAInstance(self.realm)
         if not ca.is_installed():
             raise RuntimeError(
                 "KRA configuration failed.  "
                 "A Dogtag CA must be installed first")
 
+        if promote:
+            self.step("creating ACIs for admin", self.add_ipaca_aci)
+            self.step("creating installation admin user", self.setup_admin)
         self.step("configuring KRA instance", self.__spawn_instance)
         if not self.clone:
             self.step("create KRA agent",
                       self.__create_kra_agent)
+        if promote:
+            self.step("destroying installation admin user",
+                      self.teardown_admin)
+        self.step("enabling ephemeral requests", self.enable_ephemeral)
         self.step("restarting KRA", self.restart_instance)
         self.step("configure certmonger for renewals",
                   self.configure_certmonger_renewal)
         self.step("configure certificate renewals", self.configure_renewal)
         self.step("configure HTTP to proxy connections",
                   self.http_proxy)
-        self.step("add vault container", self.__add_vault_container)
+        if not self.clone:
+            self.step("add vault container", self.__add_vault_container)
         self.step("apply LDAP updates", self.__apply_updates)
 
-        self.start_creation(runtime=126)
+        self.step("enabling KRA instance", self.__enable_instance)
+
+        try:
+            self.start_creation(runtime=120)
+        finally:
+            self.clean_pkispawn_files()
 
     def __spawn_instance(self):
         """
@@ -135,11 +153,18 @@ class KRAInstance(DogtagInstance):
         # Create an empty and secured file
         (cfg_fd, cfg_file) = tempfile.mkstemp()
         os.close(cfg_fd)
-        pent = pwd.getpwnam(constants.PKI_USER)
+        pent = pwd.getpwnam(self.service_user)
         os.chown(cfg_file, pent.pw_uid, pent.pw_gid)
+        self.tmp_agent_db = tempfile.mkdtemp(
+                prefix="tmp-", dir=paths.VAR_LIB_IPA)
+        tmp_agent_pwd = ipautil.ipa_generate_password()
+
+        # Create a temporary file for the admin PKCS #12 file
+        (admin_p12_fd, admin_p12_file) = tempfile.mkstemp()
+        os.close(admin_p12_fd)
 
         # Create KRA configuration
-        config = ConfigParser()
+        config = RawConfigParser()
         config.optionxform = str
         config.add_section("KRA")
 
@@ -159,9 +184,9 @@ class KRAInstance(DogtagInstance):
         config.set("KRA", "pki_backup_password", self.admin_password)
 
         # Client security database
-        config.set("KRA", "pki_client_database_dir", self.agent_db)
-        config.set("KRA", "pki_client_database_password", self.admin_password)
-        config.set("KRA", "pki_client_database_purge", "False")
+        config.set("KRA", "pki_client_database_dir", self.tmp_agent_db)
+        config.set("KRA", "pki_client_database_password", tmp_agent_pwd)
+        config.set("KRA", "pki_client_database_purge", "True")
         config.set("KRA", "pki_client_pkcs12_password", self.admin_password)
 
         # Administrator
@@ -172,14 +197,13 @@ class KRAInstance(DogtagInstance):
         config.set("KRA", "pki_admin_nickname", "ipa-ca-agent")
         config.set("KRA", "pki_admin_subject_dn",
                    str(DN(('cn', 'ipa-ca-agent'), self.subject_base)))
-        config.set("KRA", "pki_import_admin_cert", "True")
-        config.set("KRA", "pki_admin_cert_file", paths.ADMIN_CERT_PATH)
-        config.set("KRA", "pki_client_admin_cert_p12", paths.DOGTAG_ADMIN_P12)
+        config.set("KRA", "pki_import_admin_cert", "False")
+        config.set("KRA", "pki_client_admin_cert_p12", admin_p12_file)
 
         # Directory server
         config.set("KRA", "pki_ds_ldap_port", "389")
         config.set("KRA", "pki_ds_password", self.dm_password)
-        config.set("KRA", "pki_ds_base_dn", self.basedn)
+        config.set("KRA", "pki_ds_base_dn", six.text_type(self.basedn))
         config.set("KRA", "pki_ds_database", "ipaca")
         config.set("KRA", "pki_ds_create_new_db", "False")
 
@@ -188,7 +212,7 @@ class KRAInstance(DogtagInstance):
         # Certificate subject DNs
         config.set("KRA", "pki_subsystem_subject_dn",
                    str(DN(('cn', 'CA Subsystem'), self.subject_base)))
-        config.set("KRA", "pki_ssl_server_subject_dn",
+        config.set("KRA", "pki_sslserver_subject_dn",
                    str(DN(('cn', self.fqdn), self.subject_base)))
         config.set("KRA", "pki_audit_signing_subject_dn",
                    str(DN(('cn', 'KRA Audit'), self.subject_base)))
@@ -204,7 +228,7 @@ class KRAInstance(DogtagInstance):
         # the ca certs.
         config.set("KRA", "pki_subsystem_nickname",
                    "subsystemCert cert-pki-ca")
-        config.set("KRA", "pki_ssl_server_nickname",
+        config.set("KRA", "pki_sslserver_nickname",
                    "Server-Cert cert-pki-ca")
         config.set("KRA", "pki_audit_signing_nickname",
                    "auditSigningCert cert-pki-kra")
@@ -221,16 +245,24 @@ class KRAInstance(DogtagInstance):
             "KRA", "pki_share_dbuser_dn",
             str(DN(('uid', 'pkidbuser'), ('ou', 'people'), ('o', 'ipaca'))))
 
+        if not (os.path.isdir(paths.PKI_TOMCAT_ALIAS_DIR) and
+                os.path.isfile(paths.PKI_TOMCAT_PASSWORD_CONF)):
+            # generate pin which we know can be used for FIPS NSS database
+            pki_pin = ipautil.ipa_generate_password()
+            config.set("KRA", "pki_pin", pki_pin)
+        else:
+            pki_pin = None
+
         _p12_tmpfile_handle, p12_tmpfile_name = tempfile.mkstemp(dir=paths.TMP)
 
         if self.clone:
             krafile = self.pkcs12_info[0]
             shutil.copy(krafile, p12_tmpfile_name)
-            pent = pwd.getpwnam(constants.PKI_USER)
+            pent = pwd.getpwnam(self.service_user)
             os.chown(p12_tmpfile_name, pent.pw_uid, pent.pw_gid)
 
             # Security domain registration
-            config.set("KRA", "pki_security_domain_hostname", self.master_host)
+            config.set("KRA", "pki_security_domain_hostname", self.fqdn)
             config.set("KRA", "pki_security_domain_https_port", "443")
             config.set("KRA", "pki_security_domain_user", self.admin_user)
             config.set("KRA", "pki_security_domain_password",
@@ -246,25 +278,35 @@ class KRAInstance(DogtagInstance):
                 "https://%s" % ipautil.format_netloc(self.master_host, 443))
         else:
             # the admin cert file is needed for the first instance of KRA
-            cert = DogtagInstance.get_admin_cert(self)
-            with open(paths.ADMIN_CERT_PATH, "w") as admin_path:
-                admin_path.write(cert)
+            cert = self.get_admin_cert()
+            # First make sure that the directory exists
+            parentdir = os.path.dirname(paths.ADMIN_CERT_PATH)
+            if not os.path.exists(parentdir):
+                os.makedirs(parentdir)
+            with open(paths.ADMIN_CERT_PATH, "wb") as admin_path:
+                admin_path.write(
+                    base64.b64encode(cert.public_bytes(x509.Encoding.DER))
+                )
 
         # Generate configuration file
-        with open(cfg_file, "wb") as f:
+        with open(cfg_file, "w") as f:
             config.write(f)
 
         try:
-            DogtagInstance.spawn_instance(self, cfg_file)
+            DogtagInstance.spawn_instance(
+                self, cfg_file,
+                nolog_list=(self.dm_password,
+                            self.admin_password,
+                            pki_pin,
+                            tmp_agent_pwd)
+            )
         finally:
             os.remove(p12_tmpfile_name)
             os.remove(cfg_file)
+            os.remove(admin_p12_file)
 
         shutil.move(paths.KRA_BACKUP_KEYS_P12, paths.KRACERT_P12)
-
-        export_kra_agent_pem()
-
-        self.log.debug("completed creating KRA instance")
+        logger.debug("completed creating KRA instance")
 
     def __create_kra_agent(self):
         """
@@ -272,18 +314,14 @@ class KRAInstance(DogtagInstance):
         the appropriate groups for accessing KRA services.
         """
 
-        # get ipaCert certificate
-        with certdb.NSSDatabase(paths.HTTPD_ALIAS_DIR) as ipa_nssdb:
-           cert_data = ipa_nssdb.get_cert("ipaCert")
-        cert = x509.load_certificate(cert_data, x509.DER)
+        # get RA agent certificate
+        cert = x509.load_certificate_from_file(paths.RA_AGENT_PEM)
 
         # connect to KRA database
-        server_id = installutils.realm_to_serverid(api.env.realm)
-        dogtag_uri = 'ldapi://%%2fvar%%2frun%%2fslapd-%s.socket' % server_id
-        conn = ldap2.ldap2(api, ldap_uri=dogtag_uri)
+        conn = ldap2.ldap2(api)
         conn.connect(autobind=True)
 
-        # create ipakra user with ipaCert certificate
+        # create ipakra user with RA agent certificate
         user_dn = DN(('uid', "ipakra"), ('ou', 'people'), self.basedn)
         entry = conn.make_entry(
             user_dn,
@@ -293,10 +331,10 @@ class KRAInstance(DogtagInstance):
             sn=["IPA KRA User"],
             cn=["IPA KRA User"],
             usertype=["undefined"],
-            userCertificate=[cert_data],
+            userCertificate=[cert],
             description=['2;%s;%s;%s' % (
                 cert.serial_number,
-                DN(('CN', 'Certificate Authority'), self.subject_base),
+                DN(self.subject),
                 DN(('CN', 'IPA RA'), self.subject_base))])
         conn.add_entry(entry)
 
@@ -308,18 +346,8 @@ class KRAInstance(DogtagInstance):
         conn.disconnect()
 
     def __add_vault_container(self):
-        try:
-            self._ldap_mod('vault.ldif', {'SUFFIX': self.suffix},
-                           raise_on_err=True)
-        except ipautil.CalledProcessError as e:
-            if e.returncode == LDAPMOD_ERR_ALREADY_EXISTS:
-                self.log.debug("Vault container already exists")
-            else:
-                self.log.error("Failed to add vault container: {0}".format(e))
-        finally:
-            # we need to disconnect from LDAP, because _ldap_mod() makes the
-            # connection without actually using it
-            self.ldap_disconnect()
+        self._ldap_mod(
+            'vault.ldif', {'SUFFIX': self.suffix}, raise_on_err=True)
 
     def __apply_updates(self):
         sub_dict = {
@@ -330,8 +358,20 @@ class KRAInstance(DogtagInstance):
                                    sub_dict=sub_dict)
         ld.update([os.path.join(paths.UPDATES_DIR, '40-vault.update')])
 
-    @staticmethod
-    def update_cert_config(nickname, cert):
+    def enable_ephemeral(self):
+        """
+        Enable ephemeral KRA requests to reduce the number of LDAP
+        write operations.
+        """
+        with installutils.stopped_service('pki-tomcatd', 'pki-tomcat'):
+            directivesetter.set_directive(
+                self.config,
+                'kra.ephemeralRequests',
+                'true', quotes=False, separator='=')
+
+        # A restart is required
+
+    def update_cert_config(self, nickname, cert):
         """
         When renewing a KRA subsystem certificate the configuration file
         needs to get the new certificate as well.
@@ -349,101 +389,8 @@ class KRAInstance(DogtagInstance):
             'Server-Cert cert-pki-ca': 'kra.sslserver.cert'}
 
         if nickname in directives:
-            DogtagInstance.update_cert_cs_cfg(
-                directives[nickname], cert, paths.KRA_CS_CFG_PATH)
+            super(KRAInstance, self).update_cert_cs_cfg(
+                directives[nickname], cert)
 
     def __enable_instance(self):
-        self.ldap_enable('KRA', self.fqdn, None, self.suffix)
-
-    def configure_replica(self, host_name, master_host, dm_password,
-                          kra_cert_bundle=None, subject_base=None):
-        """Create a KRA instance.
-
-           To create a clone, pass in pkcs12_info.
-        """
-        self.fqdn = host_name
-        self.dm_password = dm_password
-        self.master_host = master_host
-        if subject_base is None:
-            self.subject_base = DN(('O', self.realm))
-        else:
-            self.subject_base = subject_base
-        self.suffix = ipautil.realm_to_suffix(self.realm)
-
-        self.pkcs12_info = kra_cert_bundle
-        self.clone = True
-        self.admin_groups = ADMIN_GROUPS
-
-        # Confirm that a KRA does not already exist
-        if self.is_installed():
-            raise RuntimeError(
-                "KRA already installed.")
-        # Confirm that a Dogtag 10 CA instance already exists
-        ca = cainstance.CAInstance(self.realm, certs.NSS_DIR)
-        if not ca.is_installed():
-            raise RuntimeError(
-                "KRA configuration failed.  "
-                "A Dogtag CA must be installed first")
-
-        self.step("creating installation admin user", self.setup_admin)
-        self.step("configuring KRA instance", self.__spawn_instance)
-        self.step("destroying installation admin user", self.teardown_admin)
-        self.step("restarting KRA", self.restart_instance)
-        self.step("configure certmonger for renewals",
-                  self.configure_certmonger_renewal)
-        self.step("configure certificate renewals", self.configure_renewal)
-        self.step("add vault container", self.__add_vault_container)
-
-        self.step("enabling KRA instance", self.__enable_instance)
-
-        self.start_creation(runtime=126)
-
-
-def install_replica_kra(config, postinstall=False):
-    """
-    Install a KRA on a replica.
-
-    There are two modes of doing this controlled:
-      - While the replica is being installed
-      - Post-replica installation
-
-    config is a ReplicaConfig object
-
-    Returns a KRA instance
-    """
-    # note that the cacert.p12 file is regenerated during the
-    # ipa-replica-prepare process and should include all the certs
-    # for the CA and KRA
-    krafile = config.dir + "/cacert.p12"
-
-    if not ipautil.file_exists(krafile):
-        raise RuntimeError(
-            "Unable to clone KRA."
-            "  cacert.p12 file not found in replica file")
-
-    _kra = KRAInstance(config.realm_name)
-    _kra.dm_password = config.dirman_password
-    _kra.subject_base = config.subject_base
-    if _kra.is_installed():
-        raise ScriptError("A KRA is already configured on this system.")
-
-    _kra.configure_instance(config.realm_name, config.host_name,
-                            config.dirman_password, config.dirman_password,
-                            pkcs12_info=(krafile,),
-                            master_host=config.master_host_name,
-                            subject_base=config.subject_base)
-
-    # Restart httpd since we changed it's config and added ipa-pki-proxy.conf
-    if postinstall:
-        services.knownservices.httpd.restart()
-
-    # The dogtag DS instance needs to be restarted after installation.
-    # The procedure for this is: stop dogtag, stop DS, start DS, start
-    # dogtag
-
-    service.print_msg("Restarting the directory and KRA servers")
-    _kra.stop('pki-tomcat')
-    services.knownservices.dirsrv.restart()
-    _kra.start('pki-tomcat')
-
-    return _kra
+        self.ldap_configure('KRA', self.fqdn, None, self.suffix)
