@@ -17,11 +17,16 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import logging
+
 from ipalib import Registry, errors
 from ipalib import Updater
 from ipapython.dn import DN
-from ipapython.ipa_log_manager import root_logger
 from ipaserver.install import sysupgrade
+from ipaserver.install.adtrustinstance import (
+    ADTRUSTInstance, map_Guests_to_nobody)
+
+logger = logging.getLogger(__name__)
 
 register = Registry()
 
@@ -40,11 +45,12 @@ class update_default_range(Updater):
         dn = DN(self.api.env.container_ranges, self.api.env.basedn)
         search_filter = "objectclass=ipaDomainIDRange"
         try:
-            (entries, truncated) = ldap.find_entries(search_filter, [], dn)
+            ldap.find_entries(search_filter, [], dn)
         except errors.NotFound:
             pass
         else:
-            root_logger.debug("default_range: ipaDomainIDRange entry found, skip plugin")
+            logger.debug("default_range: ipaDomainIDRange entry found, skip "
+                         "plugin")
             return False, []
 
         dn = DN(('cn', 'admins'), self.api.env.container_group,
@@ -52,8 +58,8 @@ class update_default_range(Updater):
         try:
             admins_entry = ldap.get_entry(dn, ['gidnumber'])
         except errors.NotFound:
-            root_logger.error("default_range: No local ID range and no admins "
-                              "group found. Cannot create default ID range")
+            logger.error("default_range: No local ID range and no admins "
+                         "group found. Cannot create default ID range")
             return False, []
 
         id_range_base_id = admins_entry['gidnumber'][0]
@@ -85,10 +91,10 @@ class update_default_range(Updater):
         search_filter = "objectclass=dnaSharedConfig"
         attrs = ['dnaHostname', 'dnaRemainingValues']
         try:
-            (entries, truncated) = ldap.find_entries(search_filter, attrs, dn)
+            (entries, _truncated) = ldap.find_entries(search_filter, attrs, dn)
         except errors.NotFound:
-            root_logger.warning("default_range: no dnaSharedConfig object found. "
-                                "Cannot check default range size.")
+            logger.warning("default_range: no dnaSharedConfig object found. "
+                           "Cannot check default range size.")
         else:
             masters = set()
             remaining_values_sum = 0
@@ -100,8 +106,9 @@ class update_default_range(Updater):
                 try:
                     remaining_values = int(remaining_values)
                 except ValueError:
-                    root_logger.warning("default_range: could not parse "
-                        "remaining values from '%s'", remaining_values)
+                    logger.warning("default_range: could not parse "
+                                   "remaining values from '%s'",
+                                   remaining_values)
                     continue
                 else:
                     remaining_values_sum += remaining_values
@@ -117,7 +124,7 @@ class update_default_range(Updater):
                        '  RANGE_SIZE = (--idmax) - (--idstart) + 1'
                       ]
 
-                root_logger.error("default_range: %s", "\n".join(msg))
+                logger.error("default_range: %s", "\n".join(msg))
 
         return False, [update]
 
@@ -145,7 +152,7 @@ class update_default_trust_view(Updater):
 
         # First, see if trusts are enabled on the server
         if not self.api.Command.adtrust_is_enabled()['result']:
-            self.log.debug('AD Trusts are not enabled on this server')
+            logger.debug('AD Trusts are not enabled on this server')
             return False, []
 
         # Second, make sure the Default Trust View does not exist yet
@@ -154,7 +161,7 @@ class update_default_trust_view(Updater):
         except errors.NotFound:
             pass
         else:
-            self.log.debug('Default Trust View already present on this server')
+            logger.debug('Default Trust View already present on this server')
             return False, []
 
         # We have a server with AD trust support without Default Trust View.
@@ -199,7 +206,7 @@ class update_sigden_extdom_broken_config(Updater):
             try:
                 entry = ldap.get_entry(dn, attrs_list=[basedn_attr])
             except errors.NotFound:
-                self.log.debug("configuration for %s not found, skipping", dn)
+                logger.debug("configuration for %s not found, skipping", dn)
             else:
                 configured_suffix = entry.single_value.get(basedn_attr)
                 if configured_suffix is None:
@@ -211,19 +218,19 @@ class update_sigden_extdom_broken_config(Updater):
                 elif configured_suffix == "$SUFFIX":
                     # configured value is wrong, fix it
                     entry.single_value[basedn_attr] = str(self.api.env.basedn)
-                    self.log.debug("updating attribute %s of %s to correct "
-                                   "value %s", basedn_attr, dn,
-                                   self.api.env.basedn)
+                    logger.debug("updating attribute %s of %s to correct "
+                                 "value %s",
+                                 basedn_attr, dn, self.api.env.basedn)
                     ldap.update_entry(entry)
                     modified = True
                 else:
-                    self.log.debug("configured basedn for %s is okay", dn)
+                    logger.debug("configured basedn for %s is okay", dn)
 
         return modified
 
     def execute(self, **options):
         if sysupgrade.get_upgrade_state('sidgen', 'config_basedn_updated'):
-            self.log.debug("Already done, skipping")
+            logger.debug("Already done, skipping")
             return False, ()
 
         restart = False
@@ -249,7 +256,7 @@ class update_sids(Updater):
         ldap = self.api.Backend.ldap2
 
         if sysupgrade.get_upgrade_state('sidgen', 'update_sids') is not True:
-            self.log.debug("SIDs do not need to be generated")
+            logger.debug("SIDs do not need to be generated")
             return False, ()
 
         # check if IPA domain for AD trust has been created, and if we need to
@@ -263,8 +270,8 @@ class update_sids(Updater):
         try:
             entry = ldap.get_entry(domain_IPA_AD_dn, attrs_list=[attr_name])
         except errors.NotFound:
-            self.log.debug("IPA domain object %s is not configured",
-                           domain_IPA_AD_dn)
+            logger.debug("IPA domain object %s is not configured",
+                         domain_IPA_AD_dn)
             sysupgrade.set_upgrade_state('sidgen', 'update_sids', False)
             return False, ()
         else:
@@ -285,9 +292,9 @@ class update_sids(Updater):
                 try:
                     ldap.add_entry(task_entry)
                 except errors.DuplicateEntry:
-                    self.log.debug("sidgen task already created")
+                    logger.debug("sidgen task already created")
                 else:
-                    self.log.debug("sidgen task has been created")
+                    logger.debug("sidgen task has been created")
 
         # we have to check all trusts domains which may been affected by the
         # bug. Symptom is missing 'ipaNTSecurityIdentifier' attribute
@@ -306,13 +313,90 @@ class update_sids(Updater):
             pass
         else:
             if truncated:
-                self.log.warning("update_sids: Search results were truncated")
+                logger.warning("update_sids: Search results were truncated")
 
             for entry in trust_domain_entries:
                 domain = entry.single_value["cn"]
-                self.log.error(
+                logger.error(
                     "Your trust to %s is broken. Please re-create it by "
                     "running 'ipa trust-add' again.", domain)
 
         sysupgrade.set_upgrade_state('sidgen', 'update_sids', False)
         return False, ()
+
+
+@register()
+class update_tdo_gidnumber(Updater):
+    """
+    Create a gidNumber attribute for Trusted Domain Objects.
+
+    The value is taken from the fallback group defined in cn=Default SMB Group.
+    """
+    def execute(self, **options):
+        ldap = self.api.Backend.ldap2
+
+        # First, see if trusts are enabled on the server
+        if not self.api.Command.adtrust_is_enabled()['result']:
+            logger.debug('AD Trusts are not enabled on this server')
+            return False, []
+
+        # Read the gidnumber of the fallback group
+        dn = DN(('cn', ADTRUSTInstance.FALLBACK_GROUP_NAME),
+                self.api.env.container_group,
+                self.api.env.basedn)
+
+        try:
+            entry = ldap.get_entry(dn, ['gidnumber'])
+            gidNumber = entry.get('gidnumber')
+        except errors.NotFound:
+            logger.error("%s not found",
+                         ADTRUSTInstance.FALLBACK_GROUP_NAME)
+            return False, ()
+
+        if not gidNumber:
+            logger.error("%s does not have a gidnumber",
+                         ADTRUSTInstance.FALLBACK_GROUP_NAME)
+            return False, ()
+
+        # For each trusted domain object, add gidNumber
+        try:
+            tdos = ldap.get_entries(
+                DN(self.api.env.container_adtrusts, self.api.env.basedn),
+                scope=ldap.SCOPE_ONELEVEL,
+                filter="(objectclass=ipaNTTrustedDomain)",
+                attrs_list=['gidnumber'])
+            for tdo in tdos:
+                # if the trusted domain object does not contain gidnumber,
+                # add the default fallback group gidnumber
+                if not tdo.get('gidnumber'):
+                    try:
+                        tdo['gidnumber'] = gidNumber
+                        ldap.update_entry(tdo)
+                        logger.debug("Added gidnumber %s to %s",
+                                     gidNumber, tdo.dn)
+                    except Exception:
+                        logger.warning(
+                            "Failed to add gidnumber to %s", tdo.dn)
+
+        except errors.NotFound:
+            logger.debug("No trusted domain object to update")
+            return False, ()
+
+        return False, ()
+
+
+@register()
+class update_mapping_Guests_to_nobody(Updater):
+    """
+    Map BUILTIN\\Guests group to nobody
+
+    Samba 4.9 became more strict on availability of builtin Guests group
+    """
+    def execute(self, **options):
+        # First, see if trusts are enabled on the server
+        if not self.api.Command.adtrust_is_enabled()['result']:
+            logger.debug('AD Trusts are not enabled on this server')
+            return False, []
+
+        map_Guests_to_nobody()
+        return False, []
