@@ -38,6 +38,14 @@ from ipapython.dn import DN
 if api.env.in_server:
     import pki.account
     import pki.key
+    # pylint: disable=no-member
+    try:
+        # pki >= 10.4.0
+        from pki.crypto import DES_EDE3_CBC_OID
+    except ImportError:
+        DES_EDE3_CBC_OID = pki.key.KeyClient.DES_EDE3_CBC_OID
+    # pylint: enable=no-member
+
 
 if six.PY3:
     unicode = str
@@ -47,7 +55,9 @@ Vaults
 """) + _("""
 Manage vaults.
 """) + _("""
-Vault is a secure place to store a secret.
+Vault is a secure place to store a secret. One vault can only
+store one secret. When archiving a secret in a vault, the
+existing secret (if any) is overwritten.
 """) + _("""
 Based on the ownership there are three vault categories:
 * user/private vault
@@ -230,7 +240,7 @@ class VaultModMember(LDAPModMember):
         return super(VaultModMember, self).get_member_dns(**options)
 
     def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
-        for fail in failed.itervalues():
+        for fail in failed.values():
             fail['services'] = fail.pop('service', [])
         self.obj.get_container_attribute(entry_attrs, options)
         return completed, dn
@@ -758,6 +768,7 @@ class vault(LDAPObject):
 
 @register()
 class vault_add_internal(LDAPCreate):
+    __doc__ = _('Add a vault.')
 
     NO_CLI = True
 
@@ -816,23 +827,22 @@ class vault_del(LDAPDelete):
     def post_callback(self, ldap, dn, *args, **options):
         assert isinstance(dn, DN)
 
-        kra_client = self.api.Backend.kra.get_client()
+        with self.api.Backend.kra.get_client() as kra_client:
+            kra_account = pki.account.AccountClient(kra_client.connection)
+            kra_account.login()
 
-        kra_account = pki.account.AccountClient(kra_client.connection)
-        kra_account.login()
+            client_key_id = self.obj.get_key_id(dn)
 
-        client_key_id = self.obj.get_key_id(dn)
+            # deactivate vault record in KRA
+            response = kra_client.keys.list_keys(
+                client_key_id, pki.key.KeyClient.KEY_STATUS_ACTIVE)
 
-        # deactivate vault record in KRA
-        response = kra_client.keys.list_keys(
-            client_key_id, pki.key.KeyClient.KEY_STATUS_ACTIVE)
+            for key_info in response.key_infos:
+                kra_client.keys.modify_key_status(
+                    key_info.get_key_id(),
+                    pki.key.KeyClient.KEY_STATUS_INACTIVE)
 
-        for key_info in response.key_infos:
-            kra_client.keys.modify_key_status(
-                key_info.get_key_id(),
-                pki.key.KeyClient.KEY_STATUS_INACTIVE)
-
-        kra_account.logout()
+            kra_account.logout()
 
         return True
 
@@ -907,6 +917,7 @@ class vault_find(LDAPSearch):
 
 @register()
 class vault_mod_internal(LDAPUpdate):
+    __doc__ = _('Modify a vault.')
 
     NO_CLI = True
 
@@ -987,12 +998,12 @@ class vaultconfig_show(Retrieve):
             raise errors.InvocationError(
                 format=_('KRA service is not enabled'))
 
-        kra_client = self.api.Backend.kra.get_client()
-        transport_cert = kra_client.system_certs.get_transport_cert()
-        config = {'transport_cert': transport_cert.binary}
-        config.update(
-            self.api.Backend.serverroles.config_retrieve("KRA server")
-        )
+        with self.api.Backend.kra.get_client() as kra_client:
+            transport_cert = kra_client.system_certs.get_transport_cert()
+            config = {'transport_cert': transport_cert.binary}
+
+        self.api.Object.config.show_servroles_attributes(
+            config, "KRA server", **options)
 
         return {
             'result': config,
@@ -1002,6 +1013,7 @@ class vaultconfig_show(Retrieve):
 
 @register()
 class vault_archive_internal(PKQuery):
+    __doc__ = _('Archive data into a vault.')
 
     NO_CLI = True
 
@@ -1038,34 +1050,33 @@ class vault_archive_internal(PKQuery):
         vault = self.api.Command.vault_show(*args, **options)['result']
 
         # connect to KRA
-        kra_client = self.api.Backend.kra.get_client()
+        with self.api.Backend.kra.get_client() as kra_client:
+            kra_account = pki.account.AccountClient(kra_client.connection)
+            kra_account.login()
 
-        kra_account = pki.account.AccountClient(kra_client.connection)
-        kra_account.login()
+            client_key_id = self.obj.get_key_id(vault['dn'])
 
-        client_key_id = self.obj.get_key_id(vault['dn'])
+            # deactivate existing vault record in KRA
+            response = kra_client.keys.list_keys(
+                client_key_id,
+                pki.key.KeyClient.KEY_STATUS_ACTIVE)
 
-        # deactivate existing vault record in KRA
-        response = kra_client.keys.list_keys(
-            client_key_id,
-            pki.key.KeyClient.KEY_STATUS_ACTIVE)
+            for key_info in response.key_infos:
+                kra_client.keys.modify_key_status(
+                    key_info.get_key_id(),
+                    pki.key.KeyClient.KEY_STATUS_INACTIVE)
 
-        for key_info in response.key_infos:
-            kra_client.keys.modify_key_status(
-                key_info.get_key_id(),
-                pki.key.KeyClient.KEY_STATUS_INACTIVE)
+            # forward wrapped data to KRA
+            kra_client.keys.archive_encrypted_data(
+                client_key_id,
+                pki.key.KeyClient.PASS_PHRASE_TYPE,
+                wrapped_vault_data,
+                wrapped_session_key,
+                algorithm_oid=DES_EDE3_CBC_OID,
+                nonce_iv=nonce,
+            )
 
-        # forward wrapped data to KRA
-        kra_client.keys.archive_encrypted_data(
-            client_key_id,
-            pki.key.KeyClient.PASS_PHRASE_TYPE,
-            wrapped_vault_data,
-            wrapped_session_key,
-            None,
-            nonce,
-        )
-
-        kra_account.logout()
+            kra_account.logout()
 
         response = {
             'value': args[-1],
@@ -1079,6 +1090,7 @@ class vault_archive_internal(PKQuery):
 
 @register()
 class vault_retrieve_internal(PKQuery):
+    __doc__ = _('Retrieve data from a vault.')
 
     NO_CLI = True
 
@@ -1105,29 +1117,28 @@ class vault_retrieve_internal(PKQuery):
         vault = self.api.Command.vault_show(*args, **options)['result']
 
         # connect to KRA
-        kra_client = self.api.Backend.kra.get_client()
+        with self.api.Backend.kra.get_client() as kra_client:
+            kra_account = pki.account.AccountClient(kra_client.connection)
+            kra_account.login()
 
-        kra_account = pki.account.AccountClient(kra_client.connection)
-        kra_account.login()
+            client_key_id = self.obj.get_key_id(vault['dn'])
 
-        client_key_id = self.obj.get_key_id(vault['dn'])
+            # find vault record in KRA
+            response = kra_client.keys.list_keys(
+                client_key_id,
+                pki.key.KeyClient.KEY_STATUS_ACTIVE)
 
-        # find vault record in KRA
-        response = kra_client.keys.list_keys(
-            client_key_id,
-            pki.key.KeyClient.KEY_STATUS_ACTIVE)
+            if not len(response.key_infos):
+                raise errors.NotFound(reason=_('No archived data.'))
 
-        if not len(response.key_infos):
-            raise errors.NotFound(reason=_('No archived data.'))
+            key_info = response.key_infos[0]
 
-        key_info = response.key_infos[0]
+            # retrieve encrypted data from KRA
+            key = kra_client.keys.retrieve_key(
+                key_info.get_key_id(),
+                wrapped_session_key)
 
-        # retrieve encrypted data from KRA
-        key = kra_client.keys.retrieve_key(
-            key_info.get_key_id(),
-            wrapped_session_key)
-
-        kra_account.logout()
+            kra_account.logout()
 
         response = {
             'value': args[-1],
@@ -1208,6 +1219,7 @@ class vault_remove_member(VaultModMember, LDAPRemoveMember):
 
 @register()
 class kra_is_enabled(Command):
+    __doc__ = _('Checks if any of the servers has the KRA service enabled')
     NO_CLI = True
 
     has_output = output.standard_value
