@@ -21,9 +21,11 @@
 Base class for all cmdline tests
 """
 
-import nose
+from __future__ import absolute_import
+
 import distutils.spawn
 import os
+import unittest
 
 from ipalib import api
 from ipalib import errors
@@ -65,6 +67,6 @@ class cmdline_test(XMLRPC_test):
             )
         super(cmdline_test, cls).setup_class()
         if not server_available:
-            raise nose.SkipTest(
+            raise unittest.SkipTest(
                 'Server not available: %r' % api.env.xmlrpc_uri
             )
