@@ -18,20 +18,24 @@
 #
 
 import sys
-import contextlib
+import os
+import shutil
+import errno
 
-from nose.tools import assert_raises  # pylint: disable=E0611
 import six
 from six import StringIO
 
 from ipalib import api, errors
-from ipalib.plugins.user import user_add
+from ipaserver.plugins.user import user_add
 import pytest
 
 if six.PY3:
     unicode = str
 
 
+pytestmark = pytest.mark.needs_ipaapi
+
+
 @pytest.mark.tier0
 class CLITestContext(object):
     """Context manager that replaces stdout & stderr, and catches SystemExit
@@ -62,6 +66,8 @@ class CLITestContext(object):
                 return False
             self.exception = exc_value
             return True
+        else:
+            return None
 
 
 def test_ipa_help():
@@ -72,6 +78,27 @@ def test_ipa_help():
     assert ctx.stderr == ''
 
 
+def test_ipa_help_without_cache():
+    """Test `ipa help` without schema cache"""
+    cache_dir = os.path.expanduser('~/.cache/ipa/schema/')
+    backup_dir = os.path.expanduser('~/.cache/ipa/schema.bak/')
+    shutil.rmtree(backup_dir, ignore_errors=True)
+    if os.path.isdir(cache_dir):
+        os.rename(cache_dir, backup_dir)
+    try:
+        with CLITestContext() as ctx:
+            return_value = api.Backend.cli.run(['help'])
+        assert return_value == 0
+        assert ctx.stderr == ''
+    finally:
+        shutil.rmtree(cache_dir, ignore_errors=True)
+        try:
+            os.rename(backup_dir, cache_dir)
+        except OSError as e:
+            if e.errno != errno.ENOENT:
+                raise
+
+
 def test_ipa_without_arguments():
     """Test that `ipa` errors out, and prints the help to stderr"""
     with CLITestContext(exception=SystemExit) as ctx:
@@ -116,7 +143,7 @@ def test_command_help():
     assert h_ctx.stderr == ''
 
     assert h_ctx.stdout == help_ctx.stdout
-    assert unicode(user_add.__doc__) in help_ctx.stdout
+    assert unicode(user_add.doc) in help_ctx.stdout
 
 
 def test_ambiguous_command_or_topic():
@@ -136,6 +163,7 @@ def test_ambiguous_command_or_topic():
 
     assert h_ctx.stdout != help_ctx.stdout
 
+
 def test_multiline_description():
     """Test that all of a multi-line command description appears in output
     """
@@ -143,6 +171,6 @@ def test_multiline_description():
     assert '\n\n' in unicode(api.Command.trust_add.doc).strip()
 
     with CLITestContext(exception=SystemExit) as help_ctx:
-        return_value = api.Backend.cli.run(['trust-add', '-h'])
+        api.Backend.cli.run(['trust-add', '-h'])
 
     assert unicode(api.Command.trust_add.doc).strip() in help_ctx.stdout
