@@ -21,12 +21,9 @@
 
 import pytest
 
-from ipapython.ipa_log_manager import log_mgr
-from ipatests.test_integration import tasks
+from ipatests.pytest_ipa.integration import tasks
 from pytest_sourceorder import ordered
 
-log = log_mgr.get_logger(__name__)
-
 
 @ordered
 @pytest.mark.usefixtures('mh')
@@ -84,6 +81,3 @@ class IntegrationTest(object):
             tasks.uninstall_master(replica)
         for client in cls.clients:
             tasks.uninstall_client(client)
-
-
-IntegrationTest.log = log_mgr.get_logger(IntegrationTest())
