@@ -2,16 +2,25 @@
 # Copyright (C) 2015  FreeIPA Contributors see COPYING for license
 #
 
+from __future__ import absolute_import
+
+import logging
+import time
+
 import dns.dnssec
 import dns.resolver
 import dns.name
-import time
-import pytest
 
 from ipatests.test_integration.base import IntegrationTest
-from ipatests.test_integration import tasks
+from ipatests.pytest_ipa.integration import tasks
 from ipaplatform.paths import paths
 
+logger = logging.getLogger(__name__)
+
+# Sleep 5 seconds at most when waiting for LDAP updates
+# for DNSSEC changes. Test zones should be updated with 1 second TTL
+DNSSEC_SLEEP = 5
+
 test_zone = "dnssec.test."
 test_zone_repl = "dnssec-replica.test."
 root_zone = "."
@@ -20,7 +29,7 @@ example2_test_zone = "example2.test."
 example3_test_zone = "example3.test."
 
 
-def resolve_with_dnssec(nameserver, query, log, rtype="SOA"):
+def resolve_with_dnssec(nameserver, query, rtype="SOA"):
     res = dns.resolver.Resolver()
     res.nameservers = [nameserver]
     res.lifetime = 10  # wait max 10 seconds for reply
@@ -33,17 +42,18 @@ def resolve_with_dnssec(nameserver, query, log, rtype="SOA"):
     ans = res.query(query, rtype)
     return ans
 
-def get_RRSIG_record(nameserver, query, log, rtype="SOA"):
-    ans = resolve_with_dnssec(nameserver, query, log, rtype=rtype)
+
+def get_RRSIG_record(nameserver, query, rtype="SOA"):
+    ans = resolve_with_dnssec(nameserver, query, rtype=rtype)
     return ans.response.find_rrset(
         ans.response.answer, dns.name.from_text(query),
         dns.rdataclass.IN, dns.rdatatype.RRSIG,
         dns.rdatatype.from_text(rtype))
 
 
-def is_record_signed(nameserver, query, log, rtype="SOA"):
+def is_record_signed(nameserver, query, rtype="SOA"):
     try:
-        get_RRSIG_record(nameserver, query, log, rtype=rtype)
+        get_RRSIG_record(nameserver, query, rtype=rtype)
     except KeyError:
         return False
     except dns.exception.DNSException:
@@ -51,27 +61,40 @@ def is_record_signed(nameserver, query, log, rtype="SOA"):
     return True
 
 
-def wait_until_record_is_signed(nameserver, record, log, rtype="SOA",
+def wait_until_record_is_signed(nameserver, record, rtype="SOA",
                                 timeout=100):
     """
     Returns True if record is signed, or False on timeout
     :param nameserver: nameserver to query
     :param record: query
-    :param log: logger
     :param rtype: record type
     :param timeout:
     :return: True if records is signed, False if timeout
     """
-    log.info("Waiting for signed %s record of %s from server %s (timeout %s "
-             "sec)", rtype, record, nameserver, timeout)
+    logger.info("Waiting for signed %s record of %s from server %s (timeout "
+                "%s sec)", rtype, record, nameserver, timeout)
     wait_until = time.time() + timeout
     while time.time() < wait_until:
-        if is_record_signed(nameserver, record, log, rtype=rtype):
+        if is_record_signed(nameserver, record, rtype=rtype):
             return True
         time.sleep(1)
     return False
 
 
+def dnszone_add_dnssec(host, test_zone):
+    """Add dnszone with dnssec and short TTL
+    """
+    args = [
+        "ipa",
+        "dnszone-add", test_zone,
+        "--skip-overlap-check",
+        "--dnssec", "true",
+        "--ttl", "1",
+        "--default-ttl", "1",
+    ]
+    return host.run_command(args)
+
+
 class TestInstallDNSSECLast(IntegrationTest):
     """Simple DNSSEC test
 
@@ -98,52 +121,35 @@ class TestInstallDNSSECLast(IntegrationTest):
 
     def test_if_zone_is_signed_master(self):
         # add zone with enabled DNSSEC signing on master
-        args = [
-            "ipa",
-            "dnszone-add", test_zone,
-            "--skip-overlap-check",
-            "--dnssec", "true",
-        ]
-        self.master.run_command(args)
-
-        tasks.restart_named(self.master, self.replicas[0])
+        dnszone_add_dnssec(self.master, test_zone)
         # test master
         assert wait_until_record_is_signed(
-            self.master.ip, test_zone, self.log, timeout=100
+            self.master.ip, test_zone, timeout=100
         ), "Zone %s is not signed (master)" % test_zone
 
         # test replica
         assert wait_until_record_is_signed(
-            self.replicas[0].ip, test_zone, self.log, timeout=200
+            self.replicas[0].ip, test_zone, timeout=200
         ), "DNS zone %s is not signed (replica)" % test_zone
 
     def test_if_zone_is_signed_replica(self):
         # add zone with enabled DNSSEC signing on replica
-        args = [
-            "ipa",
-            "dnszone-add", test_zone_repl,
-            "--skip-overlap-check",
-            "--dnssec", "true",
-        ]
-        self.replicas[0].run_command(args)
-
-        tasks.restart_named(self.replicas[0])
+        dnszone_add_dnssec(self.replicas[0], test_zone_repl)
         # test replica
         assert wait_until_record_is_signed(
-            self.replicas[0].ip, test_zone_repl, self.log, timeout=300
+            self.replicas[0].ip, test_zone_repl, timeout=300
         ), "Zone %s is not signed (replica)" % test_zone_repl
 
         # we do not need to wait, on master zones should be singed faster
         # than on replicas
 
         assert wait_until_record_is_signed(
-            self.master.ip, test_zone_repl, self.log, timeout=5
+            self.master.ip, test_zone_repl, timeout=5
         ), "DNS zone %s is not signed (master)" % test_zone
 
     def test_disable_reenable_signing_master(self):
-
         dnskey_old = resolve_with_dnssec(self.master.ip, test_zone,
-                                         self.log, rtype="DNSKEY").rrset
+                                         rtype="DNSKEY").rrset
 
         # disable DNSSEC signing of zone on master
         args = [
@@ -153,16 +159,16 @@ class TestInstallDNSSECLast(IntegrationTest):
         ]
         self.master.run_command(args)
 
-        time.sleep(20)  # sleep a bit until LDAP changes are applied to DNS
+        time.sleep(DNSSEC_SLEEP)
 
         # test master
         assert not is_record_signed(
-            self.master.ip, test_zone, self.log
+            self.master.ip, test_zone
         ), "Zone %s is still signed (master)" % test_zone
 
         # test replica
         assert not is_record_signed(
-            self.replicas[0].ip, test_zone, self.log
+            self.replicas[0].ip, test_zone
         ), "DNS zone %s is still signed (replica)" % test_zone
 
         # reenable DNSSEC signing
@@ -173,25 +179,25 @@ class TestInstallDNSSECLast(IntegrationTest):
         ]
         self.master.run_command(args)
 
-        tasks.restart_named(self.master)
+        # TODO: test require restart
+        tasks.restart_named(self.master, self.replicas[0])
         # test master
         assert wait_until_record_is_signed(
-            self.master.ip, test_zone, self.log, timeout=100
+            self.master.ip, test_zone, timeout=100
         ), "Zone %s is not signed (master)" % test_zone
 
         # test replica
         assert wait_until_record_is_signed(
-            self.replicas[0].ip, test_zone, self.log, timeout=200
+            self.replicas[0].ip, test_zone, timeout=200
         ), "DNS zone %s is not signed (replica)" % test_zone
 
         dnskey_new = resolve_with_dnssec(self.master.ip, test_zone,
-                                         self.log, rtype="DNSKEY").rrset
+                                         rtype="DNSKEY").rrset
         assert dnskey_old != dnskey_new, "DNSKEY should be different"
 
     def test_disable_reenable_signing_replica(self):
-
         dnskey_old = resolve_with_dnssec(self.replicas[0].ip, test_zone_repl,
-                                         self.log, rtype="DNSKEY").rrset
+                                         rtype="DNSKEY").rrset
 
         # disable DNSSEC signing of zone on replica
         args = [
@@ -201,16 +207,16 @@ class TestInstallDNSSECLast(IntegrationTest):
         ]
         self.master.run_command(args)
 
-        time.sleep(20)  # sleep a bit until LDAP changes are applied to DNS
+        time.sleep(DNSSEC_SLEEP)
 
         # test master
         assert not is_record_signed(
-            self.master.ip, test_zone_repl, self.log
+            self.master.ip, test_zone_repl
         ), "Zone %s is still signed (master)" % test_zone_repl
 
         # test replica
         assert not is_record_signed(
-            self.replicas[0].ip, test_zone_repl, self.log
+            self.replicas[0].ip, test_zone_repl
         ), "DNS zone %s is still signed (replica)" % test_zone_repl
 
         # reenable DNSSEC signing
@@ -221,94 +227,23 @@ class TestInstallDNSSECLast(IntegrationTest):
         ]
         self.master.run_command(args)
 
+        # TODO: test require restart
         tasks.restart_named(self.master, self.replicas[0])
-
         # test master
         assert wait_until_record_is_signed(
-            self.master.ip, test_zone_repl, self.log, timeout=100
+            self.master.ip, test_zone_repl, timeout=100
         ), "Zone %s is not signed (master)" % test_zone_repl
 
         # test replica
         assert wait_until_record_is_signed(
-            self.replicas[0].ip, test_zone_repl, self.log, timeout=200
+            self.replicas[0].ip, test_zone_repl, timeout=200
         ), "DNS zone %s is not signed (replica)" % test_zone_repl
 
         dnskey_new = resolve_with_dnssec(self.replicas[0].ip, test_zone_repl,
-                                         self.log, rtype="DNSKEY").rrset
+                                         rtype="DNSKEY").rrset
         assert dnskey_old != dnskey_new, "DNSKEY should be different"
 
 
-class TestZoneSigningWithoutNamedRestart(IntegrationTest):
-    """Test whether https://fedorahosted.org/freeipa/ticket/5348 is already
-    fixed. If the issue is not fixed, the test will expectedly fail. When
-    fixed, it will pass, which will cause the whole run to become "red"
-    """
-    num_replicas = 1
-    topology = 'star'
-
-    @classmethod
-    def install(cls, mh):
-        tasks.install_master(cls.master, setup_dns=False)
-        args = [
-            "ipa-dns-install",
-            "--dnssec-master",
-            "--forwarder", cls.master.config.dns_forwarder,
-            "-U",
-        ]
-        cls.master.run_command(args)
-
-        tasks.install_replica(cls.master, cls.replicas[0], setup_dns=True)
-
-        # backup trusted key
-        tasks.backup_file(cls.master, paths.DNSSEC_TRUSTED_KEY)
-        tasks.backup_file(cls.replicas[0], paths.DNSSEC_TRUSTED_KEY)
-
-    @classmethod
-    def uninstall(cls, mh):
-        # restore trusted key
-        tasks.restore_files(cls.master)
-        tasks.restore_files(cls.replicas[0])
-
-        super(TestZoneSigningWithoutNamedRestart, cls).uninstall(mh)
-
-    @pytest.mark.xfail(strict=True)
-    def test_sign_root_zone_no_named_restart(self):
-        args = [
-            "ipa", "dnszone-add", root_zone, "--dnssec", "true",
-            "--skip-overlap-check",
-        ]
-        self.master.run_command(args)
-
-        # make BIND happy: add the glue record and delegate zone
-        args = [
-            "ipa", "dnsrecord-add", root_zone, self.master.hostname,
-            "--a-rec=" + self.master.ip
-        ]
-        self.master.run_command(args)
-        args = [
-            "ipa", "dnsrecord-add", root_zone, self.replicas[0].hostname,
-            "--a-rec=" + self.replicas[0].ip
-        ]
-        self.master.run_command(args)
-
-        time.sleep(10)  # sleep a bit until data are provided by bind-dyndb-ldap
-
-        args = [
-            "ipa", "dnsrecord-add", root_zone, self.master.domain.name,
-            "--ns-rec=" + self.master.hostname
-        ]
-        self.master.run_command(args)
-        # test master
-        assert wait_until_record_is_signed(
-            self.master.ip, root_zone, self.log, timeout=100
-        ), "Zone %s is not signed (master)" % root_zone
-
-        # test replica
-        assert wait_until_record_is_signed(
-            self.replicas[0].ip, root_zone, self.log, timeout=300
-        ), "Zone %s is not signed (replica)" % root_zone
-
-
 class TestInstallDNSSECFirst(IntegrationTest):
     """Simple DNSSEC test
 
@@ -343,11 +278,7 @@ class TestInstallDNSSECFirst(IntegrationTest):
         super(TestInstallDNSSECFirst, cls).uninstall(mh)
 
     def test_sign_root_zone(self):
-        args = [
-            "ipa", "dnszone-add", root_zone, "--dnssec", "true",
-            "--skip-overlap-check",
-        ]
-        self.master.run_command(args)
+        dnszone_add_dnssec(self.master, root_zone)
 
         # make BIND happy: add the glue record and delegate zone
         args = [
@@ -360,22 +291,21 @@ class TestInstallDNSSECFirst(IntegrationTest):
             "--a-rec=" + self.replicas[0].ip
         ]
         self.master.run_command(args)
-        time.sleep(10)  # sleep a bit until data are provided by bind-dyndb-ldap
+        time.sleep(DNSSEC_SLEEP)
 
         args = [
             "ipa", "dnsrecord-add", root_zone, self.master.domain.name,
             "--ns-rec=" + self.master.hostname
         ]
         self.master.run_command(args)
-        tasks.restart_named(self.master, self.replicas[0])
         # test master
         assert wait_until_record_is_signed(
-            self.master.ip, root_zone, self.log, timeout=100
+            self.master.ip, root_zone, timeout=100
         ), "Zone %s is not signed (master)" % root_zone
 
         # test replica
         assert wait_until_record_is_signed(
-            self.replicas[0].ip, root_zone, self.log, timeout=300
+            self.replicas[0].ip, root_zone, timeout=300
         ), "Zone %s is not signed (replica)" % root_zone
 
     def test_chain_of_trust(self):
@@ -383,14 +313,7 @@ class TestInstallDNSSECFirst(IntegrationTest):
         Validate signed DNS records, using our own signed root zone
         :return:
         """
-
-        # add test zone
-        args = [
-            "ipa", "dnszone-add", example_test_zone, "--dnssec", "true",
-            "--skip-overlap-check",
-        ]
-
-        self.master.run_command(args)
+        dnszone_add_dnssec(self.master, example_test_zone)
 
         # delegation
         args = [
@@ -398,18 +321,21 @@ class TestInstallDNSSECFirst(IntegrationTest):
             "--ns-rec=" + self.master.hostname
         ]
         self.master.run_command(args)
+
+        # TODO: test require restart
         tasks.restart_named(self.master, self.replicas[0])
+
         # wait until zone is signed
         assert wait_until_record_is_signed(
-            self.master.ip, example_test_zone, self.log, timeout=100
+            self.master.ip, example_test_zone, timeout=100
         ), "Zone %s is not signed (master)" % example_test_zone
         # wait until zone is signed
         assert wait_until_record_is_signed(
-            self.replicas[0].ip, example_test_zone, self.log, timeout=200
+            self.replicas[0].ip, example_test_zone, timeout=200
         ), "Zone %s is not signed (replica)" % example_test_zone
 
         # GET DNSKEY records from zone
-        ans = resolve_with_dnssec(self.master.ip, example_test_zone, self.log,
+        ans = resolve_with_dnssec(self.master.ip, example_test_zone,
                                   rtype="DNSKEY")
         dnskey_rrset = ans.response.get_rrset(
             ans.response.answer,
@@ -418,7 +344,7 @@ class TestInstallDNSSECFirst(IntegrationTest):
             dns.rdatatype.DNSKEY)
         assert dnskey_rrset, "No DNSKEY records received"
 
-        self.log.debug("DNSKEY records returned: %s", dnskey_rrset.to_text())
+        logger.debug("DNSKEY records returned: %s", dnskey_rrset.to_text())
 
         # generate DS records
         ds_records = []
@@ -430,8 +356,8 @@ class TestInstallDNSSECFirst(IntegrationTest):
         assert ds_records, ("No KSK returned from the %s zone" %
                             example_test_zone)
 
-        self.log.debug("DS records for %s created: %r", example_test_zone,
-                       ds_records)
+        logger.debug("DS records for %s created: %r", example_test_zone,
+                     ds_records)
 
         # add DS records to root zone
         args = [
@@ -447,12 +373,12 @@ class TestInstallDNSSECFirst(IntegrationTest):
 
         # wait until DS records it replicated
         assert wait_until_record_is_signed(
-            self.replicas[0].ip, example_test_zone, self.log, timeout=100,
+            self.replicas[0].ip, example_test_zone, timeout=100,
             rtype="DS"
         ), "No DS record of '%s' returned from replica" % example_test_zone
 
         # extract DSKEY from root zone
-        ans = resolve_with_dnssec(self.master.ip, root_zone, self.log,
+        ans = resolve_with_dnssec(self.master.ip, root_zone,
                                   rtype="DNSKEY")
         dnskey_rrset = ans.response.get_rrset(ans.response.answer,
                                               dns.name.from_text(root_zone),
@@ -460,7 +386,7 @@ class TestInstallDNSSECFirst(IntegrationTest):
                                               dns.rdatatype.DNSKEY)
         assert dnskey_rrset, "No DNSKEY records received"
 
-        self.log.debug("DNSKEY records returned: %s", dnskey_rrset.to_text())
+        logger.debug("DNSKEY records returned: %s", dnskey_rrset.to_text())
 
         # export trust keys for root zone
         root_key_rdatas = []
@@ -474,7 +400,7 @@ class TestInstallDNSSECFirst(IntegrationTest):
         root_keys_rrset = dns.rrset.from_rdata_list(dnskey_rrset.name,
                                                     dnskey_rrset.ttl,
                                                     root_key_rdatas)
-        self.log.debug("Root zone trusted key: %s", root_keys_rrset.to_text())
+        logger.debug("Root zone trusted key: %s", root_keys_rrset.to_text())
 
         # set trusted key for our root zone
         self.master.put_file_contents(paths.DNSSEC_TRUSTED_KEY,
@@ -483,7 +409,7 @@ class TestInstallDNSSECFirst(IntegrationTest):
                                            root_keys_rrset.to_text() + '\n')
 
         # verify signatures
-        time.sleep(1)
+        time.sleep(DNSSEC_SLEEP)
         args = [
             "drill", "@localhost", "-k",
             paths.DNSSEC_TRUSTED_KEY, "-S",
@@ -529,25 +455,18 @@ class TestMigrateDNSSECMaster(IntegrationTest):
         replica_backup_filename = "/tmp/ipa-kasp.db.backup"
 
         # add test zone
-        args = [
-            "ipa", "dnszone-add", example_test_zone, "--dnssec", "true",
-            "--skip-overlap-check",
-        ]
-
-        self.master.run_command(args)
-
-        tasks.restart_named(self.master, self.replicas[0])
+        dnszone_add_dnssec(self.master, example_test_zone)
         # wait until zone is signed
         assert wait_until_record_is_signed(
-            self.master.ip, example_test_zone, self.log, timeout=100
+            self.master.ip, example_test_zone, timeout=100
         ), "Zone %s is not signed (master)" % example_test_zone
         # wait until zone is signed
         assert wait_until_record_is_signed(
-            self.replicas[0].ip, example_test_zone, self.log, timeout=200
+            self.replicas[0].ip, example_test_zone, timeout=200
         ), "Zone %s is not signed (replica)" % example_test_zone
 
         dnskey_old = resolve_with_dnssec(self.master.ip, example_test_zone,
-                                         self.log, rtype="DNSKEY").rrset
+                                         rtype="DNSKEY").rrset
 
         # migrate dnssec master to replica
         args = [
@@ -575,33 +494,28 @@ class TestMigrateDNSSECMaster(IntegrationTest):
 
         # wait until zone is signed
         assert wait_until_record_is_signed(
-            self.master.ip, example_test_zone, self.log, timeout=100
+            self.master.ip, example_test_zone, timeout=100
         ), "Zone %s is not signed after migration (master)" % example_test_zone
         # wait until zone is signed
         assert wait_until_record_is_signed(
-            self.replicas[0].ip, example_test_zone, self.log, timeout=200
+            self.replicas[0].ip, example_test_zone, timeout=200
         ), "Zone %s is not signed after migration (replica)" % example_test_zone
 
         # test if dnskey are the same
         dnskey_new = resolve_with_dnssec(self.master.ip, example_test_zone,
-                                         self.log, rtype="DNSKEY").rrset
+                                         rtype="DNSKEY").rrset
         assert dnskey_old == dnskey_new, "DNSKEY should be the same"
 
         # add test zone
-        args = [
-            "ipa", "dnszone-add", example2_test_zone, "--dnssec", "true",
-            "--skip-overlap-check",
-        ]
-        self.replicas[0].run_command(args)
-        tasks.restart_named(self.master, self.replicas[0])
+        dnszone_add_dnssec(self.replicas[0], example2_test_zone)
         # wait until zone is signed
         assert wait_until_record_is_signed(
-            self.replicas[0].ip, example2_test_zone, self.log, timeout=100
+            self.replicas[0].ip, example2_test_zone, timeout=100
         ), ("Zone %s is not signed after migration (replica - dnssec master)"
             % example2_test_zone)
         # wait until zone is signed
         assert wait_until_record_is_signed(
-            self.master.ip, example2_test_zone, self.log, timeout=200
+            self.master.ip, example2_test_zone, timeout=200
         ), ("Zone %s is not signed after migration (master)"
             % example2_test_zone)
 
@@ -611,33 +525,72 @@ class TestMigrateDNSSECMaster(IntegrationTest):
         # test if originial zones are signed on new replica
         # wait until zone is signed
         assert wait_until_record_is_signed(
-            self.replicas[1].ip, example_test_zone, self.log, timeout=200
+            self.replicas[1].ip, example_test_zone, timeout=200
         ), ("Zone %s is not signed (new replica)"
             % example_test_zone)
         # wait until zone is signed
         assert wait_until_record_is_signed(
-            self.replicas[1].ip, example2_test_zone, self.log, timeout=200
+            self.replicas[1].ip, example2_test_zone, timeout=200
         ), ("Zone %s is not signed (new replica)"
             % example2_test_zone)
 
         # add new zone to new replica
-        args = [
-            "ipa", "dnszone-add", example3_test_zone, "--dnssec", "true",
-            "--skip-overlap-check",
-        ]
-        self.replicas[1].run_command(args)
-        tasks.restart_named(self.replicas[0], self.replicas[1])
+        dnszone_add_dnssec(self.replicas[0], example3_test_zone)
         # wait until zone is signed
         assert wait_until_record_is_signed(
-            self.replicas[1].ip, example3_test_zone, self.log, timeout=200
+            self.replicas[1].ip, example3_test_zone, timeout=200
         ), ("Zone %s is not signed (new replica)"
             % example3_test_zone)
         assert wait_until_record_is_signed(
-            self.replicas[0].ip, example3_test_zone, self.log, timeout=200
+            self.replicas[0].ip, example3_test_zone, timeout=200
         ), ("Zone %s is not signed (replica)"
             % example3_test_zone)
         # wait until zone is signed
         assert wait_until_record_is_signed(
-            self.master.ip, example3_test_zone, self.log, timeout=200
+            self.master.ip, example3_test_zone, timeout=200
         ), ("Zone %s is not signed (master)"
             % example3_test_zone)
+
+
+class TestInstallNoDnssecValidation(IntegrationTest):
+    """test installation of the master with
+    --no-dnssec-validation
+
+    Test for issue 7666: ipa-server-install --setup-dns is failing
+    if using --no-dnssec-validation and --forwarder, when the
+    specified forwarder does not support DNSSEC.
+    The forwarder should not be checked for DNSSEC support when
+    --no-dnssec-validation argument is specified.
+    In order to reproduce the conditions, the test is using a dummy
+    IP address for the forwarder (i.e. there is no BIND service available
+    at this IP address). To make sure of that, the test is using the IP of
+    a replica (that is not yet setup).
+    """
+    num_replicas = 1
+
+    @classmethod
+    def install(cls, mh):
+        cls.install_args = [
+            'ipa-server-install',
+            '-n', cls.master.domain.name,
+            '-r', cls.master.domain.realm,
+            '-p', cls.master.config.dirman_password,
+            '-a', cls.master.config.admin_password,
+            '-U',
+            '--setup-dns',
+            '--forwarder', cls.replicas[0].ip,
+            '--auto-reverse'
+        ]
+
+    def test_install_withDnssecValidation(self):
+        cmd = self.master.run_command(self.install_args, raiseonerr=False)
+        # The installer checks that the forwarder supports DNSSEC
+        # but the forwarder does not answer => expect failure
+        assert cmd.returncode != 0
+
+    def test_install_noDnssecValidation(self):
+        # With the --no-dnssec-validation, the installer does not check
+        # whether the forwarder supports DNSSEC => success even if the
+        # forwarder is not reachable
+        self.master.run_command(
+            self.install_args + ['--no-dnssec-validation'])
