@@ -20,8 +20,8 @@
 import pytest
 
 from ipatests.test_integration.base import IntegrationTest
-from ipatests.test_integration.tasks import clear_sssd_cache, modify_sssd_conf
-from ipatests.test_integration import util
+from ipatests.pytest_ipa.integration.tasks import (
+    clear_sssd_cache, get_host_ip_with_hostmask, modify_sssd_conf)
 
 
 class TestSudo(IntegrationTest):
@@ -80,6 +80,11 @@ class TestSudo(IntegrationTest):
                                 'defaults',
                                 '--sudooption', "!authenticate"])
 
+        # Create test user -- member of group admins
+        cls.master.run_command(['ipa', 'user-add', 'admin2',
+                                '--first', 'Admin', '--last', 'Second'])
+        cls.master.run_command(['ipa', 'group-add-member', 'admins',
+                                '--users', 'admin2'])
 
     @classmethod
     def uninstall(cls, mh):
@@ -117,6 +122,34 @@ class TestSudo(IntegrationTest):
 
         return result
 
+    # testcases test_admins_group_does_not_have_sudo_permission and
+    # test_advise_script_enable_sudo_admins must be run before any other sudo
+    # rules are applied
+    def test_admins_group_does_not_have_sudo_permission(self):
+        result = self.list_sudo_commands('admin2', raiseonerr=False)
+        assert result.returncode == 1
+        assert "Sorry, user admin2 may not run sudo on {}.".format(
+            self.clientname) in result.stderr_text
+
+    def test_advise_script_enable_sudo_admins(self):
+        """
+            Test for advise scipt to add sudo permissions for admin users
+            https://pagure.io/freeipa/issue/7538
+        """
+        result = self.master.run_command('ipa-advise enable_admins_sudo')
+        script = result.stdout_text
+        self.master.run_command('bash', stdin_text=script)
+        try:
+            result = self.list_sudo_commands('admin2')
+            assert '(root) ALL' in result.stdout_text
+        finally:
+            result1 = self.master.run_command(
+                ['ipa', 'sudorule-del', 'admins_all'], raiseonerr=False)
+            result2 = self.master.run_command(
+                ['ipa', 'hbacrule-del', 'admins_sudo'], raiseonerr=False)
+            assert result1.returncode == 0 and result2.returncode == 0,\
+                'rules cleanup failed'
+
     def test_nisdomainname(self):
         result = self.client.run_command('nisdomainname')
         assert self.client.domain.name in result.stdout_text
@@ -298,7 +331,7 @@ class TestSudo(IntegrationTest):
 
     def test_sudo_rule_restricted_to_one_hostmask_setup(self):
         # We need to detect the hostmask first
-        full_ip = util.get_host_ip_with_hostmask(self.client)
+        full_ip = get_host_ip_with_hostmask(self.client)
 
         # Make a note for the next test, which needs to be skipped
         # if hostmask detection failed
@@ -341,7 +374,7 @@ class TestSudo(IntegrationTest):
             raise pytest.skip("Hostmask could not be detected")
 
         # Detect the hostmask first to delete the hostmask based rule
-        full_ip = util.get_host_ip_with_hostmask(self.client)
+        full_ip = get_host_ip_with_hostmask(self.client)
 
         # Remove the client's hostmask from the rule
         self.master.run_command(['ipa', '-n', 'sudorule-remove-host',
