@@ -6,6 +6,7 @@ import re
 
 import pytest
 
+
 from ipatests.test_integration.base import IntegrationTest
 from ipatests.pytest_ipa.integration import tasks
 from ipatests.pytest_ipa.integration.env_config import get_global_config
@@ -25,6 +26,7 @@ def find_segment(master, replica):
     for segment in allsegments:
         if master.hostname in segment and replica.hostname in segment:
             return '-to-'.join(segment)
+    return None
 
 
 @pytest.mark.skipif(config.domain_level == 0, reason=reasoning)
@@ -243,6 +245,10 @@ class TestCASpecificRUVs(IntegrationTest):
         master.run_command(['ipa-replica-manage', 'del', replica.hostname,
                             '-p', master.config.dirman_password])
         tasks.uninstall_master(replica)
+        # ipa-replica-manage del launches a clean-ruv task which is
+        # ASYNCHRONOUS
+        # wait for the task to finish before checking list-ruv
+        tasks.wait_for_cleanallruv_tasks(self.master.ldap_connect())
         res2 = master.run_command(['ipa-replica-manage', 'list-ruv', '-p',
                                   master.config.dirman_password]).stdout_text
         assert(replica.hostname not in res2), (
