@@ -17,12 +17,13 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import nose
+from __future__ import absolute_import
+
 import re
+import unittest
 
 from ipatests.test_integration.base import IntegrationTest
-from ipatests.test_integration import tasks
-from ipatests.test_integration import util
+from ipatests.pytest_ipa.integration import tasks
 from ipaplatform.paths import paths
 
 
@@ -36,7 +37,7 @@ class ADTrustBase(IntegrationTest):
     @classmethod
     def install(cls, mh):
         if not cls.master.transport.file_exists('/usr/bin/rpcclient'):
-            raise nose.SkipTest("Package samba-client not available "
+            raise unittest.SkipTest("Package samba-client not available "
                                 "on {}".format(cls.master.hostname))
         super(ADTrustBase, cls).install(mh)
         cls.ad = cls.ad_domains[0].ads[0]
@@ -80,8 +81,8 @@ class ADTrustBase(IntegrationTest):
                     % dict(idauth=_sid_identifier_authority)
         stdout_re = re.escape('  ipaNTSecurityIdentifier: ') + sid_regex
 
-        util.run_repeatedly(cls.master, command,
-                            test=lambda x: re.search(stdout_re, x))
+        tasks.run_repeatedly(cls.master, command,
+                             test=lambda x: re.search(stdout_re, x))
 
     @classmethod
     def configure_dns_and_time(cls):
@@ -100,7 +101,7 @@ class ADTrustBase(IntegrationTest):
         """
 
         if self.ad_subdomain is None:
-            raise nose.SkipTest('AD subdomain is not available.')
+            raise unittest.SkipTest('AD subdomain is not available.')
 
         result = self.master.run_command(['ipa',
                                           'trustdomain-find',
@@ -126,7 +127,7 @@ class ADTrustSubdomainBase(ADTrustBase):
     def install(cls, mh):
         super(ADTrustSubdomainBase, cls).install(mh)
         if not cls.ad_subdomain:
-            raise nose.SkipTest('AD subdomain is not available.')
+            raise unittest.SkipTest('AD subdomain is not available.')
 
 
 class ADTrustTreedomainBase(ADTrustBase):
@@ -143,7 +144,7 @@ class ADTrustTreedomainBase(ADTrustBase):
     def install(cls, mh):
         super(ADTrustTreedomainBase, cls).install(mh)
         if not cls.ad_treedomain:
-            raise nose.SkipTest('AD tree root domain is not available.')
+            raise unittest.SkipTest('AD tree root domain is not available.')
 
 
 class TestBasicADTrust(ADTrustBase):
@@ -171,8 +172,8 @@ class TestBasicADTrust(ADTrustBase):
 
         # This regex checks that Test User does not have UID 10042 nor belongs
         # to the group with GID 10047
-        testuser_regex = "^testuser@%s:\*:(?!10042)(\d+):(?!10047)(\d+):"\
-                         "Test User:/home/%s/testuser:/bin/sh$"\
+        testuser_regex = r"^testuser@%s:\*:(?!10042)(\d+):(?!10047)(\d+):"\
+                         r"Test User:/home/%s/testuser:/bin/sh$"\
                          % (re.escape(self.ad_domain),
                             re.escape(self.ad_domain))
 
@@ -189,8 +190,8 @@ class TestBasicADTrust(ADTrustBase):
                                 stdin_text=original_passwd)
 
         # change password for the user to be able to kinit
-        util.ldappasswd_user_change(ipauser, original_passwd, new_passwd,
-                                    self.master)
+        tasks.ldappasswd_user_change(ipauser, original_passwd, new_passwd,
+                                     self.master)
 
         # try to kinit as ipauser
         self.master.run_command(
@@ -207,8 +208,10 @@ class TestPosixADTrust(ADTrustBase):
     """Integration test for Active Directory with POSIX support"""
 
     def test_establish_trust(self):
-        # Not specifying the --range-type directly, it should be detected
-        tasks.establish_trust_with_ad(self.master, self.ad_domain)
+        tasks.establish_trust_with_ad(
+            self.master, self.ad_domain,
+            extra_args=['--range-type', 'ipa-ad-trust-posix']
+        )
 
     def test_range_properties_in_posix_trust(self):
         # Check the properties of the created range
@@ -259,8 +262,11 @@ class TestEnforcedPosixADTrust(TestPosixADTrust):
     This test is intented to copycat PosixADTrust, since enforcing the POSIX
     trust type should not make a difference.
     """
+    """Re-difene method from test_establish_trust_with_posix_attributes
+    to test_establish_trust. win server 2016 no more have support for MFU/NIS,
+    so autodetection doesn't work"""
 
-    def test_establish_trust_with_posix_attributes(self):
+    def test_establish_trust(self):
         tasks.establish_trust_with_ad(self.master, self.ad_domain,
             extra_args=['--range-type', 'ipa-ad-trust-posix'])
 
@@ -315,9 +321,9 @@ class TestExternalTrustWithSubdomain(ADTrustSubdomainBase):
         testuser = 'subdomaintestuser@{0}'.format(self.ad_subdomain)
         result = self.master.run_command(['getent', 'passwd', testuser])
 
-        testuser_regex = ("^subdomaintestuser@{0}:\*:(?!10142)(\d+):"
-                          "(?!10147)(\d+):Subdomaintest User:"
-                          "/home/{1}/subdomaintestuser:/bin/sh$".format(
+        testuser_regex = (r"^subdomaintestuser@{0}:\*:(?!10142)(\d+):"
+                          r"(?!10147)(\d+):Subdomaintest User:"
+                          r"/home/{1}/subdomaintestuser:/bin/sh$".format(
                               re.escape(self.ad_subdomain),
                               re.escape(self.ad_subdomain)))
 
@@ -334,7 +340,7 @@ class TestNonexternalTrustWithSubdomain(ADTrustSubdomainBase):
     """
     def test_establish_trust(self):
         """ Tests establishing non-external trust with Active Directory """
-        self.master.run_command(['kinit', '-kt', paths.IPA_KEYTAB,
+        self.master.run_command(['kinit', '-kt', paths.HTTP_KEYTAB,
                                  'HTTP/%s' % self.master.hostname])
         self.master.run_command(['systemctl', 'restart', 'krb5kdc.service'])
         self.master.run_command(['kdestroy', '-A'])
@@ -354,7 +360,7 @@ class TestNonexternalTrustWithSubdomain(ADTrustSubdomainBase):
             self.ad_subdomain) in result.stderr_text)
 
     def test_all_trustdomains_found(self):
-        raise nose.SkipTest(
+        raise unittest.SkipTest(
             'Test case unapplicable, present for inheritance reason only')
 
 
@@ -382,9 +388,9 @@ class TestExternalTrustWithTreedomain(ADTrustTreedomainBase):
         testuser = 'treetestuser@{0}'.format(self.ad_treedomain)
         result = self.master.run_command(['getent', 'passwd', testuser])
 
-        testuser_regex = ("^treetestuser@{0}:\*:(?!10242)(\d+):"
-                          "(?!10247)(\d+):TreeTest User:"
-                          "/home/{1}/treetestuser:/bin/sh$".format(
+        testuser_regex = (r"^treetestuser@{0}:\*:(?!10242)(\d+):"
+                          r"(?!10247)(\d+):TreeTest User:"
+                          r"/home/{1}/treetestuser:/bin/sh$".format(
                               re.escape(self.ad_treedomain),
                               re.escape(self.ad_treedomain)))
 
@@ -401,7 +407,7 @@ class TestNonexternalTrustWithTreedomain(ADTrustTreedomainBase):
     """
     def test_establish_trust(self):
         """ Tests establishing non-external trust with Active Directory """
-        self.master.run_command(['kinit', '-kt', paths.IPA_KEYTAB,
+        self.master.run_command(['kinit', '-kt', paths.HTTP_KEYTAB,
                                  'HTTP/%s' % self.master.hostname])
         self.master.run_command(['systemctl', 'restart', 'krb5kdc.service'])
         self.master.run_command(['kdestroy', '-A'])
@@ -421,7 +427,7 @@ class TestNonexternalTrustWithTreedomain(ADTrustTreedomainBase):
             self.ad_treedomain) in result.stderr_text)
 
     def test_all_trustdomains_found(self):
-        raise nose.SkipTest(
+        raise unittest.SkipTest(
             'Test case unapplicable, present for inheritance reason only')
 
 
@@ -477,9 +483,11 @@ class TestTrustWithUPN(ADTrustBase):
                                           self.upn_principal])
 
         # result will contain AD domain, not UPN
-        upnuser_regex = "^{}@{}:\*:(\d+):(\d+):{}:/home/{}/{}:/bin/sh$".format(
-            self.upn_username, self.ad_domain, self.upn_name,
-            self.ad_domain, self.upn_username)
+        upnuser_regex = (
+            r"^{}@{}:\*:(\d+):(\d+):{}:/home/{}/{}:/bin/sh$".format(
+                self.upn_username, self.ad_domain, self.upn_name,
+                self.ad_domain, self.upn_username)
+        )
         assert re.search(upnuser_regex, result.stdout_text)
 
     def test_upn_user_authentication(self):
