@@ -30,7 +30,6 @@ from ipatests.util import TempDir, TempHome
 from ipalib.constants import OVERRIDE_ERROR, SET_ERROR, DEL_ERROR
 from ipalib.constants import NAME_REGEX, NAME_ERROR
 from ipalib import config, constants, base
-from ipaplatform.paths import paths
 
 import pytest
 
@@ -450,8 +449,8 @@ class test_Env(ClassChecker):
         assert o.dot_ipa == home.join('.ipa')
         assert o.in_tree is False
         assert o.context == 'default'
-        assert o.confdir == paths.ETC_IPA
-        assert o.conf == paths.IPA_DEFAULT_CONF
+        assert o.confdir == '/etc/ipa'
+        assert o.conf == '/etc/ipa/default.conf'
         assert o.conf_default == o.conf
 
         # Test overriding values created by _bootstrap()
@@ -463,11 +462,11 @@ class test_Env(ClassChecker):
         assert o.in_tree is False
         assert o.context == 'default'
         assert o.conf == '/my/wacky/whatever.conf'
-        assert o.conf_default == paths.IPA_DEFAULT_CONF
+        assert o.conf_default == '/etc/ipa/default.conf'
         (o, home) = self.bootstrap(conf_default='/my/wacky/default.conf')
         assert o.in_tree is False
         assert o.context == 'default'
-        assert o.conf == paths.IPA_DEFAULT_CONF
+        assert o.conf == '/etc/ipa/default.conf'
         assert o.conf_default == '/my/wacky/default.conf'
 
         # Test various overrides and types conversion
@@ -564,7 +563,8 @@ class test_Env(ClassChecker):
         # Test using DEFAULT_CONFIG:
         defaults = dict(constants.DEFAULT_CONFIG)
         (o, home) = self.finalize_core(None, **defaults)
-        assert list(o) == sorted(defaults)
+        list_o = [key for key in o if key != 'fips_mode']
+        assert list_o == sorted(defaults)
         for (key, value) in defaults.items():
             if value is object:
                 continue
@@ -577,7 +577,7 @@ class test_Env(ClassChecker):
         Test the `ipalib.config.Env._finalize` method.
         """
         # Check that calls cascade up the chain:
-        (o, home) = self.new(in_tree=True)
+        o, _home = self.new(in_tree=True)
         assert o._isdone('_bootstrap') is False
         assert o._isdone('_finalize_core') is False
         assert o._isdone('_finalize') is False
@@ -591,7 +591,7 @@ class test_Env(ClassChecker):
         assert str(e) == 'Env._finalize() already called'
 
         # Check that _finalize() calls __lock__()
-        (o, home) = self.new(in_tree=True)
+        o, _home = self.new(in_tree=True)
         assert o.__islocked__() is False
         o._finalize()
         assert o.__islocked__() is True
@@ -599,7 +599,7 @@ class test_Env(ClassChecker):
         assert str(e) == 'Env.__lock__() already called'
 
         # Check that **lastchance works
-        (o, home) = self.finalize_core(None)
+        o, _home = self.finalize_core(None)
         key = 'just_one_more_key'
         value = u'with one more value'
         lastchance = {key: value}
