@@ -22,12 +22,17 @@ Test the `ipalib.rpc` module.
 """
 from __future__ import print_function
 
-from six.moves.xmlrpc_client import Binary, Fault, dumps, loads
+import unittest
 
-import nose
+import pytest
 import six
+# pylint: disable=import-error
+from six.moves.xmlrpc_client import Binary, Fault, dumps, loads
+# pylint: enable=import-error
+from six.moves import urllib
 
 from ipatests.util import raises, assert_equal, PluginTester, DummyClass
+from ipatests.util import Fuzzy
 from ipatests.data import binary_bytes, utf8_bytes, unicode_str
 from ipalib.frontend import Command
 from ipalib.request import context, Connection
@@ -42,7 +47,7 @@ std_compound = (binary_bytes, utf8_bytes, unicode_str)
 
 
 def dump_n_load(value):
-    (param, method) = loads(
+    param, _method = loads(
         dumps((value,), allow_none=True)
     )
     return param[0]
@@ -113,7 +118,7 @@ def test_xml_wrap():
     u = f(u'hello', API_VERSION)
     assert type(u) is unicode
     assert u == u'hello'
-    value = f([dict(one=False, two=u'hello'), None, b'hello'], API_VERSION)
+    f([dict(one=False, two=u'hello'), None, b'hello'], API_VERSION)
 
 
 def test_xml_unwrap():
@@ -209,7 +214,7 @@ class test_xmlclient(PluginTester):
         class user_add(Command):
             pass
 
-        (o, api, home) = self.instance('Backend', user_add, in_server=False)
+        o, _api, _home = self.instance('Backend', user_add, in_server=False)
         args = (binary_bytes, utf8_bytes, unicode_str)
         kw = dict(one=binary_bytes, two=utf8_bytes, three=unicode_str)
         params = [args, kw]
@@ -255,17 +260,19 @@ class test_xmlclient(PluginTester):
         assert context.xmlclient.conn._calledall() is True
 
 
+@pytest.mark.skip_ipaclient_unittest
+@pytest.mark.needs_ipaapi
 class test_xml_introspection(object):
     @classmethod
-    def setup_class(self):
+    def setup_class(cls):
         try:
             api.Backend.xmlclient.connect()
         except (errors.NetworkError, IOError):
-            raise nose.SkipTest('%r: Server not available: %r' %
+            raise unittest.SkipTest('%r: Server not available: %r' %
                                 (__name__, api.env.xmlrpc_uri))
 
     @classmethod
-    def teardown_class(self):
+    def teardown_class(cls):
         request.destroy_context()
 
     def test_list_methods(self):
@@ -279,7 +286,7 @@ class test_xml_introspection(object):
 
     def test_list_methods_many_params(self):
         try:
-            result = api.Backend.xmlclient.conn.system.listMethods('foo')
+            api.Backend.xmlclient.conn.system.listMethods('foo')
         except Fault as f:
             print(f)
             assert f.faultCode == 3003
@@ -299,7 +306,7 @@ class test_xml_introspection(object):
 
     def test_signature_no_params(self):
         try:
-            result = api.Backend.xmlclient.conn.system.methodSignature()
+            api.Backend.xmlclient.conn.system.methodSignature()
         except Fault as f:
             print(f)
             assert f.faultCode == 3007
@@ -309,7 +316,7 @@ class test_xml_introspection(object):
 
     def test_signature_many_params(self):
         try:
-            result = api.Backend.xmlclient.conn.system.methodSignature('a', 'b')
+            api.Backend.xmlclient.conn.system.methodSignature('a', 'b')
         except Fault as f:
             print(f)
             assert f.faultCode == 3004
@@ -320,7 +327,7 @@ class test_xml_introspection(object):
 
     def test_help_no_params(self):
         try:
-            result = api.Backend.xmlclient.conn.system.methodHelp()
+            api.Backend.xmlclient.conn.system.methodHelp()
         except Fault as f:
             print(f)
             assert f.faultCode == 3007
@@ -330,7 +337,7 @@ class test_xml_introspection(object):
 
     def test_help_many_params(self):
         try:
-            result = api.Backend.xmlclient.conn.system.methodHelp('a', 'b')
+            api.Backend.xmlclient.conn.system.methodHelp('a', 'b')
         except Fault as f:
             print(f)
             assert f.faultCode == 3004
@@ -338,3 +345,55 @@ class test_xml_introspection(object):
                 "command 'system.methodHelp' takes at most 1 argument")
         else:
             raise AssertionError('did not raise')
+
+
+@pytest.mark.skip_ipaclient_unittest
+@pytest.mark.needs_ipaapi
+class test_rpcclient_context(PluginTester):
+    """
+    Test the context in `ipalib.rpc.rpcclient` plugin.
+    """
+    def setup(self):
+        try:
+            api.Backend.rpcclient.connect(ca_certfile='foo')
+        except (errors.NetworkError, IOError):
+            raise unittest.SkipTest('%r: Server not available: %r' %
+                                (__name__, api.env.xmlrpc_uri))
+
+    def teardown(self):
+        if api.Backend.rpcclient.isconnected():
+            api.Backend.rpcclient.disconnect()
+
+    def test_context_cafile(self):
+        """
+        Test that ca_certfile is set in `ipalib.rpc.rpcclient.connect`
+        """
+        ca_certfile = getattr(context, 'ca_certfile', None)
+        assert_equal(ca_certfile, 'foo')
+
+    def test_context_principal(self):
+        """
+        Test that principal is set in `ipalib.rpc.rpcclient.connect`
+        """
+        principal = getattr(context, 'principal', None)
+        assert_equal(principal, 'admin@%s' % api.env.realm)
+
+    def test_context_request_url(self):
+        """
+        Test that request_url is set in `ipalib.rpc.rpcclient.connect`
+        """
+        request_url = getattr(context, 'request_url', None)
+        assert_equal(request_url, 'https://%s/ipa/session/json' % api.env.host)
+
+    def test_context_session_cookie(self):
+        """
+        Test that session_cookie is set in `ipalib.rpc.rpcclient.connect`
+        """
+        fuzzy_cookie = Fuzzy(
+            r'^ipa_session=MagBearerToken=[A-Za-z0-9+\/]+=*;$')
+
+        session_cookie = getattr(context, 'session_cookie', None)
+        # pylint-2 is incorrectly spewing Too many positional arguments
+        # pylint: disable=E1121
+        unquoted = urllib.parse.unquote(session_cookie)
+        assert(unquoted == fuzzy_cookie)
