@@ -25,15 +25,14 @@ from __future__ import print_function
 import os
 import shutil
 import tempfile
+import unittest
 
-import nose
 import six
 import pytest
 
 from ipatests.i18n import create_po, po_file_iterate
 from ipalib.request import context
 from ipalib import text
-from ipapython.ipautil import file_exists
 
 if six.PY3:
     unicode = str
@@ -52,11 +51,36 @@ def test_create_translation():
 
 
 class test_TestLang(object):
+    lang_env_vars = {'LC_ALL', 'LC_MESSAGES', 'LANGUAGE', 'LANG'}
+
+    def setup_lang(self):
+        """
+        Set all env variables used by gettext to localize translation files
+        to xh_ZA
+        """
+        self.lang = 'xh_ZA'
+        self.saved_locale = {
+            k: v for k, v in os.environ.items() if k in self.lang_env_vars}
+
+        os.environ.update(
+            {env_var: self.lang for env_var in self.lang_env_vars}
+        )
+
+    def teardown_lang(self):
+        """
+        Revert the locale settings to original values. If the original env
+        variable was not set before, it will be popped off os.environ
+        """
+        for env_var in self.lang_env_vars:
+            if env_var not in self.saved_locale:
+                os.environ.pop(env_var, None)
+
+        os.environ.update(self.saved_locale)
+
     def setup(self):
         self.tmp_dir = None
-        self.saved_lang  = None
+        self.setup_lang()
 
-        self.lang = 'xh_ZA'
         self.domain = 'ipa'
 
         self.pot_basename = '%s.pot' % self.domain
@@ -64,7 +88,6 @@ class test_TestLang(object):
         self.mo_basename = '%s.mo' % self.domain
 
         self.tmp_dir = tempfile.mkdtemp()
-        self.saved_lang  = os.environ['LANG']
 
         self.locale_dir = os.path.join(self.tmp_dir, 'test_locale')
         self.msg_dir = os.path.join(self.locale_dir, self.lang, 'LC_MESSAGES')
@@ -79,22 +102,23 @@ class test_TestLang(object):
 
         result = create_po(self.pot_file, self.po_file, self.mo_file)
         if result:
-            raise nose.SkipTest('Unable to create po file "%s" & mo file "%s" from pot file "%s"' %
-                                (self.po_file, self.mo_file, self.pot_file))
+            raise unittest.SkipTest(
+                'Unable to create po file "%s" & mo file "%s" from pot '
+                'file "%s"' % (self.po_file, self.mo_file, self.pot_file)
+            )
 
-        if not file_exists(self.po_file):
-            raise nose.SkipTest(
+        if not os.path.isfile(self.po_file):
+            raise unittest.SkipTest(
                 'Test po file unavailable: {}'.format(self.po_file))
 
-        if not file_exists(self.mo_file):
-            raise nose.SkipTest(
+        if not os.path.isfile(self.mo_file):
+            raise unittest.SkipTest(
                 'Test mo file unavailable: {}'.format(self.mo_file))
 
         self.po_file_iterate = po_file_iterate
 
     def teardown(self):
-        if self.saved_lang is not None:
-            os.environ['LANG'] = self.saved_lang
+        self.teardown_lang()
 
         if self.tmp_dir is not None:
             shutil.rmtree(self.tmp_dir)
@@ -107,7 +131,6 @@ class test_TestLang(object):
         # but the setlocale function demands the locale be a valid
         # known locale, Zambia Xhosa is a reasonable choice :)
 
-        os.environ['LANG'] = self.lang
 
         # Create a gettext translation object specifying our domain as
         # 'ipa' and the locale_dir as 'test_locale' (i.e. where to
@@ -176,7 +199,13 @@ class test_Gettext(object):
 
     def test_mod(self):
         inst = self.klass('hello %(adj)s nurse', 'foo', 'bar')
-        assert inst % dict(adj='naughty', stuff='junk') == 'hello naughty nurse'
+        assert inst % dict(adj='tall', stuff='junk') == 'hello tall nurse'
+
+    def test_format(self):
+        inst = self.klass('{0} {adj} nurse', 'foo', 'bar')
+        posargs = ('hello', 'bye')
+        knownargs = {'adj': 'caring', 'stuff': 'junk'}
+        assert inst.format(*posargs, **knownargs) == 'hello caring nurse'
 
     def test_eq(self):
         inst1 = self.klass('what up?', 'foo', 'bar')
@@ -184,6 +213,7 @@ class test_Gettext(object):
         inst3 = self.klass('Hello world', 'foo', 'bar')
         inst4 = self.klass('what up?', 'foo', 'baz')
 
+        # pylint: disable=comparison-with-itself
         assert (inst1 == inst1) is True
         assert (inst1 == inst2) is True
         assert (inst1 == inst3) is False
@@ -241,12 +271,28 @@ class test_NGettext(object):
         assert inst % dict(count=1, dish='stew') == '1 goose makes a stew'
         assert inst % dict(count=2, dish='pie') == '2 geese make a pie'
 
+    def test_format(self):
+        singular = '{count} goose makes a {0} {dish}'
+        plural = '{count} geese make a {0} {dish}'
+        inst = self.klass(singular, plural, 'foo', 'bar')
+        posargs = ('tasty', 'disgusting')
+        knownargs0 = {'count': 0, 'dish': 'frown', 'stuff': 'junk'}
+        knownargs1 = {'count': 1, 'dish': 'stew', 'stuff': 'junk'}
+        knownargs2 = {'count': 2, 'dish': 'pie', 'stuff': 'junk'}
+        expected_str0 = '0 geese make a tasty frown'
+        expected_str1 = '1 goose makes a tasty stew'
+        expected_str2 = '2 geese make a tasty pie'
+        assert inst.format(*posargs, **knownargs0) == expected_str0
+        assert inst.format(*posargs, **knownargs1) == expected_str1
+        assert inst.format(*posargs, **knownargs2) == expected_str2
+
     def test_eq(self):
         inst1 = self.klass(singular, plural, 'foo', 'bar')
         inst2 = self.klass(singular, plural, 'foo', 'bar')
         inst3 = self.klass(singular, '%(count)d thingies', 'foo', 'bar')
         inst4 = self.klass(singular, plural, 'foo', 'baz')
 
+        # pylint: disable=comparison-with-itself
         assert (inst1 == inst1) is True
         assert (inst1 == inst2) is True
         assert (inst1 == inst3) is False
@@ -364,6 +410,12 @@ class test_ConcatenatedText(object):
         inst = self.klass('[', text.Gettext('%(color)s', 'foo', 'bar'), ']')
         assert inst % dict(color='red', stuff='junk') == '[red]'
 
+    def test_format(self):
+        inst = self.klass('{0}', text.Gettext('{color}', 'foo', 'bar'), ']')
+        posargs = ('[', '(')
+        knownargs = {'color': 'red', 'stuff': 'junk'}
+        assert inst.format(*posargs, **knownargs) == '[red]'
+
     def test_add(self):
         inst = (text.Gettext('pale ', 'foo', 'bar') +
                 text.Gettext('blue', 'foo', 'bar'))
