@@ -169,6 +169,26 @@ class TestGroup(XMLRPC_test):
                 error=u'may only include letters, numbers, _, -, . and $')):
             command()
 
+    def test_create_with_name_starting_with_numeric(self):
+        """Successfully create a group with name starting with numeric chars"""
+        testgroup = GroupTracker(
+            name=u'1234group',
+            description=u'Group name starting with numeric chars',
+        )
+        testgroup.create()
+        testgroup.delete()
+
+    def test_create_with_numeric_only_group_name(self):
+        """Try to create a group with name only contains numeric chars"""
+        testgroup = GroupTracker(
+            name=u'1234', description=u'Numeric only group name',
+        )
+        with raises_exact(errors.ValidationError(
+            name='group_name',
+            error=u'may only include letters, numbers, _, -, . and $',
+        )):
+            testgroup.create()
+
 
 @pytest.mark.tier1
 class TestFindGroup(XMLRPC_test):
@@ -231,7 +251,7 @@ class TestFindGroup(XMLRPC_test):
     def test_search_for_all_groups(self, group, group2):
         """ Search for all groups """
         group.ensure_exists()
-        group2.create()
+        group2.ensure_exists()
         command = group.make_command('group_find')
         result = command()
         assert_deepequal(dict(
@@ -631,7 +651,7 @@ class TestManagedGroupObjectclasses(XMLRPC_test):
     def test_check_objectclasses_after_detach(self, user, managed_group):
         """ Check objectclasses after user was detached from managed group """
         # https://fedorahosted.org/freeipa/ticket/4909#comment:1
-        user.create()
+        user.ensure_exists()
         user.run_command('group_detach', *[user.uid])
         managed_group.retrieve(all=True)
         managed_group.add_member(dict(user=user.uid))
