@@ -17,7 +17,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
-Test the `ipalib/plugins/selinuxusermap.py` module.
+Test the `ipaserver/plugins/selinuxusermap.py` module.
 """
 
 from ipalib import api, errors
@@ -240,6 +240,7 @@ class test_selinuxusermap(Declarative):
                     member=dict(
                         group=tuple(),
                         user=tuple(),
+                        service=tuple(),
                     ),
                 ),
                 result={
@@ -272,6 +273,7 @@ class test_selinuxusermap(Declarative):
                     description=[u'Test host 1'],
                     l=[u'Undisclosed location 1'],
                     krbprincipalname=[u'host/%s@%s' % (host1, api.env.realm)],
+                    krbcanonicalname=[u'host/%s@%s' % (host1, api.env.realm)],
                     objectclass=objectclasses.host,
                     ipauniqueid=[fuzzy_uuid],
                     managedby_host=[host1],
@@ -619,9 +621,11 @@ class test_selinuxusermap(Declarative):
             command=(
                 'selinuxusermap_add', [rule1], dict(ipaselinuxuser=u'bad+user')
             ),
-            expected=errors.ValidationError(name='selinuxuser',
-                error=u'Invalid SELinux user name, only a-Z and _ are allowed'
-                ),
+            expected=errors.ValidationError(
+                name='selinuxuser',
+                error=u'Invalid SELinux user name, only a-Z, _ '
+                      'and . are allowed'
+            ),
         ),
 
 
