@@ -13,12 +13,13 @@ from ipatests.test_xmlrpc.xmlrpc_test import (
     fuzzy_digits, fuzzy_uuid, raises_exact)
 from ipatests.test_xmlrpc.tracker.base import Tracker
 from ipatests.test_xmlrpc.tracker.kerberos_aliases import KerberosAliasMixin
+from ipatests.test_xmlrpc.tracker.certmapdata import CertmapdataMixin
 
 if six.PY3:
     unicode = str
 
 
-class UserTracker(KerberosAliasMixin, Tracker):
+class UserTracker(CertmapdataMixin, KerberosAliasMixin, Tracker):
     """ Class for host plugin like tests """
 
     retrieve_keys = {
@@ -62,22 +63,42 @@ class UserTracker(KerberosAliasMixin, Tracker):
 
     primary_keys = {u'uid', u'dn'}
 
-    def __init__(self, name, givenname, sn, **kwargs):
+    def __init__(self, name=None, givenname=None, sn=None, **kwargs):
+        """ Check for non-empty unicode string for the required attributes
+        in the init method """
+
+        if not (isinstance(givenname, six.string_types) and givenname):
+            raise ValueError(
+                "Invalid first name provided: {!r}".format(givenname)
+                )
+        if not (isinstance(sn, six.string_types) and sn):
+            raise ValueError("Invalid second name provided: {!r}".format(sn))
+
         super(UserTracker, self).__init__(default_version=None)
-        self.uid = name
-        self.givenname = givenname
-        self.sn = sn
+        self.uid = unicode(name)
+        self.givenname = unicode(givenname)
+        self.sn = unicode(sn)
         self.dn = DN(('uid', self.uid), api.env.container_user, api.env.basedn)
 
         self.kwargs = kwargs
 
     def make_create_command(self, force=None):
-        """ Make function that crates a user using user-add """
-        return self.make_command(
-            'user_add', self.uid,
-            givenname=self.givenname,
-            sn=self.sn, **self.kwargs
-            )
+
+        """ Make function that creates a user using user-add
+            with all set of attributes and with minimal values,
+            where uid is not specified """
+
+        if self.uid is not None:
+            return self.make_command(
+                'user_add', self.uid,
+                givenname=self.givenname,
+                sn=self.sn, **self.kwargs
+                )
+        else:
+            return self.make_command(
+                'user_add', givenname=self.givenname,
+                sn=self.sn, **self.kwargs
+                )
 
     def make_delete_command(self, no_preserve=True, preserve=False):
         """ Make function that deletes a user using user-del
@@ -175,14 +196,14 @@ class UserTracker(KerberosAliasMixin, Tracker):
                         (self.kwargs[key].split('@'))[0].lower(),
                         (self.kwargs[key].split('@'))[1]
                     )]
-                except IndexError as ex:
+                except IndexError:
                     # we can provide just principal part
                     self.attrs[key] = [u'%s@%s' % (
                         (self.kwargs[key].lower(),
                          self.api.env.realm)
                     )]
             else:
-                if not type(self.kwargs[key]) is list:
+                if type(self.kwargs[key]) is not list:
                     self.attrs[key] = [self.kwargs[key]]
                 else:
                     self.attrs[key] = self.kwargs[key]
@@ -493,7 +514,7 @@ class UserTracker(KerberosAliasMixin, Tracker):
         assert_deepequal(dict(
             completed=1,
             failed=dict(
-                member=dict(group=tuple(), user=tuple())
+                member=dict(group=tuple(), user=tuple(), service=tuple())
             ),
             result={
                 'dn': get_group_dn(admin_group),
@@ -510,3 +531,10 @@ class UserTracker(KerberosAliasMixin, Tracker):
 
     def _make_remove_alias_cmd(self):
         return self.make_command('user_remove_principal', self.name)
+
+    # Certificate identity mapping methods
+    def _make_add_certmap(self):
+        return self.make_command('user_add_certmapdata', self.name)
+
+    def _make_remove_certmap(self):
+        return self.make_command('user_remove_certmapdata', self.name)
